/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C)      2011  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.antlr.runtime.RecognitionException;
import org.ow2.dsrg.fm.badger.BadgerLog;
import org.ow2.dsrg.fm.badger.ca.karpmiller.ConfigurationNode;
import org.ow2.dsrg.fm.badger.ca.karpmiller.Context;
import org.ow2.dsrg.fm.badger.ca.karpmiller.TreeConstruction;
import org.ow2.dsrg.fm.badger.ca.karpmiller.opt.TreeConstructionEarlierEnd;
import org.ow2.dsrg.fm.badger.ca.karpmiller.util.KarpMillerTree;
import org.ow2.dsrg.fm.badger.ca.karpmiller.util.KarpMillerTreeWriter;
import org.ow2.dsrg.fm.badger.ca.statespace.ContextAdapter;
import org.ow2.dsrg.fm.badger.ca.statespace.GlobalState;
import org.ow2.dsrg.fm.badger.ca.statespace.LocationReference;
import org.ow2.dsrg.fm.badger.ca.statespace.StateSpaceExplorer;
import org.ow2.dsrg.fm.badger.ca.statespace.encoding.impl.PlainLocalStateEncoderFactoryImpl;
import org.ow2.dsrg.fm.badger.ca.statespace.impl.GlobalStateImpl;
import org.ow2.dsrg.fm.badger.ca.statespace.impl.StateSpaceExplorerImpl;
import org.ow2.dsrg.fm.badger.ca.stats.StatsCollector;
import org.ow2.dsrg.fm.badger.ca.stats.ValueHandler;
import org.ow2.dsrg.fm.badger.ca.stats.util.AppearanceValueHandler;
import org.ow2.dsrg.fm.badger.ca.stats.util.CounterValueHandler;
import org.ow2.dsrg.fm.badger.ca.stats.util.IntervalValueHandler;
import org.ow2.dsrg.fm.badger.ca.stats.util.ValueDictionary;
import org.ow2.dsrg.fm.badger.ca.util.CAbsStructuresBuilder;
import org.ow2.dsrg.fm.badger.ca.util.LocalStateFactory;
import org.ow2.dsrg.fm.badger.translation.LTSEncodingVisitor;
import org.ow2.dsrg.fm.tbplib.architecture.Architecture;
import org.ow2.dsrg.fm.tbplib.architecture.Interface;
import org.ow2.dsrg.fm.tbplib.event.EventTable;
import org.ow2.dsrg.fm.tbplib.event.EventTableImpl;
import org.ow2.dsrg.fm.tbplib.ltsa.Automaton;
import org.ow2.dsrg.fm.tbplib.ltsa.AutomatonToDot;
import org.ow2.dsrg.fm.tbplib.ltsa.DFA;
import org.ow2.dsrg.fm.tbplib.ltsa.LTS;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSA;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSAInlineHepler;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSAMethod;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSASpecification;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSAThread;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSSpecification;
import org.ow2.dsrg.fm.tbplib.ltsa.Observer;
import org.ow2.dsrg.fm.tbplib.ltsa.visitor.FindMethodCallsVisitor;
import org.ow2.dsrg.fm.tbplib.node.TBPAssignment;
import org.ow2.dsrg.fm.tbplib.node.TBPAtomic;
import org.ow2.dsrg.fm.tbplib.node.TBPEmit;
import org.ow2.dsrg.fm.tbplib.node.TBPImperativeSequence;
import org.ow2.dsrg.fm.tbplib.node.TBPMethodDeclaration;
import org.ow2.dsrg.fm.tbplib.node.TBPNode;
import org.ow2.dsrg.fm.tbplib.node.TBPProvisionContainerNode;
import org.ow2.dsrg.fm.tbplib.node.TBPSpecification;
import org.ow2.dsrg.fm.tbplib.node.TBPValue;
import org.ow2.dsrg.fm.tbplib.node.TBPVariableDefinition;
import org.ow2.dsrg.fm.tbplib.provision.ObservingProvisionAutomaton;
import org.ow2.dsrg.fm.tbplib.provision.ProvisionAutomaton;
import org.ow2.dsrg.fm.tbplib.provision.Transition;
import org.ow2.dsrg.fm.tbplib.provision.Transition.OutputSymbolDirection;
import org.ow2.dsrg.fm.tbplib.provision.actuator.visitor.VariableFactory;
import org.ow2.dsrg.fm.tbplib.provision.actuator.visitor.VariableFactoryImpl;
import org.ow2.dsrg.fm.tbplib.reference.Constant;
import org.ow2.dsrg.fm.tbplib.reference.EnumerationType;
import org.ow2.dsrg.fm.tbplib.reference.Reference;
import org.ow2.dsrg.fm.tbplib.reference.Variable;
import org.ow2.dsrg.fm.tbplib.util.TransformationChain;

/**
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class Check {
	private static final String PATHS_COUNTER_NAME = "paths";
	private static final String DEPTH_COUNTER_NAME = "max-depth";
	private static final String DEPTH_HISTO_NAME = "depth";
	private static final String TIME_INTERVAL_NAME = "time";	

	public static void main(String[] args) {
		try {
			Parameters p = Parameters.parse(args);
			if (p != null) {
				StatsCollector s;
				if (p.getPrintStats()) {
					s = createStatsCollector();
					registerStatsWriterHook(p.getStatsFileName(), s);
				} else {
					s = new ValueDictionary();
				}

				perform(p, s);
			}
		} catch (Exception e) {
			System.err.printf("Uncaught exception %s: %s", e.getClass().toString(), e.getMessage());
			e.printStackTrace(System.err);
		}
	}
	/** 
	 * 
	 * @param params @link {@link Parameters}
	 * @throws RecognitionException on error during parsing input files
	 * @throws IOException on input/output error
	 */
	public static void perform(final Parameters params, final StatsCollector stats)
	throws IOException, RecognitionException {
		
		
	    BadgerLog.logger.info("Parsing files: " + Arrays.toString(params.getFileNames()));
	
	    EventTable table = new EventTableImpl();
	    Architecture arch = new Architecture("testing-arch");
	    List<TBPSpecification<String>> tbps =
	            TransformationChain.parseFiles(arch, params.getFileNames());
	
	    TransformationChain.collectSymbolsFromArchitecture(tbps, arch);
	    
	    BadgerLog.logger.info("Implicit binding");
	    TransformationChain.performImplicitBinding(arch);
	    
	    BadgerLog.logger.info("Resolving symbols");
	    List<TBPSpecification<Reference>> ref_tbps =
	            TransformationChain.resolveArchitecture(tbps, arch);
	    TransformationChain.fillEventTable(arch, table);
	    
	            
	    // (1) find unbound provided interfaces and their provisions
	    List<Interface> unboundProvidedInterfaces = new LinkedList<Interface>();
	    List<Interface> unboundRequiredInterfaces = new LinkedList<Interface>();
	    TransformationChain.findUnboundInterfaces(arch, unboundProvidedInterfaces, unboundRequiredInterfaces);
	            
	    // (2) find provisions of that interfaces
	    Map<TBPProvisionContainerNode<Reference>, Collection<Interface>> unboundProvisionsMap
	    	= TransformationChain.findProvisionsForInterfaces(ref_tbps, unboundProvidedInterfaces);
	
	    // Intermezzo: create composition of components
	    //             optionally remove provisions that'll be transformed to
	    //             actuators
	    BadgerLog.logger.info("Creating composition of components");
	   	
	    // remove provisions translated to actuators (it would be redundant to check them)
	    ref_tbps = TransformationChain.removeProvisionsFromSpecifications(ref_tbps, unboundProvisionsMap.keySet());
	   	
	    TBPSpecification<Reference> tbpComp =
	            TransformationChain.performComposition(ref_tbps, null);
	    TransformationChain.replaceArbitraryValuesInImperativePartsByAlternatives(tbpComp);
	    
	    
	    // (3) transform provisions to provision actuators        
	    VariableFactory<Reference> variableFactory = new VariableFactoryImpl(tbpComp.getVariables());
	    Map<LTSASpecification<TBPNode<Reference>, Reference>, Collection<Interface>> provisionActuatorsMap
	    	= TransformationChain.transformProvisionsToActuators(unboundProvisionsMap, variableFactory);
	    
	    // TODO:
	    // (4) create reactions to unbound required interfaces
	    //     these reactions should be used within inlining
	                    
	    BadgerLog.logger.info("Transformation to LTS represenation");
	    LTSSpecification<Reference> ltsSpecification =
	    	(new LTSEncodingVisitor<Reference>(EnumerationType.LOCKED, EnumerationType.UNLOCKED)).transformTBP(tbpComp);
	    
	    if (params.getPrintDebugInfo()) {
			BadgerLog.logger.info("Writing LTS to DOT.");
		    AutomatonToDot.printLTSSpecificationToDot(ltsSpecification, getLTSALabelFactory(), new FileOutputStream(params.getDebugInfoFileNamePrefix() + "LTS.dot"));
	    }
	    
	    BadgerLog.logger.info("Transformation to LTSA represenation");        
	    LTSASpecification<TBPNode<Reference>, Reference> ltsaSpecification =
	    	TransformationChain.transformToLTSA(ltsSpecification);
	    
	    // Add provision actuators...
	    for (LTSASpecification<TBPNode<Reference>, Reference> provisionActuator : provisionActuatorsMap.keySet()) {
	    	for (LTSAThread<TBPNode<Reference>, Reference> thread : provisionActuator.getThreads()) {
	    		ltsaSpecification.addThread(thread);
	    	}
	    	for (LTSAMethod<TBPNode<Reference>, Reference> method : provisionActuator.getMethods()) {
	    		ltsaSpecification.addMethod(method);
	    	}
	    }
	    
		final LTSASpecification<TBPNode<Reference>, Reference> spec = ltsaSpecification;
		
		if (params.getPrintDebugInfo()) {
			BadgerLog.logger.info("Writing LTSA to DOT.");
			AutomatonToDot.printLTSASpecificationToDot(ltsaSpecification, getLTSALabelFactory(), new FileOutputStream(params.getDebugInfoFileNamePrefix() + "LTSA.dot"));
		}
		 	
		FindMethodCallsVisitor<Reference> finder = new FindMethodCallsVisitor<Reference>() {
			@Override
			protected LTSAMethod<TBPNode<Reference>, Reference> resolveCalledMethod(
					Reference iface, Reference method,
					List<Reference> parameters) {
				LTSAMethod<TBPNode<Reference>, Reference> theMethod = null;
				for(LTSAMethod<TBPNode<Reference>, Reference> m: spec.getMethods()) {
					TBPNode<Reference> code = m.getCodeReference();
					if (code instanceof TBPMethodDeclaration<?>) {
						TBPMethodDeclaration<Reference> md = (TBPMethodDeclaration<Reference>)code;
						if (md.getMethod().equals(method)) {
							theMethod = m;
							break;
						}
					}
				}
				if (theMethod == null) {
					throw new RuntimeException("No suitable method found!");
				}
				return theMethod;
			}
			}; 
	            
	    BadgerLog.logger.info("In-lining the LTSA represenation");        
	    LTSASpecification<TBPNode<Reference>, Reference>
	    	inlinedLtsaSpecification = LTSAInlineHepler.inline(ltsaSpecification, finder, variableFactory);
	
	    if (params.getPrintDebugInfo()) {
	    	BadgerLog.logger.info("Writing in-lined LTSA to DOT.");
	    	AutomatonToDot.printLTSASpecificationToDot(inlinedLtsaSpecification, getLTSALabelFactory(), new FileOutputStream(params.getDebugInfoFileNamePrefix() + "LTSA_inlined.dot"));
	    }
	    
	    
	    BadgerLog.logger.info("Creating CAbsStructuresBuilder.");
	    CAbsStructuresBuilder caBuilder = new CAbsStructuresBuilder(new PlainLocalStateEncoderFactoryImpl<Variable, Byte>());
	    
	    BadgerLog.logger.info("Transforming provisions.");
	    Map<String, DFA> observers = null;
	    Map<String, DFA> observationAutomata = null;
	    for(TBPProvisionContainerNode<Reference> prov: tbpComp.getProvisions()) {
	    	boolean useFA = false;
	    	if (params.getForceFiniteAutomata() && ! params.getForceCounterAutomata()) {
	    		useFA = true;
	    	} else if (params.getForceFiniteAutomata() == params.getForceCounterAutomata()) {
	    		// TODO: use best fit
	    		//       if the provision contain unlimited re-entrancy, use counter automata (makeObserver = false)
	    		//       use finite automata otherwise (makeObserver == true)
	        	throw new UnsupportedOperationException("Combination of Observers and ObservingAutomata not supported yet.");
	    	}
	    	if(useFA) { // observer ~ plain finite automaton
	    		Observer o = 
	    			caBuilder.transformProvision(prov, table);
	    		if (params.getPrintDebugInfo()) {
	    			observers = addToMap(observers, prov.getName(), o);
	    		}
	    	} else { // observing provision automata ~ counter automaton
	    		final ObservingProvisionAutomaton<LocationReference> opa =
	    			caBuilder.transformProvisionWithUnlimitedParallel(prov, table);
	    		if (params.getPrintDebugInfo()) {
	    			observationAutomata = addToMap(observationAutomata, prov.getName(), opa);
	    		}
	    	}
	    }
	    
	    if (	(observers != null && !observers.isEmpty())
	    		|| (observationAutomata != null && !observationAutomata.isEmpty())) {
	    	BadgerLog.logger.info("Writing provisions to dot.");
	    	
	    	AutomatonToDot.LabelFactory<Reference> tmpLabelFactory = getProvisionAutomataLabelFactory(table);
	    	
	    	if (observers != null && !observers.isEmpty()) {
	    		AutomatonToDot.printDFAsToDot(observers, 
	    				tmpLabelFactory,
	    				new FileOutputStream(String.format("%sDFAs.dot", params.getDebugInfoFileNamePrefix())));
	    	}
	    	if (observationAutomata != null && !observationAutomata.isEmpty()) {
	    		AutomatonToDot.printDFAsToDot(observationAutomata,
	    				tmpLabelFactory,
	    				new FileOutputStream(String.format("%sPAs.dot", params.getDebugInfoFileNamePrefix())));
	    	}
	    }
	    
	    BadgerLog.logger.info("Transforming LTSAThreads (setting program counter values).");
	    LTSASpecification<TBPNode<Reference>, Reference> transformedSpecification = new LTSASpecification<TBPNode<Reference>, Reference>();
		for(LTSAThread<TBPNode<Reference>, Reference> t : inlinedLtsaSpecification.getThreads()) {
			transformedSpecification.addThread(caBuilder.transformThread(t, table, variableFactory));
		}
		
		if (params.getPrintDebugInfo()) {
			BadgerLog.logger.info("Writing transformed LTSA threads to dot.");
			AutomatonToDot.printLTSASpecificationToDot(transformedSpecification, getLTSALabelFactory(), new FileOutputStream(params.getDebugInfoFileNamePrefix() + "LTSA_withPC.dot"));
			
		}
		
		if (params.getPrintDebugInfo()) {
			String fileName = params.getDebugInfoFileNamePrefix() + "LTSARepository.txt";
			BadgerLog.logger.fine(String.format("Writing LTSA repository content to file %s.", fileName));
			FileWriter fw = new FileWriter(fileName);
			fw.write(caBuilder.getLTSARepository().toString());
			fw.close();
		}
		
		StateSpaceExplorer<Variable, Byte> sse = new StateSpaceExplorerImpl(caBuilder.getLTSARepository(), table);
		    	
	    // run karp-miller
	    
	    // 1) create initial configuration
		// Initial configuration contains as many replicas of each thread as is the thread replication factor
		// of the thread. In the case of threads with unbounded replication factor - they won't be
		// present in the initial configuration but they will be created as the verification procedure explores
		// possible code paths.
		//
		{
	        BadgerLog.logger.info("Creating initial configuration for Karp-Miller Tree construction.");
	        
			GlobalState<Variable, Byte> g;
			
	        // create global state
	        Map<Variable, Byte> valuation = new HashMap<Variable, Byte>();
	        for (TBPVariableDefinition<Reference> var : variableFactory.getStateVariables()) {
	        	Variable v = (Variable)var.getName();
	        	Constant c;
	        	if (v.getType() == EnumerationType.MUTEX_TYPE) {
	        		c = EnumerationType.UNLOCKED;
	        	} else {
		        	c = (Constant)var.getInitialValue();	        		
	        	}
	        	valuation.put(v, c.getCode());
	        }
	        g = new GlobalStateImpl<Variable, Byte>()
	        	.modify(valuation);
	        
	        
	        Context<Variable, Byte> ctx = new ContextAdapter<Variable, Byte>( caBuilder.getLTSARepository(), sse, g, 
	        		LocalStateFactory.createLocals(transformedSpecification, caBuilder.getLTSARepository(), sse));
	        	        
	        Collection<ConfigurationNode<Variable, Byte>> allNodes = null;
	        
	        if (params.getPrintKMTree() || params.getPrintKMTreeFragments()) {
	        	if (params.getPrintKMTreeFragments()) {
	        		BadgerLog.logger.fine("Preparing Karp-Miller tree container with logging capability.");
	    	        allNodes = new LinkedList<ConfigurationNode<Variable,Byte>>() {
	    				private static final long serialVersionUID = 1309997014711397031L;
	    				private int bufferSize = params.getKMTreeFragmentWidth();
	    	        	private int batchNo = 0;
	    	        	private int added = 0;
	    	        	private int maxDepth = 0;
	    	        	private boolean keepAllNodes = params.getPrintKMTree();
	    	        	private ArrayList<ConfigurationNode<Variable, Byte>> buffer = params.getKMTreeFragmentWidth() > 0 ? new ArrayList<ConfigurationNode<Variable, Byte>>(bufferSize) : null;
	    	        	// stats:
	    	        	private ValueHandler maxDCtr = stats.getValue(DEPTH_COUNTER_NAME);
	    	        	private ValueHandler depthHisto = stats.getValue(DEPTH_HISTO_NAME);
	    	        	private ValueHandler paths = stats.getValue(PATHS_COUNTER_NAME);
	    	        	
	    	        	@Override
	    	        	public boolean add(ConfigurationNode<Variable, Byte> n) {
	    	        		// call super collection
		        			boolean result = true;
	    	        		if (keepAllNodes) {
		    	        		result = super.add(n);	    	        			
	    	        		}
	    	        		// adjust stats
	    	        		int depth = n.depth();
	    	        		if (depth > maxDepth) {
	    	        			maxDepth = n.depth();
	    	        		}
	    	        		++added;	    	        		
	    	        		this.maxDCtr.adjustValue(maxDepth);
	    	        		this.depthHisto.adjustValue(depth);
	    	        		this.paths.adjustValue(added);
	    	        		
	    	        		if (buffer != null) {
		    	        		if (buffer.size() >= bufferSize) {
		    	        			BadgerLog.logger.info(String.format("Flushing portion of tree consisting of %d paths.", buffer.size()));
		    	    	        	int ix = 0;
	    		    	        	for(KarpMillerTree<Variable, Byte> tree: KarpMillerTree.build(buffer)) {
	    		    	        		String fileName = String.format("%s%cTree-Fragment-%05d-%05d.dot", params.getKMTreeFragmentsDir(), File.separatorChar, ++batchNo, ++ix);
	    		    	        		try {
		    		    	        		OutputStream o = openFile(fileName);
		    		    	        		KarpMillerTreeWriter.writeAsDot(o, tree);
		    		    	        		o.close();
		    		    	        		BadgerLog.logger.fine(String.format("Part of Karp-Miler tree printed to file %s.", fileName));
	    		    	        		} catch (Exception e) {
	    		    	        			BadgerLog.logger.warning(String.format("Unable to write a part of tree to file %s. because of exception %s: %s", fileName, e.getClass().getName(), e.getMessage()));	    		    	        			
	    		    	        		}
	    		    	        	}
		    	    	        	buffer.clear();
		    	        		} else {
		    	        			buffer.add(n);
		    	        		}
	    	        		}
	    	        		return result;
	    	        	}
	    	        };
	        	} else {
	        		BadgerLog.logger.fine("Preparing Karp-Miller tree container without logging capability.");	        		
	        		allNodes = new LinkedList<ConfigurationNode<Variable,Byte>>();
	        	}
	        } else {
	        	BadgerLog.logger.fine("Karp-Miller tree won't be retained at all.");
	        }
	        
	        BadgerLog.logger.info("Starting Karp-Miller Tree construction.");
	        TreeConstruction<Variable, Byte> KMTCA = new TreeConstructionEarlierEnd<Variable, Byte>(
	    			new TreeConstruction.Settings(	TreeConstruction.SearchStrategy.DepthFirst,
							params.getContinueAfterVioloation() ? TreeConstruction.StopStrategy.DoNotStop : TreeConstruction.StopStrategy.StopOnError,
							TreeConstruction.Settings.DEPTH_UNLIMITED),
							allNodes);
	        
	        ValueHandler timeInterval = stats.getValue(TIME_INTERVAL_NAME); 
	        timeInterval.adjustValue(new Date().getTime());
	        Collection<ConfigurationNode<Variable, Byte>> errorTraces = KMTCA.constructTree(ctx);
	        timeInterval.adjustValue(new Date().getTime());
	        
	        // TODO: write out error trace(s)
	        
	        BadgerLog.logger.info(String.format("Karp-Miller Tree construction finished with %d error trace nodes.", errorTraces.size()));
	        
	       	int ix = 0;
	    	if (params.getPrintKMTreeFragments()) {
	        	for(ConfigurationNode<Variable, Byte> n: errorTraces) {
	        		for(KarpMillerTree<Variable, Byte> tree: KarpMillerTree.build(Collections.singleton(n))) {	        			
	        			KarpMillerTreeWriter.writeAsDot(openFile(String.format("%s%cError-Trace-Fragment-%d.dot", params.getKMTreeFragmentsDir(), File.separatorChar, ++ix)), tree);
	        		}
	        	}
	        	ix = 0;
	    	}
	    	if (params.getPrintDebugInfo()) {
	    		for(KarpMillerTree<Variable, Byte> tree: KarpMillerTree.build(errorTraces)) {
       				KarpMillerTreeWriter.writeAsDot(openFile(String.format("%sError-Trace-Tree-%d.dot", params.getDebugInfoFileNamePrefix(), ++ix)), tree);
	    		}
		        BadgerLog.logger.info(String.format("%d Error trace(s) written to DOT.", ix));
		        ix = 0;
	    	}        
	        if (params.getPrintKMTree()) {
		        BadgerLog.logger.info(String.format("Karp-Miller Tree construction finished - printing tree of %d distinct paths.", allNodes.size()));
	        	for(KarpMillerTree<Variable, Byte> tree: KarpMillerTree.build(allNodes)) {
        			KarpMillerTreeWriter.writeAsDot(openFile(String.format("%sKMTree-%d.dot", params.getDebugInfoFileNamePrefix(), ++ix)), tree);
	        	}
		        BadgerLog.logger.info(String.format("%d Karp-Miller Tree(s) written to DOT.", ix));
		        ix = 0;
	        }        
		}
	}
	
	private static OutputStream openFile(String fname) {
		OutputStream o = null;
		try {
			o = new FileOutputStream(fname);
		} catch(FileNotFoundException fne) {
			BadgerLog.logger.warning(String.format("File %s not found.", fname));
		} catch (Exception e) {
			BadgerLog.logger.warning(String.format("Unable to open file %s. Exception %s: %s.", fname, e.getClass().getName(), e.getMessage()));
		} finally {
			if (o == null) {
				o = new OutputStream() {
					@Override
					public void write(int b) throws IOException {
					}
				};
			}
		}
		return o;
	}
	
	private static Map<String, DFA> addToMap(Map<String, DFA> map, String name, DFA automaton) {
		Map<String, DFA> currentMap = map != null ? map : new HashMap<String, DFA>();
		String nameBase = name == null || name.isEmpty() ? "UnnamedProvision" : name;
		int offset = 0;
		String tmpName = String.format("%s (%s@%h)", nameBase, automaton != null ? automaton.getClass().getSimpleName() : "null", automaton);
		while(currentMap.containsKey(tmpName)) {
			tmpName = String.format("%s%d (%s@%h)", nameBase, offset, automaton != null ? automaton.getClass().getSimpleName() : "null", automaton);
		}
		currentMap.put(tmpName, automaton);
		return currentMap;
	}
	
	private static void registerStatsWriterHook(final String fileName, final StatsCollector s) {
		Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
			private String outputFileName = fileName;
			private StatsCollector stats = s;
			@Override
			public void run() {
				try {
					FileWriter fw = new FileWriter(fileName);
					fw.write(stats.toString());
					fw.close();
				} catch (IOException ioe) {
					throw new RuntimeException(String.format("Stats-Hook: unable to flush stats to file %s.", outputFileName), ioe);
				}
			}
		}));
	}
	
	private static StatsCollector createStatsCollector() {
		StatsCollector c = new ValueDictionary();
		c.registerValue(new CounterValueHandler(DEPTH_COUNTER_NAME));
		c.registerValue(new CounterValueHandler(PATHS_COUNTER_NAME));
		c.registerValue(new IntervalValueHandler(TIME_INTERVAL_NAME));
		c.registerValue(new AppearanceValueHandler(DEPTH_HISTO_NAME, 0, 500));
		return c;
	}
		
	private static AutomatonToDot.LabelFactory<Reference> getProvisionAutomataLabelFactory(final EventTable eventTable) {
		return new AutomatonToDot.LabelFactory<Reference>() {
				
			private EventTable tab = eventTable;
			
			@Override
			public String createNodeLabel(Automaton a, int state) {
				String label = String.format("%d", state);
				if (a instanceof ProvisionAutomaton<?>) {
					Object sym = ((ProvisionAutomaton<?>)a).getStateSymbol(state);
					label += ":";
					label += sym != null ? sym.toString() : "";
				}
				return label;
			}
			@Override
			public String createEdgeLabel(Automaton a, int sourceState, long event, int targetState) {
				String result = String.format("%d:%s", event, tab.decodeEventName(event));
				if (a instanceof ProvisionAutomaton<?>) {            				
					Transition<?> transition = ((ProvisionAutomaton<?>)a).getTransition(sourceState, event);
					if (transition != null && transition.getOutputSymbols() != null) {
						for(Transition.OutputSymbol<?> sym : transition.getOutputSymbols()) {
							if (sym != null && sym.getSymbol() != null) {
								result += " [";
								result += (sym.getDirection() == OutputSymbolDirection.Up) ? "++(" : "--(";
								result += sym.getSymbol().toString();
								result += ")]";
							}
						}
					}
				};
				return result;
			}
			@Override
			public String createTransitionLabel(
					LTS<? extends TBPNode<Reference>, Reference> transition) {
				return "";
			}
			@Override
			public String referenceToString(Reference reference) {
				return reference.toString();
			}
			@Override
			public String createTransitionLabel(
					LTSA<?, Reference> transition) {
				return "";
			}
		};
	}
	
	private static AutomatonToDot.LabelFactory<Reference> getLTSALabelFactory() {
		return new AutomatonToDot.LTSALabelFactoryBase<Reference>() {
			@Override
			public String referenceToString(Reference reference) {
				return reference != null ? reference.getName() : "";
			}
			@Override
			protected String ltsLabelToString(TBPNode<Reference> label) {
				String result = null;
				if (label != null) {
					if (label instanceof TBPEmit<?>) {
						TBPEmit<Reference> emit = (TBPEmit<Reference>)label; 
						result = emit.getMethodCall().getFullname();
					}
				}
				return result;
			}
			@Override
			protected String ltsAssignmentToString(
					TBPNode<Reference> label) {
				String result = null;
				if (label != null && label instanceof TBPAssignment<?>) {
					TBPAssignment<Reference> a = (TBPAssignment<Reference>)label;
					String value = "<unknown>";
					if (a.getChild() != null && a.getChild() instanceof TBPValue<?>) {
						TBPValue<Reference> v = (TBPValue<Reference>)a.getChild();
						if (v.isMethodCall()) {
							value = v.getMethodCall().getFullname(); 
						} else {
							value = referenceToString(v.getVarname());
						}
					}
					result = referenceToString(a.getIdf()) + " := " + value;
				} else if (label != null && label instanceof TBPAtomic<?> && label.getChild(0) != null && label.getChild(0) instanceof TBPImperativeSequence<?>) {
					result = "";
					TBPImperativeSequence<Reference> atomic = (TBPImperativeSequence<Reference>)(label.getChild(0));
					
					for(TBPNode<Reference> n : atomic.getChildren()) {
						if (!result.isEmpty()) {
							result += "; ";
						}
						String nAssignment = ltsAssignmentToString(n);
						if (nAssignment == null || nAssignment.isEmpty()) {
							result += "* := *";
						} else {
							result += nAssignment;
						}
					}
					if (result.isEmpty()) {
						result = null;
					}
				}
				return result;
			}
	 	};
	}
}
