/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2011       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.karpmiller;

/** Represents node of the Karp-Miller Tree.
 * 
 * <p>Consists of {@link Configuration} object - the actual content of node
 *    and link to {@link ConfigurationNode predecessor}.</p>
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface ConfigurationNode<NAME, VAL> {
		
	/** Gives content of this.
	 * 
	 * @return {@link Configuration}
	 */
	public Configuration<NAME, VAL> getConfiguration();
	
	/** Gives predecessor of this.
	 * 
	 * @return {@link ConfigurationNode}
	 */
	public ConfigurationNode<NAME, VAL> getPredecessor();

	/** Determines whether any predecessor (recursively) is equal to given {@link Configuration} <b>c</b>.
	 * 
	 * @param c {@link Configuration}
	 * @return <code>true</code> if and only if <b>c</b> was found in predecessors (recursively, including this)
	 */
	public boolean containsPredecessor(Configuration<NAME, VAL> c);
	
	/** Allows iteration over all predecessors till the root (without this). 
	 * 
	 * @return {@link Iterable} of all predecessors
	 */
	public Iterable<Configuration<NAME, VAL>> getAllPredecessors();
	
	/** Gives depth of this node which is number of its predecessors.
	 * 
	 * @return depth
	 */
	public int depth();
	
	public void setPreviousConfiguration(Configuration<NAME, VAL> c);
	
	public Configuration<NAME, VAL> getPreviousConfiguration();
	
}
