/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2011       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.karpmiller;

import java.util.Collection;

import org.ow2.dsrg.fm.badger.ca.statespace.GlobalState;
import org.ow2.dsrg.fm.badger.ca.statespace.LocalState;

/** Represents entry point to the Karp-Miller Tree construction algorithm.
 * 
 * <p>Implementation of this interface gives initial {@link Configuration}
 *    to the algorithm and provides <i>context-aware state-space exploration
 *    step</i> as well.
 * </p>
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface Context<NAME, VAL> {
	
	/** Returns initial {@link Configuration}
	 *  (start point of Karp-Miller Tree construction algorithm).
	 * 
	 * @return {@link Configuration}
	 */
	public Configuration<NAME, VAL> getInitialConfiguration();

	/** Executes program or model from shared state <b>s</b>, local state <b>l</b>
	 *  and returns all possible {@link Transition}s (including those
	 *  representing <i>creation of new threads</i>).
	 * 
	 * @param s {@link GlobalState}
	 * @param l {@link LocalState}
	 * @return {@link Transition}
	 */
	public Collection<Transition<NAME, VAL>> execute(GlobalState<NAME, VAL> s, LocalState<NAME, VAL> l);
	
	/** Evaluates given configuration <b>c</b> and returns <code>true</code> <i>iff</i> it represents error state.
	 * 
	 * @param c {@link Configuration}
	 * @return boolean
	 */
	public boolean hitsError(Configuration<NAME, VAL> c);

}
