/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2011       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.karpmiller;

import java.util.Collection;

import org.ow2.dsrg.fm.badger.ca.statespace.GlobalState;
import org.ow2.dsrg.fm.badger.ca.statespace.LocalState;

/** Represents transition from one <i>configuration</i> to another.
 * 
 * <p>
 * A transition consist of 
 * <ul><li>{@link GlobalState global state} (as the result of transition)</li>
 *     <li><i>abandoned</i> {@link LocalState local states}</li>
 *     <li><i>taken</i> {@link LocalState local states}</li>
 * </ul>
 * </p>
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface Transition<NAME, VAL> {
	/** Returns abandoned local states.
	 * 
	 * @return {@link Collection} of abandoned {@link LocalState}s
	 */
	public Collection<LocalState<NAME, VAL>> getAbandonedLocalStates();
	/** Returns taken global state.
	 * 
	 * @return taken {@link GlobalState}
	 */
	public GlobalState<NAME, VAL> getGlobalState();
	/** Returns taken local states
	 * 
	 * @return {@link Collection} of taken {@link LocalState}s
	 */
	public Collection<LocalState<NAME, VAL>> getTakenLocalStates();	
}
