/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2011       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.karpmiller.impl;

import org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration;

/** Represents node of the Karp-Miller Tree.
 * 
 * <p>Consists of {@link Configuration} object - the actual content of node
 *    and link to {@link ConfigurationNode predecessor}.</p>
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class ConfigurationNode<NAME, VAL> implements
	org.ow2.dsrg.fm.badger.ca.karpmiller.ConfigurationNode<NAME, VAL> {
	
	
	private org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration<NAME, VAL> configuration;
	private org.ow2.dsrg.fm.badger.ca.karpmiller.ConfigurationNode<NAME, VAL> predecessor;
	private Iterable<org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration<NAME, VAL>> predecessorsIterable;
	private int depth;
	private org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration<NAME, VAL> previouslyExploredConfiguration;
	
	/** Creates new node.
	 * 
	 * @param configuration {@link Configuration}
	 * @param predecessor {@link ConfigurationNode} (link to predecessor)
	 */
	public ConfigurationNode(
			org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration<NAME, VAL> configuration,
			org.ow2.dsrg.fm.badger.ca.karpmiller.ConfigurationNode<NAME, VAL> predecessor) {
		this.configuration = configuration;
		this.predecessor = predecessor;
		this.predecessorsIterable = new ConfigurationsIterable<NAME, VAL>(this.predecessor);
		assert(this.configuration != null);
		this.depth = this.predecessor == null ? 0 : this.predecessor.depth() + 1;
		this.previouslyExploredConfiguration = null;
	}
	
	/** Creates new node with null predecessor.
	 * 
	 * @param configuration {@link Configuration}
	 */
	public ConfigurationNode(
			org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration<NAME, VAL> configuration) {
		this(configuration, null);
	}

	@Override
	public org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration<NAME, VAL> getConfiguration() {
		return configuration;
	}
	
	@Override
	public org.ow2.dsrg.fm.badger.ca.karpmiller.ConfigurationNode<NAME, VAL> getPredecessor() {
		return predecessor;
	}
	
	@Override
	public boolean containsPredecessor(org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration<NAME, VAL> c) {
		if (c != null) {
			for(org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration<NAME, VAL> p: this.predecessorsIterable) {
				if (c.equals(p)) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public Iterable<org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration<NAME, VAL>> getAllPredecessors() {
		return this.predecessorsIterable;
	}

	@Override
	public int depth() {
		return this.depth;
	}

	@Override
	public void setPreviousConfiguration(Configuration<NAME, VAL> c) {
		this.previouslyExploredConfiguration = c;
		
	}

	@Override
	public Configuration<NAME, VAL> getPreviousConfiguration() {
		return this.previouslyExploredConfiguration;
	}

}

