/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.karpmiller.opt;

/** Wraps partial order &rarr;.
 * 
 * <p>Crucial features:
 *    <ul><li>&rarr; is transitive (a &rarr; b &amp; b &rarr; c then a &rarr; c)</li>
 *        <li>&rarr; is antisymetric (a &rarr; b &amp; b &rarr; a then a = b)</li>
 *    </li>
 * </p> 
 *
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface PartialOrder<E> {
	/** Determines whether <b>a</b> &rarr; <b>b</b>.
	 * 
	 * @param a
	 * @param b
	 * @return <code>true</code> if and only if <b>a</b> &rarr; </b>b</b>
	 */
	public boolean inOrder(E a, E b);
}
