/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.karpmiller.opt;

import java.util.HashMap;
import java.util.Map;

import org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration;
import org.ow2.dsrg.fm.badger.ca.statespace.GlobalState;

/** Represents closure of <i>covers</i> relation.
 * 
 * <p>The closure not maintained in one place (such as {@link MinimalCover}),
 *    but is cut by shared state of configurations.</p>
 * 
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class PartitionedMinimalCover<NAME, VAL> implements Cover<Configuration<NAME, VAL>> {
	
	private PartialOrder<Configuration<NAME, VAL>> coversLocallyRelation;
	private Map<GlobalState<NAME, VAL>, Cover<Configuration<NAME, VAL>>> partitions;
	
	public PartitionedMinimalCover() {
		partitions = new HashMap<GlobalState<NAME,VAL>, Cover<Configuration<NAME,VAL>>>();
		this. coversLocallyRelation = new PartialOrder<Configuration<NAME, VAL>>() {
			@Override
			public boolean inOrder(Configuration<NAME, VAL> a,
					Configuration<NAME, VAL> b) {
				return a.coversLocally(b);
			}
		};
	}



	@Override
	public Configuration<NAME, VAL> add(Configuration<NAME, VAL> e) {
		Cover<Configuration<NAME, VAL>> partition = partitions.get(e.getShared());
		if (partition == null) {
			partition = new MinimalClosure<Configuration<NAME,VAL>>(coversLocallyRelation);
			partitions.put(e.getShared(), partition);
		}
		return partition.add(e); 
	}
	
}
