/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C)      2011  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.karpmiller.opt;

import java.util.Collection;

import org.ow2.dsrg.fm.badger.ca.karpmiller.Configuration;
import org.ow2.dsrg.fm.badger.ca.karpmiller.ConfigurationNode;
import org.ow2.dsrg.fm.badger.ca.karpmiller.Context;
import org.ow2.dsrg.fm.badger.ca.karpmiller.TreeConstruction;

/** Wraps Karp-Miller tree construction algorithm with earlier <i>end</i> discovery optimization.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 * @param <NAME> variable name type
 * @param <VAL> variable value type
 */
public class TreeConstructionEarlierEnd<NAME, VAL> extends TreeConstruction<NAME, VAL> {
	
	private Cover<Configuration<NAME, VAL>> coverOfReachedNodes;

	/** Creates instance of this without container for end nodes.
	 * 
	 * <p>Note that reached <i>end</i> nodes without <i>error</i> label
	 *    will <b>NOT</b> be stored anywhere.</p>
	 * 
	 * @param settings algorithm modifiers
	 */	
	public TreeConstructionEarlierEnd(Settings settings) {
		super(settings, null);
	}
	/** Creates instance of this with container for end nodes <b>endNodesContainer</b>.
	 * 
	 * <p>The <b>endNodesContainer</b> will be filled with reached <i>end</i> nodes
	 *    during tree construction.</p>
	 * 
	 * @param settings algorithm modifiers
	 * @param endNodesContainer
	 */
	public TreeConstructionEarlierEnd(Settings settings, Collection<ConfigurationNode<NAME, VAL>> endNodesContainer) {
		super(settings, endNodesContainer);
	}
	
	@Override
	protected void init(Context<NAME, VAL> ctx) {
		super.init(ctx);
		this.coverOfReachedNodes = new PartitionedMinimalCover<NAME, VAL>();
			//new NaiveCover<NAME, VAL>();
	}
	
	@Override
	protected boolean isEndNode(ConfigurationNode<NAME, VAL> node) {
		// basic Karp-Miller Tree construction: node.containsPredecessor(node.getConfiguration())
		// which means that node is end node if its configuration already occured within its predecessers
		
		// optimization: using covers closure
		Configuration<NAME, VAL> c = this.coverOfReachedNodes.add(node.getConfiguration());
		if (c != null) {
			node.setPreviousConfiguration(c);
		}
		return c != null;
	}
}
