/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.provision;

import java.util.HashMap;
import java.util.Map;

import org.ow2.dsrg.fm.tbplib.ltsa.LTSA;
import org.ow2.dsrg.fm.tbplib.provision.CounterLimit;
import org.ow2.dsrg.fm.tbplib.provision.ProvisionAutomaton;
import org.ow2.dsrg.fm.tbplib.provision.SymbolTransformer;
import org.ow2.dsrg.fm.badger.ca.statespace.LocationReference;
import org.ow2.dsrg.fm.badger.ca.statespace.encoding.LocalStateEncoder;
import org.ow2.dsrg.fm.badger.ca.statespace.impl.LocationReferenceImpl;
import org.ow2.dsrg.fm.badger.ca.statespace.impl.ltsa.CompactLTSAFactory;

/** Transforms output symbols of {@link ProvisionAutomaton}
 *  (<i>SYM</i> &rarr; {@link LocationReference}). .
 * 
 * <p>Note that the original output symbol type <i>SYM</i>
 *    is expected to provide <i>symbol</i> equivalence using equivalence
 *    imposed by {@link Object#equals(Object) equals()}
 *    and its {@link Object#hashCode() hashCode()} must be
 *    consistent with it.
 * </p>
 * 
 * <p>Since the {@link LocationReference} refers a {@link LTSA} transition
 *    (via program counter value), the transition is created using
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class AutomataSymbolTransformer<SYM extends CounterLimit, VARIABLE_NAME, VARIABLE_VALUE> implements
		SymbolTransformer<SYM, LocationReference> {
	
	public AutomataSymbolTransformer(
			CompactLTSAFactory<?, ?, VARIABLE_NAME, VARIABLE_VALUE> factory,
			LocalStateEncoder<VARIABLE_NAME, VARIABLE_VALUE> localStateEncoder) {
		this.factory = factory;
		this.localStateEncoder = localStateEncoder;
	}
	
	private Map<SYM, LocationReference> alreadyTransformedSymbols
		= new HashMap<SYM, LocationReference>();
	
	private CompactLTSAFactory<?, ?, VARIABLE_NAME, VARIABLE_VALUE> factory;
	private LocalStateEncoder<VARIABLE_NAME, VARIABLE_VALUE> localStateEncoder;
	
	@Override
	public LocationReference transform(SYM s) {
		LocationReference r = alreadyTransformedSymbols.get(s);

		if (r == null) {
			int pc = factory.create(localStateEncoder, s).hashCode();
			r = new LocationReferenceImpl(pc, s.getUpperLimit());
			alreadyTransformedSymbols.put(s, r);
		}
		return r;
	}
}
