/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.provision;

import java.util.Collection;
import java.util.List;

import org.ow2.dsrg.fm.badger.ca.statespace.LocationReference;
import org.ow2.dsrg.fm.badger.ca.util.Pair;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSAThread;
import org.ow2.dsrg.fm.tbplib.ltsa.Observer;
import org.ow2.dsrg.fm.tbplib.provision.ObservingProvisionAutomaton;

/** Retains {@link ObservingProvisionAutomaton provision automata}
 *  and {@link Observer observers}
 *  associated with observed {@link LTSAThread}.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface ThreadObservationSystem<VARNAME> {
	/** Returns {@link ObservingProvisionAutomaton provision automata} and their state variables.
	 * 
	 * @return {@link Collection} of (<i>a</i>, <i>e</i>) {@link Pair}s,
	 *         where <i>a</i> is an {@link ObservingProvisionAutomaton} and
	 *         <i>e</i> is a {@link List} of thread-local variables
	 *         that encode state of <i>a</i>. 
	 */
	public Collection<Pair<ObservingProvisionAutomaton<LocationReference>, List<VARNAME>>> getProvisionAutomata();
	
	/** Returns {@link Observer}s that may be eventually affected by the observed thread. 
	 * 
	 * @return {@link Collection} of (<i>a</i>, <i>e</i>) {@link Pair}s,
	 *         where <i>a</i> is an {@link Observer} and
	 *         <i>e</i> is a {@link List} of global variables
	 *         that encode state of <i>a</i>.
	 */
	public Collection<Pair<Observer, List<VARNAME>>> getObservers();
}
