/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.statespace;

import java.util.Map;

import org.ow2.dsrg.fm.badger.ca.statespace.encoding.LocalStateCode;

/** Represents <i>local state</i> i.e. valuation of local variables and program counter.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface LocalState<NAME, VAL> extends VariableValuation<NAME, VAL>, LocalStateCode {
	/** Returns value of program counter.
	 * 
	 * @return value of the program counter
	 */
	public int getProgramCounterValue();
	
	/** Returns {@link LocalState} with modified program counter.
	 *  
	 * @param newProgramCounterValue new value of program counter
	 * @return {@link LocalState} with modified program counter 
	 */
	public LocalState<NAME, VAL> modify(int newProgramCounterValue);
	/** Returns {@link LocalState} with modified values of variables.
	 * 
	 * @param newVariableValues variable values modifications
	 * @return {@link LocalState} with modified values of variables
	 */
	public LocalState<NAME, VAL> modify(Map<NAME, VAL> newVariableValues);
	/** Returns {@link LocalState} with modified program counter and values of variables.
	 * 
	 * @param newProgramCounterValue new value of program counter
	 * @param newVariableValues variable values modifications
	 * @return {@link LocalState} with modified program counter and values of variables
	 */
	public LocalState<NAME, VAL> modify(int newProgramCounterValue, Map<NAME, VAL> newVariableValues);
}
