/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.statespace;

import org.ow2.dsrg.fm.tbplib.provision.CounterLimit;
import org.ow2.dsrg.fm.tbplib.provision.ProvisionAutomaton;

/** Isolated program location reference.
 * 
 * <p>
 * <ul>
 * <li>Refers to a program location (by <i>program counter</i>)</li>
 * <li>Keeps maximal parallel execution limit of such location
 *     (see {@link CounterLimit#getUpperLimit()})</li>
 * </ul>
 * <b>Note:</b> intended to be used as <i>output symbol</i>
 *              of {@link ProvisionAutomaton}.
 * 
 * </p>
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface LocationReference extends CounterLimit{
	
	/** Returns <i>program counter</i> value.
	 * 
	 * @return <i>program counter</i> value of referred program location
	 */
	int getLocationProgramCounter();	
}
