/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.statespace;

import java.util.Collection;
import java.util.List;
import java.util.Observer;

import org.ow2.dsrg.fm.badger.ca.karpmiller.Transition;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSASpecification;
import org.ow2.dsrg.fm.tbplib.provision.ProvisionAutomaton;

/** Provides state space exploration phase.
 * 
 * <p>An implementation is assumed to be created of
 *    {@link LTSASpecification}, a collection of {@link Observer Observers}
 *    and a collection of {@link ProvisionAutomaton Provision Automata}.
 * </p>
 * <p>The main purpose of the implementation is to explore state space
 *    from given context (composed of global and local states) and give
 *    all possible transitions (which cover subsequent contexts).
 * </p>
 *
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface StateSpaceExplorer<NAME, VAL> {
	/** Performs exploration of state space from given state.
	 * 
	 * <p>Searches state space for all possible transitions from given state,
	 * which is given by {@link GlobalState} and {@link LocalState}.
	 * </p>
	 * 
	 * @param s {@link GlobalState} - valuation of global (shared) variables
	 * @param l {@link LocalState} - valuation of local variables and program counter
	 * @return All possible {@link Transition}s from given state in a {@link Collection}.  
	 */
	public Collection<Transition<NAME, VAL>> explore(GlobalState<NAME, VAL> s, LocalState<NAME, VAL> l);
	
	/** Encodes state number of an automaton to values of variables.
	 *
	 * @param stateNumber state number
	 * @param variableNames list of variable names that carry the encoding
	 * @return list of variable values (in respective order)
	 */
	public List<VAL> encodeStateNumber(int stateNumber, List<NAME> variableNames);	
}