/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C)      2011  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.statespace;

import java.util.Map;

/**
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface VariableValuation<NAME, VAL> {
	/** Returns value of a variable.
	 * 
	 * @param variableName name of the variable
	 * @return value of the variable
	 */
	public VAL getValueOfVariable(NAME variableName);
	/** Does variable valuation of this contain <b>v</b>?
	 * 
	 * @param v
	 * @return <code>true</code> if and only if this contains
	 *         valuation of <b>v</b>
	 */
	public boolean contains(NAME v);
	/** Returns valuation of variables of this.
	 * 
	 * @return {@link Map} of variable names to values.
	 */
	public Map<NAME, VAL> getVariableValuation();
}
