/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.statespace.encoding;

import org.ow2.dsrg.fm.badger.ca.statespace.LocalState;

/** Root interface for {@link LocalState} encoding.
 * 
 * <p>Forces implementation to override {@link Object}'s
 *    {@link Object#equals(Object) equals(Object)}
 *    and
 *    {@link Object#hashCode() hashCode()}
 *    methods.</p>
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface LocalStateCode {
	
	/** Compares this with given object <b>o</b>.
	 * 
	 * <p>Forces implementation to override Object's {@link Object#equals(Object) equals()}.</p>
	 * 
	 * @param o object
	 * @return <code>true</code> if and only if <b>o</b> equals this
	 */
	@Override
	boolean equals(Object o);
	
	/** Returns hash code of this. 
	 * 
	 * <p>Forces implementation to override Object's {@link Object#hashCode() hashCode()}.</p>
	 * 
	 * @return hash code
	 */
	@Override
	int hashCode();
}
