/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.statespace.impl;

import java.util.Map;

import org.ow2.dsrg.fm.badger.ca.statespace.LocalState;

/**
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class LocalStateImpl<NAME, VAL>
	extends VariableValuationImpl<NAME, VAL>
	implements LocalState<NAME, VAL> {
	
	private int pc;
		
	public LocalStateImpl() {
		this(0, null);
		this.finishInitialization();
	}
	
	protected LocalStateImpl(int programCounterValue, Map<NAME, VAL> variableValues) {
		super(variableValues);
		this.pc = programCounterValue;
	}
	
	@Override
	public int getProgramCounterValue() {
		return this.pc;
	}

	@Override
	public LocalState<NAME, VAL> modify(int newProgramCounterValue) {
		return modify(newProgramCounterValue, null);
	}

	@Override
	public LocalState<NAME, VAL> modify(Map<NAME, VAL> newVariableValues) {
		return modify(this.pc, newVariableValues);
	}

	@Override
	public LocalState<NAME, VAL> modify(int newProgramCounterValue,
			Map<NAME, VAL> newVariableValues) {
		LocalStateImpl<NAME, VAL> modified = new LocalStateImpl<NAME, VAL>(newProgramCounterValue, this.vars);
		modified.addVariableValuesMapping(newVariableValues);
		modified.finishInitialization();
		return modified; 
	}
	
	@Override
	public int hashCode() {
		return super.hashCode();
	}	
				
	@Override
	public boolean equals(Object o) {
		if (o == this) {
			return true;
		} else if (o != null && o instanceof LocalState<?, ?>) {
			return ((LocalState<?, ?>)o).getProgramCounterValue() == this.getProgramCounterValue()
				&& super.equals(o);
		} else {
			return false;
		}
	}
	
	@Override
	public String toString() {
		
		StringBuilder sb = new StringBuilder();
		sb.append("PC=").append(this.pc);
		this.appendAsString(sb, ", ", true);
		return sb.toString();
	}	
}
