/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.statespace.impl.ltsa;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.ow2.dsrg.fm.tbplib.ltsa.LTSA;
import org.ow2.dsrg.fm.tbplib.ltsa.Label;
import org.ow2.dsrg.fm.tbplib.ltsa.visitor.LTSAVisitor;
import org.ow2.dsrg.fm.tbplib.node.TBPAssignment;
import org.ow2.dsrg.fm.tbplib.node.guard.ITBPCondition;

/**
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class CompactLTSAImpl<CODEREF_TYPE, REFERENCE> implements LTSA<CODEREF_TYPE, REFERENCE> {
	
	private int id;
	private ITBPCondition<REFERENCE> guard;
	private List<Label<REFERENCE>> labels;
	private List<TBPAssignment<REFERENCE>> assignments;
	private List<LTSA<CODEREF_TYPE, REFERENCE>> transitions;
	private CODEREF_TYPE codeReference;
	
	protected CompactLTSAImpl(int id, LTSA<CODEREF_TYPE, REFERENCE> transition) {
		this.id = id;
		this.guard = transition.getGuard();
		this.labels = Collections.unmodifiableList(
				new ArrayList<Label<REFERENCE>>(transition.getLabels()));
		this.assignments = Collections.unmodifiableList(
				new ArrayList<TBPAssignment<REFERENCE>>(transition.getAssignments()));
		this.transitions = new LinkedList<LTSA<CODEREF_TYPE,REFERENCE>>();
		this.codeReference = transition.getCodeReference();		
	}
	

	@Override
	public ITBPCondition<REFERENCE> getGuard() {
		return guard;
	}

	@Override
	public List<Label<REFERENCE>> getLabels() {
		return labels;
	}

	@Override
	public List<TBPAssignment<REFERENCE>> getAssignments() {
		return assignments;
	}

	@Override
	public CODEREF_TYPE getCodeReference() {
		return codeReference;
	}

	@Override
	public List<LTSA<CODEREF_TYPE, REFERENCE>> getTransitions() {
		return transitions;
	}

	@Override
	public void connectTransition(LTSA<CODEREF_TYPE, REFERENCE> transition) {
		this.transitions.add(transition);
	}
	
	public void connectingFinished() {
		this.transitions = Collections.unmodifiableList(this.transitions);		
	}

	@Override
	public <RESULT> RESULT visit(
			LTSAVisitor<RESULT, CODEREF_TYPE, REFERENCE> visitor) {
		return visitor.visit(this);
	}
	
	@Override
	public int hashCode(){
		return id;
	}
}
