/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.statespace.impl.ltsa;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.ow2.dsrg.fm.tbplib.ltsa.LTSAThread;
import org.ow2.dsrg.fm.tbplib.ltsa.visitor.LTSAVisitor;
import org.ow2.dsrg.fm.tbplib.node.TBPVariableDefinition;

/**
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class CompactLTSAThreadImpl<CODEREF_TYPE, REFERENCE> extends
		CompactLTSAImpl<CODEREF_TYPE, REFERENCE> implements
		LTSAThread<CODEREF_TYPE, REFERENCE> {
	
	private REFERENCE name;
	private List<TBPVariableDefinition<REFERENCE>> localVariables;
	private int replicationFactor;
	
	
	protected CompactLTSAThreadImpl(
			int id,
			LTSAThread<CODEREF_TYPE, REFERENCE> thread) {
		super(id, thread);
		
		this.name = thread.getName();
		this.localVariables = Collections.unmodifiableList(
				new ArrayList<TBPVariableDefinition<REFERENCE>>(thread.getLocalVariables()));
		this.replicationFactor = thread.getReplicationFactor();
		
	}

	@Override
	public REFERENCE getName() {
		return name;
	}

	@Override
	public List<TBPVariableDefinition<REFERENCE>> getLocalVariables() {
		return localVariables;
	}

	@Override
	public int getReplicationFactor() {
		return replicationFactor;
	}
	
	@Override
	public <RESULT> RESULT visit(
			LTSAVisitor<RESULT, CODEREF_TYPE, REFERENCE> visitor) {
		return visitor.visitThread(this);
	}
	
	@Override
	public int hashCode(){
		return super.hashCode();
	}
}
