/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2011       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.stats;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/** Represents a value used during benchmark or data collection.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface ValueHandler {
	/** Gets name of value.
	 * 
	 * @return name
	 */
	public String getName();
	/** Adjusts value.
	 * 
	 * <p>The particular way of adjusting the value depends
	 *    on particular implementations.
	 * </p>
	 * 
	 * @param value new value or its modification
	 */
	public void adjustValue(long value);
	/** Appends this value to {@link Document} as a DOM {@link Node}.
	 *
	 * @param d {@link Document} where this contents will be added to  
	 * @param n {@link Node} where this contents will be added to
	 * 
	 */
	public void appendTo(Document d, Node n);
}
