package org.ow2.dsrg.fm.badger.ca.stats.util;

import org.ow2.dsrg.fm.badger.ca.stats.ValueHandler;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class AppearanceValueHandler extends AbstractValueHandler implements
		ValueHandler {
	
	private static final int MAXIMAL_WIDTH = 1024 * 1024;
	
	private long minTrackedValue = 0;
	private long maxTrackedValue = 0;
	private int bellowMinValues = 0;
	private int beyondMaxValues = 0;
	
	public int[] values; 

	public AppearanceValueHandler(String name, long minTrackedValue, long maxTrackedValue) {
		super(name);
		assert(minTrackedValue <= maxTrackedValue && (maxTrackedValue - minTrackedValue) < Integer.MAX_VALUE);
		long width = maxTrackedValue - minTrackedValue;
		if (width > MAXIMAL_WIDTH) {
			long core = minTrackedValue + width / 2;
			this.minTrackedValue = core - MAXIMAL_WIDTH / 2;
			this.maxTrackedValue = core + MAXIMAL_WIDTH / 2;
		} else {
			this.minTrackedValue = minTrackedValue;
			this.maxTrackedValue = maxTrackedValue;
		}
		this.values = new int [((int)(this.maxTrackedValue - this.minTrackedValue + 1))];
	}

	@Override
	public void adjustValue(long value) {
		if (value < this.minTrackedValue) {
			++bellowMinValues;
		} else if (value > this.maxTrackedValue) {
			++beyondMaxValues;
		} else {
			int index = (int)(value - this.minTrackedValue);
			++this.values[index];
		}
	}
		
	private Element appendAppearanceElement(Document d, Node n, String of, int value) {
		Element a = d.createElement("appearance");
		a.setAttribute("of", of);
		a.setTextContent(Integer.toString(value));
		n.appendChild(a);
		return a;
	}


	@Override
	protected void appendValueTo(Document d, Element e) {
		appendAppearanceElement(d, e, String.format("<%d", this.minTrackedValue), this.bellowMinValues);
		for(int i = 0; i < this.values.length; ++i) {
			Integer app = this.values[i];
			appendAppearanceElement(d, e, Long.toString(this.minTrackedValue + i), app.intValue());
		}
		appendAppearanceElement(d, e, String.format(">%d", this.maxTrackedValue), this.beyondMaxValues);		
	}
}
