/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2011       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.stats.util;

import org.ow2.dsrg.fm.badger.ca.stats.ValueHandler;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/** Holds counter value.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class CounterValueHandler extends AbstractValueHandler implements ValueHandler {
	
	private boolean initialized = false;
	private long value = 0;
	
	public CounterValueHandler(String name, long initialValue) {
		this(name);
		this.value = initialValue;
		initialized = true;
	}
	
	public CounterValueHandler(String name) {
		super(name);
	}

	@Override
	public void adjustValue(long value) {
		this.initialized = true;
		this.value = value;
	}
	
	@Override
	protected void appendValueTo(Document d, Element e) {
		if (this.initialized) {
			e.setTextContent(Long.toString(this.value));
		} else {
			e.setAttribute("initializaed", "false");
		}
	}
}
