/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2011       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.stats.util;

import org.ow2.dsrg.fm.badger.ca.stats.ValueHandler;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/** Hold interval value. 
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class IntervalValueHandler extends AbstractValueHandler implements ValueHandler {
	
	private long lb = Long.MAX_VALUE;
	private long ub = Long.MIN_VALUE;
	private boolean initialized = false;
	
	public IntervalValueHandler(String name) {
		super(name);
	}

	@Override
	public void adjustValue(long value) {
		initialized = true;
		if (value < lb) {
			lb = value;
		}
		if (value > ub) {
			ub = value;
		}
	}
	
	@Override
	protected void appendValueTo(Document d, Element e) {
		if (this.initialized) {
			Element v = d.createElement("width");
			v.setTextContent(Long.toString(this.ub - this.lb));
			e.appendChild(v);
			v = d.createElement("lower-bound");
			v.setTextContent(Long.toString(this.lb));
			e.appendChild(v);
			v = d.createElement("upper-bound");
			v.setTextContent(Long.toString(this.ub));
			e.appendChild(v);
		} else {
			e.setAttribute("initialized", "false");
		}
	}
}
