/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2011       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.stats.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;

import org.ow2.dsrg.fm.badger.ca.stats.StatsCollector;
import org.ow2.dsrg.fm.badger.ca.stats.ValueHandler;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/** Provides dictionary of values.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class ValueDictionary implements StatsCollector {
	
	private ArrayList<ValueHandler> handlers = new ArrayList<ValueHandler>();
	private Map<String, Integer> nameMap = new HashMap<String, Integer>();
	private ValueHandler fakeValueHandler = new ValueHandler() {
		@Override
		public String getName() {
			return "";
		}
		@Override
		public void adjustValue(long value) {
		}
		@Override
		public String toString() {
			return "";
		}
		@Override
		public void appendTo(Document d, Node n) {
		}
	};

	@Override
	public int registerValue(ValueHandler value) {
		ValueHandler valueHandler = value != null ? value : fakeValueHandler;
		Integer oldId = nameMap.get(valueHandler.getName());
		int id;
		
		if (oldId == null) {
			id = handlers.size();
			nameMap.put(valueHandler.getName(), id);
		} else {
			id = oldId.intValue();
		}
		handlers.add(valueHandler);
		return id;
	}

	@Override
	public ValueHandler getValue(int id) {
		if (id >= 0 && id < handlers.size()) {
			return handlers.get(id);
		} else {
			return fakeValueHandler;
		}
	}

	@Override
	public ValueHandler getValue(String name) {
		Integer id = null;
		if (name != null) {
			id = nameMap.get(name);
		}
			
		return getValue(id != null ? id.intValue() : -1);
	}
	
	@Override
	public String toString() {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		String t;
		try {
			this.writeTo(baos);
			t = baos.toString();
		} catch (Exception e) {
			t = String.format("Exception %s with message \"%s\" occured during converision to string.",
						e.getClass().toString(), e.getMessage());
		}
		return t;
	}
	
	public void appendTo(Document xmlDoc, Node node) {
		Element stats = xmlDoc.createElement("stats");
		node.appendChild(stats);
		for(ValueHandler v: this.handlers) {
			v.appendTo(xmlDoc, stats);
		}		
	}
	
	public void writeTo(OutputStream o) throws IOException {
		try {
			DocumentBuilder xmlB = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document xmlDoc = xmlB.newDocument();
			xmlDoc.setXmlStandalone(true);
			this.appendTo(xmlDoc, xmlDoc);
			
			Transformer t = TransformerFactory.newInstance().newTransformer();
			t.setOutputProperty(OutputKeys.INDENT, "yes");
            t.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
           	t.transform(	new javax.xml.transform.dom.DOMSource(xmlDoc),
            				new javax.xml.transform.stream.StreamResult(o));
			
		} catch (TransformerException te) {
			throw new IOException("Can write ValueDictionary to stream.", te);
		} catch (ParserConfigurationException pe) {
			throw new IOException("Can not create document for ValueDictionary.", pe);
		}
	}
}
