/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.util;

/** Forms equivalence above objects of type <b>T</b>.
 * 
 * <p>For an equivalence ~, following must hold:
 * <ul><li><i>a</i> ~ <i>a</i> always holds
 *         <br/>(<i>reflexivity</i>)</li> 
 *     <li>let <i>a</i> ~ <i>b</i>, then <i>b</i> ~ <i>a</i>
 *         <br/>(<i>symmetry</i>)</li>
 *     <li>let <i>a</i> ~ <i>b</i> and <i>b</i> ~ </i>c</i>,
 *         then <i>a</i> ~ </i>c</i> as wll
 *         <br/>(<i>transitivity</i>) 
 *     
 * </ul>
 *    The equivalence is imposed by the {@link #equals(Object, Object)} method.
 * </p>
 * 
 * <p>Note that the equivalence may but does not have to be consistent
 *    with the natural equivalance (impoesed by {@link Object#equals(Object)}).
 * </p>
 * 
 * @param T domain of the relation
 *
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface EqualityComparator<T> {
	
	/** Compares two objects for <i>equality</i>.
	 * 
	 * <p>In particular, determines whether the pair of objects
	 *    (<b>a</b>, <b>b</b>) is member of the <i>equals</i> equivalence
	 *    defined by this implementation. 
	 * </p>
	 * 
	 * @param a
	 * @param b
	 * @return <code>true</code> if and only if <b>a</b> is <i>equal</i> to <b>b</b>.
	 */
	public boolean equals(T a, T b);
}
