/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.ca.util;

import org.ow2.dsrg.fm.badger.ca.karpmiller.Counter;
import org.ow2.dsrg.fm.badger.ca.provision.ThreadObservationSystem;
import org.ow2.dsrg.fm.badger.ca.statespace.LTSARepository;
import org.ow2.dsrg.fm.badger.ca.statespace.LocalState;
import org.ow2.dsrg.fm.badger.ca.statespace.LocationReference;
import org.ow2.dsrg.fm.badger.ca.statespace.StateSpaceExplorer;
import org.ow2.dsrg.fm.badger.ca.statespace.impl.LocalStateImpl;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSASpecification;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSAThread;
import org.ow2.dsrg.fm.tbplib.node.TBPNode;
import org.ow2.dsrg.fm.tbplib.node.TBPVariableDefinition;
import org.ow2.dsrg.fm.tbplib.provision.ObservingProvisionAutomaton;
import org.ow2.dsrg.fm.tbplib.reference.Constant;
import org.ow2.dsrg.fm.tbplib.reference.Reference;
import org.ow2.dsrg.fm.tbplib.reference.Variable;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/** Provides creation of initial local states map.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class LocalStateFactory {
	
	public static Map<LocalState<Variable, Byte>, Counter>
	createLocals(	LTSASpecification<TBPNode<Reference>, Reference> specification,
					LTSARepository<?, Reference, Variable, Byte> repository,
					StateSpaceExplorer<Variable, Byte> encoder) {
        // for each thread create local state
		Map<LocalState<Variable, Byte>, Counter> locals = new HashMap<LocalState<Variable, Byte>, Counter>();
    	Map<Variable, Byte> valuation = new HashMap<Variable, Byte>();
        for(LTSAThread<TBPNode<Reference>, Reference> t : specification.getThreads()) {

        	int replicationFactor = t.getReplicationFactor();
        	if (replicationFactor == 0) {
        		continue; // skip unbound or void threads
        	}
        	
	        valuation.clear();
	        
	        for (TBPVariableDefinition<Reference> var: t.getLocalVariables()) {
	        	Constant c = (Constant)var.getInitialValue();
	        	Variable v = (Variable)var.getName();
	        	valuation.put(v, c.getCode());
	        }
	        ThreadObservationSystem<Variable> tos = repository.getThreadObservationSystem(
	        		repository.getProgramCounterValue(t));
	        if (tos != null) {
		        for(Pair<ObservingProvisionAutomaton<LocationReference>, List<Variable>> p: tos.getProvisionAutomata()) {
		        	ObservingProvisionAutomaton<LocationReference> opa = p.getFirst();
		        	List<Variable> vars = p.getSecond();
		        	int initialState = opa.initialState();
		        	List<Byte> vals = encoder.encodeStateNumber(initialState, vars);
		        	Iterator<Variable> varsIt = vars.iterator();
		        	Iterator<Byte> valsIt = vals.iterator();
		        	while(valsIt.hasNext() && varsIt.hasNext()) {
		        		valuation.put(varsIt.next(), valsIt.next());
		        	}
		        	assert(valsIt.hasNext() == varsIt.hasNext());
		        }
	        }
	        LocalState<Variable, Byte> l = new LocalStateImpl<Variable, Byte>()
	        	.modify(t.hashCode(), valuation);
	        
	        locals.put(l, new Counter(replicationFactor));    
        }
        return locals;
	}
}
