/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.reference;

import org.ow2.dsrg.fm.tbplib.architecture.Autonomous;
import org.ow2.dsrg.fm.tbplib.reference.LocalVariable;
import org.ow2.dsrg.fm.tbplib.util.CodeWriter;

/**
 * Representation of an autonomous activity (=thread) in an architectural
 * information specialized for translation into Java.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class TranslationAutonomous extends Autonomous implements TranslationReference {
    private int initialPC;

    public TranslationAutonomous(String name) {
        super(name);
    }

    @Override
    public boolean writeDefinition(CodeWriter output) {
        output.print("public static final class ");
        output.print("THREAD_");
        output.append(getName()).println(" {");
        output.increaseIndent();

        for (LocalVariable var : getLocals().values()) {
            ((TranslationLocalVariable) var).writeDefinition(output);
        }

        output.decreaseIndent();
        output.println('}');

        return output.checkError();
    }

    @Override
    public boolean writeReadValue(CodeWriter output) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public boolean writeAssignValue(CodeWriter output, TranslationReference value) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    void setInitialPC(int initialPC) {
        this.initialPC = initialPC;
    }

    public int getInitialPC() {
        return initialPC;
    }
}
