/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.reference;

import org.ow2.dsrg.fm.tbplib.reference.*;
import org.ow2.dsrg.fm.tbplib.util.CodeWriter;

/**
 * Representation of a type specialized for translation into Java. Contains
 * special type for mutex variables as a static field.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class TranslationEnumerationType extends EnumerationType implements TranslationReference {

    public TranslationEnumerationType(EnumerationType type) {
        super(type.getName());

        for (Constant c : type.getEnums().values()) {
            TranslationConstant newC =
                    new TranslationConstant(this, c.getName(), (byte)enums.size());

            enums.put(c.getName(), newC);
        }
    }

    @Override
    public boolean writeDefinition(CodeWriter output) {
        output.append("public static final class ").append(getName()).println(" {");
        output.increaseIndent();
        for(Constant c : enums.values()) {
            ((TranslationConstant)c).writeDefinition(output);
        }
        output.decreaseIndent();
        output.println('}');

        return output.checkError();
    }

    @Override
    public boolean writeReadValue(CodeWriter output) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public boolean writeAssignValue(CodeWriter output, TranslationReference value) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
