/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.reference;

import org.ow2.dsrg.fm.tbplib.architecture.Method;
import org.ow2.dsrg.fm.tbplib.reference.EnumerationType;
import org.ow2.dsrg.fm.tbplib.reference.Parameter;
import org.ow2.dsrg.fm.tbplib.util.CodeWriter;

/**
 * Representation of a method parameter specialized for translation into Java.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class TranslationParameter extends Parameter implements TranslationReference {

    private int code;

    public TranslationParameter(Method method, EnumerationType type, String name,
            int code) {
        super(method, type, name);

        this.code = code;
    }

    @Override
    public boolean writeDefinition(CodeWriter output) {
        output.print("public static final int ");
        output.print(getName());
        output.print(" = ");
        output.print(code);
        output.println(";");

        return output.checkError();
    }

    @Override
    public boolean writeReadValue(CodeWriter output) {
        output.print("threadState.getParameters().load(");
        output.print(code);
        output.print(")");

        return output.checkError();
    }

    @Override
    public boolean writeAssignValue(CodeWriter output, TranslationReference value) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
