/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.badger.reference;

import org.ow2.dsrg.fm.tbplib.reference.Reference;
import org.ow2.dsrg.fm.tbplib.util.CodeWriter;

/**
 * Substitute for the Reference class of the tbp-lib. This interface adds
 * functionality necessary for easy translation of imperative behavior
 * into Java.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public interface TranslationReference extends Reference {

    /**
     * Writes a Java representation of the definitions necessary for
     * this concrete refeference.
     *
     * @param output CodeWriter to where the Java code should be printed
     * @return <code>true</code> on success.
     */
    public boolean writeDefinition(CodeWriter output);

    /**
     * Writes a Java representation of reading/loading the value refered to
     * by this reference into the given output stream. No newline is appended.
     * 
     * @param output CodeWriter to where the Java code should be printed
     * @return <code>true</code> on success.
     */
    public boolean writeReadValue(CodeWriter output);

    /**
     * Writes a Java representation of assigning/storing a value into the
     * variable represented by this reference into the given output stream.
     * A new line is appended.
     *
     * @param output CodeWriter to where the Java code should be printed
     * @param value TranslationReference holding the value to be assigned.
     *      The writeReadValue method of this reference will be called.
     * @return <code>true</code> on success.
     */
    public boolean writeAssignValue(CodeWriter output, TranslationReference value);
}
