/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.reference;

import org.ow2.dsrg.fm.tbplib.architecture.Component;
import org.ow2.dsrg.fm.tbplib.reference.*;
import org.ow2.dsrg.fm.tbplib.util.CodeWriter;

/**
 * Representation of a state variable of a component specialized for translation
 * into Java.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class TranslationStateVariable extends StateVariable implements TranslationReference {

    private int code;
    private int stateVariableSet;

    public TranslationStateVariable(EnumerationType type, String name, 
            Constant initialValue, Component component, int stateVariableSet, int code) {
        super(type, name, initialValue, component);

        this.code = code;
        this.stateVariableSet = stateVariableSet;
    }

    public int getCode() {
        return code;
    }

    @Override
    public boolean writeDefinition(CodeWriter output) {
        output.print("public static final int VAR_");
        output.print(getName());
        output.print(" = ");
        output.print(code);
        output.println(";");
        return output.checkError();
    }

    @Override
    public boolean writeReadValue(CodeWriter output) {
        output.print("state.loadStateVariable(");
        output.print(stateVariableSet);
        output.print(", Components.");
        output.print(component.getName());
        output.print(".VAR_");
        output.print(getName());
        output.print(")");

        return output.checkError();
    }

    @Override
    public boolean writeAssignValue(CodeWriter output, TranslationReference value) {
        output.print("state.assignStateVariable(");
        output.print(stateVariableSet);
        output.print(", Components.");
        output.print(component.getName());
        output.print(".VAR_");
        output.print(getName());
        output.print(", ");
        value.writeReadValue(output);
        output.println(");");
        return output.checkError();
    }
}
