/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.badger.simulation;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;

/**
 * Thrown during simulation to signalize property violation.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public abstract class TBPPropertyViolationException extends Exception {

    /**
     * Constructs an instance of <code>TBPPropertyViolationException</code> with the specified detail message.
     * @param msg the detail message.
     */
    public TBPPropertyViolationException(String msg) {
        super(msg);
    }

    long statesVisited;

    public void setStatesVisited(long sv){
        statesVisited = sv;
    }

    
    
    abstract String errorDetails();


    ByteArrayOutputStream callStack = new ByteArrayOutputStream();

    public PrintStream getCallStackForWriting(){
        return  new PrintStream(callStack );
    }

    public void printError(PrintStream ps) throws IOException{

        ps.println("-----------------------------");
        ps.println("ERROR detected ");
        ps.println("Error type:"+errorDetails());
        ps.println("Total visited states: " + statesVisited);

        ps.write(callStack.toByteArray());
    }

    
}
