/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.badger.simulation.state;

/**
 * Mutable, top StackFrame of the active thread is copied on every modification.
 * This class expects explicit calls to <code>recomputeHashCode</code> to keep
 * the hashCode up-to-date.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class StackFrame extends StateElement {

    public ConstantVariableStore parameters;
    public VariableStore variables;
    public byte returnValue;
    public int returnAddress;
    public StackFrame nextFrame;

    public StackFrame(ConstantVariableStore parameters, VariableStore variables,
            int returnAddress, StackFrame nextFrame) {
        // TODO: Better hashcode computation (try to minimize collisions).
        super( 
                347 * (parameters == null ? 0 : parameters.hashCode()) +
                349 * (variables == null ? 0 : variables.hashCode()) +
                137 * returnAddress +
                409 * (nextFrame == null ? 0 : nextFrame.hashCode())
                );
        this.parameters = parameters;
        this.variables = variables;
        this.returnAddress = returnAddress;
        this.returnValue = 0;
        this.nextFrame = nextFrame;
    }

    public void recomputeHashCode() {
        // TODO: Better hashcode computation (try to minimize collisions).
        precomputedHashCode =
                347 * (parameters == null ? 0 : parameters.hashCode()) +
                349 * (variables == null ? 0 : variables.hashCode()) +
                137  * returnAddress +
                139  * returnValue +
                409 * (nextFrame == null ? 0 : nextFrame.hashCode());
    }

    @Override
    public StackFrame clone() {
        return new StackFrame(parameters, variables, returnAddress, nextFrame);
    }


}
