/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.badger.simulation.state;

/**
 * An abstract predecessor for all parts of the state identification.
 * All successors should:
 * <ul>
 * <li>be immutable (an exception is State and ThreadState, which are
 *      copied for every new state)</li>
 * <li>be easily cloneable</li>
 * <li>precompute the hashcode on creation (<code>precomputedHashCode</code>)</li>
 * </ul>
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public abstract class StateElement implements Cloneable {

    /** A precomputed hashcode is to be stored here by all successors of this 
     * class during their construction. */
    protected int precomputedHashCode;

    protected StateElement(int precomputedHashCode) {
        this.precomputedHashCode = precomputedHashCode;
    }

    /**
     * Simply returns the precomputed hashcode. No computation is involved.
     * @return
     */
    @Override
    public int hashCode() {
        return precomputedHashCode;
    }

    @Override
    public Object clone() {
        try {
            return super.clone();
        } catch (CloneNotSupportedException ex) {
            // Will never occur
            return null;
        }
    }


}
