/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.badger.simulation.state;

import java.util.Arrays;

/**
 * Instances of this class are immutable.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class VariableStore extends StateElement implements ConstantVariableStore {

    /** Actual values of the variables. */
    public byte[] values;

    /**
     * Initializes the variable store.
     * 
     * @param values Initial values of all the variables, the array is expected 
     *      to be immutable (the reference is kept without cloning).
     */
    public VariableStore(byte[] values) {
        super(19 * Arrays.hashCode(values));

        this.values = values;
    }

    /**
     * Assigns the specified value into the variable identified by varId.
     *
     * @param varId
     * @param value
     * @return Returns either the modified version of VariableStore or
     *      <code>null</code> in case the value equals the already stored one.
     */
    public VariableStore assign(int varId, byte value) {
        if (values[varId] == value)
            return null;

        VariableStore modifiedStore = this.clone();
        modifiedStore.values[varId] = value;

        return modifiedStore;
    }

    @Override
    public byte load(int varId) {
        return values[varId];
    }

    public int getSize(){
        return values.length;
    }

    @Override
    public VariableStore clone() {
        return new VariableStore(values.clone());
    }
}
