package org.ow2.dsrg.fm.badger.simulation.transition;


import org.ow2.dsrg.fm.badger.simulation.TBPBadActivityException;
import org.ow2.dsrg.fm.badger.simulation.TBPPropertyViolationException;
import org.ow2.dsrg.fm.badger.simulation.state.State;
import org.ow2.dsrg.fm.tbplib.event.EventCode;
import org.ow2.dsrg.fm.tbplib.event.EventTable;
import org.ow2.dsrg.fm.tbplib.ltsa.DFA;
import org.ow2.dsrg.fm.tbplib.ltsa.Observer;

import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: tom
 * Date: 26.7.2010
 * Time: 15:18:17
 * To change this template use File | Settings | File Templates.
 */
public abstract class BehaviorInterpreterImpl implements BehaviorInterpreter{

    private EventTable eventTable;
    private Observer[] provisions;


    protected BehaviorInterpreterImpl(Observer[] provisions,EventTable eventTable) {
        this.eventTable = eventTable;
        this.provisions = provisions;
    }

    protected final long  advanceProvisions(State state, long event) throws TBPPropertyViolationException {
        if (!EventCode.isObservable(event)) return event;

        int method = EventCode.decodeMethod(event);
        boolean reallocated = false;
        int[] provStates = state.provisions;
        for (int i=0; i<provisions.length; i++){
            Observer prov = provisions[i];
            if (!prov.observesMethod(method)) continue;
            int nextState = prov.nextState(provStates[i], event);
            // No change
            if (nextState == provStates[i]) {
                continue;
            } else if (nextState == DFA.NO_SUCH_TRANSITION) {
                // Signalize the bad activity error
                throw new TBPBadActivityException(event);
            }
            if (!reallocated) {
                // Make a copy
                provStates = Arrays.copyOf(provStates, provStates.length);
                reallocated = true;
            }
            provStates[i] = nextState;
        }
        state.provisions = provStates;
        return event;
    }

    public boolean isFinal(State s) {
        if (s.allThreadsAreInFinalState()==false) {
            return false;
        }

        for (int i = 0 ; i < provisions.length ; i++){
            int provStateId =s.provisions[i];
            if (!provisions[i].isFinal(provStateId)){
                return false;
            }
        }
        return true;
    }

    public Long getAnEventExpectedByObserver(State s) {

        for (int i = 0 ; i < provisions.length ; i++){
            Observer o = provisions[i];
            int provStateId =s.provisions[i];
            if (!o.isFinal(provStateId)){
                Map<Long,Integer> map = o.getEdges(provStateId);
                Iterator<Long> it =map.keySet().iterator();
                if (it.hasNext()){
                    return it.next();
                }
            }
        }

        return null;
    }
}
