/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.generated;

import org.ow2.dsrg.fm.badger.simulation.TBPPropertyViolationException;
import org.ow2.dsrg.fm.badger.simulation.state.State;
import org.ow2.dsrg.fm.badger.simulation.state.ThreadState;
import org.ow2.dsrg.fm.badger.simulation.state.VariableStore;
import org.ow2.dsrg.fm.badger.simulation.transition.BehaviorInterpreterImpl;
import org.ow2.dsrg.fm.badger.simulation.transition.Transition;
import org.ow2.dsrg.fm.tbplib.event.EventCode;
import org.ow2.dsrg.fm.tbplib.event.EventTable;
import org.ow2.dsrg.fm.tbplib.event.EventTableImpl;
import org.ow2.dsrg.fm.tbplib.ltsa.Observer;

/**
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class GeneratedBehavior extends BehaviorInterpreterImpl {
    private static final int Threads = 1;
    private static final int StateVariables = 1;
    private static final int Provisions = 0;
    private Observer[] provisions;

    private static final State initialState = new State(
            Threads == 0 ? null : new ThreadState[]{
                new ThreadState(null, 0)},
            StateVariables == 0 ? null : new VariableStore[]{
                new VariableStore(new byte[]{Types.Signal.TACK})},
            Provisions == 0 ? null : new int[]{0, 0}
                );

    private static final Transition[][] transtionTable = new Transition[][]{
        new Transition[]{new Transition(0, 0)},
        new Transition[]{new Transition(1, 0)},
        new Transition[]{new Transition(2, 0)},
        new Transition[]{new Transition(3, 0)},
        null,
        null,
        null,
        null,
        null,
        null,
        new Transition[]{new Transition(10, 0)},
        new Transition[]{new Transition(11, 0)},
    };

    private static final MethodEncoder1 Logger_log = new MethodEncoder1("Logger.log", 0, Types.Signal.__domain);

    public GeneratedBehavior(Observer[] provisions, EventTable eventTable) {
        super(provisions, eventTable);


        eventTable.addMethodEncoder(Logger_log);
    }

    @Override
    public long executeTransition(State state, int threadId, int transition) throws TBPPropertyViolationException {
        state.prepareThreadExecution(threadId);
        ThreadState threadState = state.threads[threadId];
        long event;

        switch(transition) {
            case 0:     // !Logger.log(TICK)
                threadState.pushStackFrame(
                        new VariableStore(new byte[]{Types.Signal.TICK}),
                        null,
                        1);
                threadState.setProgramCounter(10);
                state.finishThreadExecution(threadId);
                // Signalize event Logger.log(TICK)^;
                event = Logger_log.encodeMethodCall1(Types.Signal.TICK);
                break;
            case 1:     // Pop - can be merged with return in case of no return
                        // value
                threadState.popStackFrame();
                threadState.setProgramCounter(2);
                state.finishThreadExecution(threadId);
                event = EventCode.TAU_ACTION;   // tau
                break;
            case 2:     // !Logger.log(TACK)
                threadState.pushStackFrame(
                        new VariableStore(new byte[]{Types.Signal.TACK}),
                        null,
                        3);
                threadState.setProgramCounter(10);
                state.finishThreadExecution(threadId);
                // Signalize event Logger.log(TACK)^;
                event = Logger_log.encodeMethodCall1(Types.Signal.TACK);
                break;
            case 3:     // Pop - can be merged with return in case of no return
                        // value
                threadState.popStackFrame();
                threadState.setProgramCounter(0);
                state.finishThreadExecution(threadId);
                event = EventCode.TAU_ACTION;   // tau
                break;
            case 10:     // Logger.log(x): Logger.last <- x
                state.assignStateVariable(0, Components.Clock.State,
                        threadState.getParameters().load(0));

                threadState.setProgramCounter(11);
                state.finishThreadExecution(threadId);
                event = EventCode.TAU_ACTION;   // tau
                break;
            case 11:     // Logger.log(x): return
                threadState.setProgramCounter(threadState.getReturnAddress());
                // Signalize event Logger.log$;
                event = Logger_log.encodeMethodReturn(EventCode.VOID_VALUE);
                break;
            default:
                throw new UnsupportedOperationException(
                    "Transition number out of bounds.");
        }
        event = advanceProvisions(state, event);
        state.finishThreadExecution(threadId);

        return event;
    }

    @Override
    public boolean evaluateGuard(State state, int threadId, int guard) {
        ThreadState threadState = state.threads[threadId];
        
        switch (guard) {
            case 0:
                return true;
        }
        throw new UnsupportedOperationException(
                "Guard number out of bounds.");
    }

    @Override
    public State getInitialState() {
        return initialState;
    }

    @Override
    public Transition[] nextTransitions(State state, int threadId) {
        return transtionTable[state.threads[threadId].getProgramCounter()];
    }


    /**
     * Human readable constants for encoding enumerations, state variables,
     * parameters, and local variables. 
     */
    private static class Types {

        public static class Signal {
            public static final int TICK = 0;
            public static final int TACK = 1;
            public static final int __domain = 2;
        }
    }

    private static class Components {

        public static class Clock {
            public static final int State = 0;
        }

        public static class Logger {
        }
    }

    private static class MethodEncoder1 extends EventTableImpl.MethodEncoderImpl {

        public MethodEncoder1(String methodName, int retValDomain, int dom1) {
            super(methodName, retValDomain, dom1);
        }

        public long encodeMethodCall1(int param1) {
            return EventCode.encodeMethodCall(methodCode, param1);
        }
    }
}
