/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.badger.translation;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.ow2.dsrg.fm.badger.reference.TranslationArchitecture;
import org.ow2.dsrg.fm.badger.reference.TranslationReference;
import org.ow2.dsrg.fm.badger.simulation.TBPPropertyViolationException;
import org.ow2.dsrg.fm.badger.simulation.transition.BehaviorInterpreter;
import org.ow2.dsrg.fm.badger.traversal.Traversal;
import org.ow2.dsrg.fm.badger.util.CheckingChain;
import org.ow2.dsrg.fm.tbplib.architecture.Architecture;
import org.ow2.dsrg.fm.tbplib.event.EventTable;
import org.ow2.dsrg.fm.tbplib.event.EventTableImpl;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSSpecification;
import org.ow2.dsrg.fm.tbplib.ltsa.Observer;
import org.ow2.dsrg.fm.tbplib.node.TBPSpecification;
import org.ow2.dsrg.fm.tbplib.reference.Reference;
import org.ow2.dsrg.fm.tbplib.util.TransformationChain;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.ow2.dsrg.fm.badger.CheckSingleLTSRepresentation;
import org.ow2.dsrg.fm.badger.simulation.TBPBadActivityException;
import org.ow2.dsrg.fm.badger.simulation.TBPNoActivityException;

/**
 * Tests encoding on trivial examples.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
@RunWith(Parameterized.class)
public class EncodingLTSTest {

    private static String testPrefix = "test/input/simple/";

    @Parameterized.Parameters
    public static Collection<Object[]> testInputs() {
        return Arrays.asList(new Object[][]{
                     {"uisimple.tbp", null}
                    ,{"whiletest.tbp", null}
                    ,{"iftest.tbp", null}
                    ,{"01_ok.tbp", null}
                    ,{"02_ok.tbp", null}
                    ,{"03_ok.tbp", null}
                    ,{"04_ok.tbp", null}
                    ,{"05_ok.tbp", null}
                    ,{"06_ok.tbp", null}
                    ,{"07_ok.tbp", null}
                    ,{"08_ok.tbp", null}
                    ,{"09_ok.tbp", null}
                    ,{"kaibling_hammer.tbp", null}
                    ,{"synctest.tbp", null}
                    ,{"waittest.tbp", TBPBadActivityException.class}
                    ,{"waittest2.tbp", null}
                    ,{"internal_na.tbp", TBPNoActivityException.class}
                    ,{"external_na.tbp", TBPNoActivityException.class}
                    ,{"external_na2.tbp", TBPNoActivityException.class}
                });
    }
    ;
    public String file;
    public Class exClass;

    public EncodingLTSTest(String file, Class exClass) {
        this.file = file;
        this.exClass = exClass;
    }


    @Test
    public void testEncoding() throws Exception {
        try {
            Assert.assertNotNull(CheckSingleLTSRepresentation.createInterpreterFromFile(
                    testPrefix + file, new EventTableImpl()));
        } catch (Exception ex) {
            if (ex.getClass().equals(exClass)) {
                System.out.println("An exception " + exClass.getCanonicalName() + " thrown as expected.");
                return;
            } else {
                System.out.println("An unexpected exception " + ex.getClass().getCanonicalName() + " thrown.");
                throw ex;
            }
        }
        if (exClass != null && exClass.isAssignableFrom(TBPPropertyViolationException.class)) {
            Assert.fail("An exception " + exClass.getCanonicalName() + " expected.");
        }
    }


    private void checkException(Exception ex) throws Exception{
            if (ex.getClass().equals(exClass)) {
                System.out.println("An exception " + exClass.getCanonicalName() + " thrown as expected.");
                return;
            } else {
                System.out.println("An unexpected exception " + ex.getClass().getCanonicalName() + " thrown.");
                throw ex;
            }
    }

    @Test
    public void testEncodingAndTraversal() throws Exception {
        try {
            EventTable table = new EventTableImpl();

            BehaviorInterpreter interpreter = CheckSingleLTSRepresentation.createInterpreterFromFile(
                    testPrefix + file, table);
            Assert.assertNotNull(interpreter);

            Traversal tr = new Traversal(interpreter, table);

            tr.traverse();

            if (exClass != null) {
              Assert.fail("An exception " + exClass.getCanonicalName() + " expected.");
            }
        } catch (TBPPropertyViolationException ex) {
            ex.printError(System.out);
            checkException(ex);
        } catch (Exception ex) {
            checkException(ex);
        }
        
    }
}