tree grammar TBPTree;

options {
	tokenVocab=TBP;
	ASTLabelType=CommonTree;
}

@header{ 
	package org.ow2.dsrg.fm.tbplib.parser;
	
	import org.ow2.dsrg.fm.tbplib.node.*;
	import org.ow2.dsrg.fm.tbplib.node.guard.*;
	import org.ow2.dsrg.fm.tbplib.reference.*;
	import org.ow2.dsrg.fm.tbplib.architecture.*;
	import org.ow2.dsrg.fm.tbplib.util.*;
	import org.ow2.dsrg.fm.tbplib.Annotation;
	import org.ow2.dsrg.fm.tbplib.TBPProvisionMethodsNameChecker;
	import org.ow2.dsrg.fm.tbplib.TBPMethodNamePattern;
	import java.util.Map;
	import java.util.HashMap;
	import java.util.LinkedHashMap;
	import java.util.HashSet;
	import java.util.Set;
	import java.util.List;
	import java.util.ArrayList;
}

@members{
/*    private Component component;

    public Component getComponent() {
        return component;
    }*/

    private ErrorCollector errorReporter = null;
    public void setErrorReporter(ErrorCollector errorReporter) {
        this.errorReporter = errorReporter;
    }
    public void emitErrorMessage(String msg) {
	if (errorReporter!=null){
	    errorReporter.addError(msg);
	} else {
	    super.emitErrorMessage(msg);	
	}
    }

}

component returns [ParserResult value]
@init{ 
	List<EnumerationType> types = new ArrayList<EnumerationType>();
	List<TBPVariableDefinition<String>> defs = new
        ArrayList<TBPVariableDefinition<String>>();
	List<TBPProvisionContainerNode<String>> provs = new
        ArrayList<TBPProvisionContainerNode<String>>();
	List<TBPMethodDeclaration<String>> reacts = new ArrayList<TBPMethodDeclaration<String>>();
	List<TBPThreadContainerNode<String>> threads = new
        ArrayList<TBPThreadContainerNode<String>>();
	String name = null;
}
	:  ^('component' IDF {name=$IDF.text;}
	                  ^('types'      (type {types.add($type.value);})* ) 		
                          ^('vars'       (var  {defs.add($var.value);})* )
                          ^('provisions' (provision {provs.add($provision.value);})* )
                          ^('reactions'  (reaction  {reacts.add($reaction.value);})* )
                          ^('threads'    (thread    {threads.add($thread.value);})* )
                          )
        ;        
	finally	{ 
        //component = new Component(name, types);
        $value = new ParserResult(
                     new TBPSpecification(name, defs, provs, reacts, threads),
                     types);
    }
     
type	returns [EnumerationType value]
	@init{ List<String> vals = new ArrayList<String>(); }
	:  ^(TYPEDEF name=IDF (v=IDF {vals.add($v.text);})+ )
	     	{
	     		$value = new EnumerationType($name.text, vals); 
	     	}
     	;
     	
var	returns [TBPVariableDefinition value]
	: ^(VARDEF a=IDF b=IDF c=IDF)
		{ $value = new TBPVariableDefinition($a.text,$b.text,$c.text); }
	| ^(VARDEF IDF)
		{ $value = new TBPVariableDefinition($IDF.text); };
		
var_decl returns [TBPVariableDefinition value]
	: ^(VARDEF a=IDF b=IDF c=IDF)
		{$value = new TBPVariableDefinition($a.text,$b.text,$c.text);};
		
	
provision returns [TBPProvisionContainerNode<String> value]
	: ^(PROVISION p_exp method_list IDF?) 
 		{
 			
			TBPProvisionContainerNode<String> val = new TBPProvisionContainerNode<String>($p_exp.value,$IDF.text); 
			val.setFilter($method_list.value);
 			$value = val;
                        
 		};
 		
method_list returns [Set<String> value]
	@init{ value = new HashSet<String>(); }
	: ^(METHOD_LIST (method_list_element {value.add($method_list_element.value);})*)
		;
		
method_list_element returns [String value]
	: ^(METHOD_LIST_ELEMENT '*') {value = "*";}
	| ^(METHOD_LIST_ELEMENT IDF '*') {
		value = $IDF.text+".*";}
	| ^(METHOD_LIST_ELEMENT i=IDF m=IDF) {value = $i.text+"."+$m.text;}
	;

reaction returns [TBPMethodDeclaration<String> value]
	: ^(REACTION method_decl (a=annotation)?)
	 	{ $value = $method_decl.value;  
	 	  if(a != null){
	 	  	$value.setAnnotation($a.value);
	 	  }
	 	};
		
thread returns [TBPThreadContainerNode value]
@init {
	List<TBPVariableDefinition<String>> defs = new
        ArrayList<TBPVariableDefinition<String>>();
}
	: ^(THREAD 
		'vars' (var_decl {defs.add($var_decl.value);})* 
		block 
		IDF?) 
 		{ $value = new TBPThreadContainerNode($IDF.text, $block.value, defs); };

annotation returns [ Annotation value ]
@init{ List<String> keys = new ArrayList<String>(), vals = new ArrayList<String>(); } 
    : ^(ANNOTATION name=IDF 
          (k=IDF v=IDF { keys.add($k.text); vals.add($v.text); } )*
          )
	 	{ 
	 	  $value = new Annotation($name.text, keys, vals);
	 	};


// Provisions 
p_exp	returns [TBPProvisionNode<String> value]:
	  ^('+'  a=p_exp b=p_exp) { $value = new TBPAlternative<String> (a,b); }
	| ^(';'  a=p_exp b=p_exp) { $value = new TBPSequence<String> (a,b); } 	
	| ^('|'  a=p_exp b=p_exp) { $value = new TBPParallel<String> (a,b); } 
	| ^('||' a=p_exp b=p_exp) { $value = new TBPParallelOr<String> (a,b); }	
	| ^('*'  a=p_exp        ) { $value = new TBPRepetition<String> (a); } 
	| ^('|*' a=p_exp        ) { $value = new TBPUnlimitedReentrancy<String> (a); } 
	| ^('|'  d=DIGITS b=p_exp){ $value = new TBPLimitedReentrancy<String> (b, Integer.parseInt($d.text)); }
	| ^('||'  d=DIGITS b=p_exp){ $value = new TBPLimitedReentrancy<String> (b, Integer.parseInt($d.text)); }
	| ^(EVENT (m=method_call)? IDF?)
		{ 
		  if(m != null)
			$value = new TBPAccept<String>($m.value,$IDF.text);
		  else 
		  	$value = new TBPProvisionNull<String>();
		}
	| ^(ANN_PROVISION annotation inner=p_exp) 
	    { 
	      $value = $inner.value;
	      $value.setAnnotation($annotation.value);
	    }
	;

method_decl returns [ TBPMethodDeclaration<String> value ]
@init {
	LinkedHashMap<String,String> params = new LinkedHashMap<String,String>();
	List<String> param_names = new ArrayList<String>();
	List<TBPVariableDefinition<String>> defs = new
        ArrayList<TBPVariableDefinition<String>>();
}
	: ^(METHOD_DECL itf_=IDF meth_=IDF 
			^(PARLIST_DECL (pt=IDF p=IDF {params.put($p.text,$pt.text);param_names.add($p.text);} )* )
			ret_type=IDF?)
			'vars' (var_decl {defs.add($var_decl.value);})* 
			block
		{ $value = new TBPMethodDeclaration<String>($itf_.text, $meth_.text,
                new MethodSignature<String>(
                    params, $ret_type.text), $block.value, defs); }
	;

// //XXX: co kdyz anotace tu opravdu bude??
block returns [TBPImperativeNode<String> value]	
	: ^(';' s1=block s2=block) { $value = new TBPImperativeSequence<String>(s1,s2); }
	| ^('switch' val body)     { $value = new TBPSwitch<String>($val.value,$body.branches,$body.cases); }
	| ^(SWITCH_ND nd_body)     { $value = new TBPSwitch<String>($nd_body.branches); }
	| ^('while' cond b=block)  { $value = new TBPWhile<String>($cond.value, $b.value); }
	| ^('if' cond b1=block b2=block?) { $value = new TBPIf<String>($cond.value, $b1.value, $b2.value); }
	| ^('sync' IDF b=block)    { $value = new TBPSync<String>($IDF.text, $b.value); }
	| ^('wait' IDF )    { $value = new TBPWait<String>($IDF.text); }
	| ^(ASSIGNMENT IDF val)    { $value = new TBPAssignment<String>($IDF.text,$val.value); }
	| ^(RETURN IDF?)            { $value = new TBPReturn<String>($IDF.text); }
	| ^(BLOCK b=block)         { $value = $b.value; }
	| ^( 'NULL' empty)         { $value = new TBPImperativeNull<String>(); }
	| method_call              { $value = new TBPEmit<String>($method_call.value); }
	| ^(ANN_STATEMENT annotation inner=block)
	  {
	    $value = $inner.value;
	    $value.setAnnotation($annotation.value);
	  }
	;
	
empty : ;

body returns [List<TBPImperativeNode<String>> branches, List<String> cases]
@init { 
	$branches = new ArrayList<TBPImperativeNode<String>>(); 
	$cases =  new ArrayList<String>();
      }
	:  (^('case' IDF b=block { $branches.add($b.value); $cases.add($IDF.text); } )
	    | ^('default' b2=block)  { $branches.add($b2.value); } 
	   )+ 	;
	
nd_body returns [List<TBPImperativeNode<String>> branches]
@init { $branches = new ArrayList<TBPImperativeNode<String>>(); }
	: (^('case' block { $branches.add($block.value); }))+ 
	;
	
val returns [TBPValue<String> value]
	: ^(VALUE (IDF { $value=new TBPValue<String>($IDF.text); } | m=method_call {
                $value=new TBPValue<String>($m.value); } ))
	;

cond returns [ITBPCondition<String> value]
	:c1=condor {$value=c1;} ('&&' c2=condor {$value = new TBPAnd<String>($value,c2);} )*

	;

condor returns [ITBPCondition<String> value]
	: c1=condelem {$value=c1;} ('||' c2=condelem {$value = new TBPOr<String>($value,c2); })*
	;


condelem returns [ITBPCondition<String> value]
	: f=IDF '==' s=IDF { $value = new TBPEquals<String>($f.text,$s.text); }
	| f=IDF '!=' s=IDF { $value = new TBPNotEquals<String>($f.text,$s.text); }
	|'!' c1=condelem              { $value = new TBPNot<String>(c1); }
	| '?'	      { $value = new TBPNondetCondition<String>(); }
        | '(' c=cond ')'     { $value = c; }
	;



method_call returns [MethodCall value]:
	^(METHOD_CALL itf_=IDF meth_=IDF parlist) 
		{  $value = new MethodCall($itf_.text, $meth_.text, $parlist.value);  }
	;

parlist returns [List<String> value]
@init{ ArrayList<String> params = new  ArrayList<String>(); }
	: ^(PARLIST (param {params.add($param.value);})*) { $value = params; }
	;


param returns [String value]
        : v=IDF {$value = $v.text;}
        | '?' {$value="?";}
        ;
