/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

/**
 * Provides information what are provided and what required methods.
 * The class is deprecated. It was used just to check that all methods used in provisions are
 * listed as provided methods. 
 * 
 * @author David Hauzar
 *
 */
public class MethodsInfo {
	private static MethodsInfo actualMethodsInfo;
	
	private final Set<String> providedMethods = new HashSet<String>();
	private final Set<String> requiredMethods = new HashSet<String>();
	
	/**
	 * Creates an instance of class MethodsInfo.
	 * @param tbpFileName the name of the file with tbp specification that
	 * 	is actually handled.
	 */
	private MethodsInfo(String tbpFileName) {
		tbpFileName = tbpFileName.substring(0, tbpFileName.length()-4);
		readMethodsFromFileToSet(tbpFileName + "_provided_methods", providedMethods);
		readMethodsFromFileToSet(tbpFileName + "_required_methods", requiredMethods);
	}
	
	/**
	 * Change tbp specification that is actually handled.
	 * @param tbpSpecificationFilename the name of the file with tbp 
	 * 	specification that is actually handled.
	 */
	public static void changeTBPSpecification(String tbpSpecificationFilename) {
		actualMethodsInfo = new MethodsInfo(tbpSpecificationFilename);
	}
	
	@Override
	public String toString() {
		return "provided methods: " + providedMethods.toString() + "\n" + "required methods: " + requiredMethods.toString();
	}
	
	/**
	 * Return true if given method is provided.
	 * @param interfaceName the name of interface of the method.
	 * @param methodName the name of the method.
	 * @return true if the method is provided.
	 */
	public static boolean isMethodProvided(String interfaceName, String methodName) {
		return actualMethodsInfo.providedMethods.contains(interfaceName + "." + methodName);
	}
	
	private void readMethodsFromFileToSet(String filename, Set<String> set) {
		BufferedReader in = null;
		try {
			in = new BufferedReader(new FileReader(filename));
			String str;
			while ( (str = in.readLine()) != null ) {
                str = str.trim();
				set.add(str);
			}
		} catch (IOException e) {
            System.out.println("File "+filename+" does not exist. The set remains empty");
		}
	}
}
