/*
 *
 * `Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib;

import org.ow2.dsrg.fm.tbplib.node.TBPAccept;
import org.ow2.dsrg.fm.tbplib.node.TBPNode;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Stores information in for section of a provision and uses them to 
 * check if all method calls in given instance of TBPProvisionNode are 
 * calls of provided methods and match to some pattern in for section 
 * of the provision.
 * 
 * @author David Hauzar
 *
 */
public class TBPProvisionMethodsNameChecker {
	private List<TBPMethodNamePattern> methodsPattern = new LinkedList<TBPMethodNamePattern>();


    
	/**
	 * Adds new method pattern (from for section of provision) to 
	 * the checker.
	 * @param pattern the pattern to be added.
	 */
	public void addMethodPattern(TBPMethodNamePattern pattern) {
		methodsPattern.add(pattern);
	}
	
	/**
	 * Checks if all methods in subtree of given provision node are provided
	 * and match to some method name pattern in for section of the provision.
	 * That is, to some pattern stored in this object. 
	 * @param provisionNode the node of AST representing provision.
	 */
	public void checkMethodsInSubtree(TBPNode<String> provisionNode) {
		Iterator<TBPNode<String>> it = provisionNode.getChildren().iterator();
		while (it.hasNext()) {
			TBPNode<String> node = it.next();
			if (node instanceof TBPAccept<?>) {
				TBPAccept<String> accept = (TBPAccept<String>) node;
				
				/*if (!MethodsInfo.isMethodProvided(accept.getInterface(), accept.getMethod())) {
					throw new RuntimeException("Method " + accept.getFullname() + " used in a provision is not provided method.");
				}*/
				if ( !matchMethodToSomeMethodNamePatternInForSection(accept.getInterface(), accept.getMethod()) ) {
					// TODO: pass line of provision for better error message
					throw new RuntimeException("Method " + accept.getFullname() + " used in a provision is not specified in for section of the provision.");
				}
			}
			
			checkMethodsInSubtree(node);
		}
	}
	
	private boolean matchMethodToSomeMethodNamePatternInForSection(String interfaceName, String methodName) {
		Iterator<TBPMethodNamePattern> it = methodsPattern.iterator();
		while (it.hasNext()) {
			if (it.next().matchMethod(interfaceName, methodName)) return true;
		}
		
		return false;
	}
	
}
