/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.architecture;

import org.ow2.dsrg.fm.tbplib.TBPResolvingException;
import org.ow2.dsrg.fm.tbplib.reference.EnumerationType;
import org.ow2.dsrg.fm.tbplib.reference.Reference;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Representation of an architecture.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class Architecture extends Container<Reference> implements Reference {

    protected Map<String, EnumerationType> types = new HashMap<String, EnumerationType>();
    protected List<Component> components = new ArrayList<Component>();
    protected Map<String, Component> componentsMap = new HashMap();

    public Architecture(String name) {
        super(name);
        addType(EnumerationType.MUTEX_TYPE);
    }

    public List<Component> getComponents() {
        return components;
    }

    public EnumerationType addType(EnumerationType type) {
        EnumerationType older = types.get(type.getName());
        if (older != null) {
            if (older.equals(type)) {
                type = older;
            } else {
                throw new TBPResolvingException(
                        "Duplicate definition of an enumeration type that is not identical: " +
                        type.getName());
            }
        }
        types.put(type.getName(), type);
        addSymbol(type);
        
        return type;
    }

    public Map<String, EnumerationType> getTypes() {
        return types;
    }

    @Override
    public Namespace getParent() {
        return null;
    }


    public void addComponent(Component comp) {
        // Check conflicts
        String cName = comp.getName();
        if (componentsMap.containsKey(cName)) {
                throw new TBPResolvingException("Component of the given name is already" +
                        " present in the architecture: " + cName);
        }
        
        comp.setArchitecture(this);
        components.add(comp);
        componentsMap.put(cName, comp);
    }

    public Component getComponent(String instanceName) {
        return componentsMap.get(instanceName);
    }
    
}