/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.architecture;

import java.util.*;
import org.ow2.dsrg.fm.tbplib.reference.Reference;

/**
 * Class representing a simple namespace, i.e., a container for other named
 * entities.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public abstract class Container<REFERENCE extends Reference> implements Namespace {

    protected String name;
    protected Map<String, REFERENCE> symbols = new HashMap<String, REFERENCE>();

    public Container(String name) {
        this.name = name;
    }

    @Override
    public String getName() {
        return name;
    }

    protected void addSymbol(REFERENCE symbol) {
        symbols.put(symbol.getName(), symbol);
    }

    protected <ANY extends REFERENCE> void addSymbols(Collection<ANY> what) {
        for (REFERENCE s : what) {
            addSymbol(s);
        }
    }

    @Override
    public REFERENCE findSymbol(String name) {
        return symbols.get(name);
    }

    @Override
    public boolean containsSymbol(String name) {
        return symbols.containsKey(name);
    }

    protected static <ANY extends Reference> void setSymbolsIn(Map<String, ANY> where, Collection<ANY> what) {
        where.clear();
        for (ANY item : what) {
            where.put(item.getName(), item);
        }
    }

    protected static <ANY extends Reference> void switchSymbolsIn(Map<String, ANY> where, Collection<ANY> oldList, Collection<ANY> newList) {
        if (oldList != null) {
            for (ANY item : oldList) {
                ANY old = where.remove(item.getName());
                assert (old == item);
            }
        }
        for (ANY item : newList) {
            where.put(item.getName(), item);
        }
    }

    protected <ANY extends REFERENCE> void switchSymbols(Collection<ANY> oldList, Collection<ANY> newList) {
        if (oldList != null) {
            for (ANY item : oldList) {
                REFERENCE old = symbols.remove(item.getName());
                assert (old == item);
            }
        }
        addSymbols(newList);
    }

    @Override
    public String toString() {
        return name;
    }
}