/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.architecture;

import java.util.Map;
import org.ow2.dsrg.fm.tbplib.TBPLibLog;
import org.ow2.dsrg.fm.tbplib.reference.Reference;

/**
 * Representation of an interface in an architectural information.
 * FIXME: Get rid of the Container...it was a poor idea.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class Interface extends Container<Method> implements Reference {

    protected Component parent;
    protected Interface boundTo;
    protected Direction direction;

    public enum Direction {
        PROVIDED,
        REQUIRED
    }

    public Interface(String name, Component parent, Direction direction) {
        super(name);
        this.parent = parent;
        this.direction = direction;
    }

    public void addMethod(Method method) {
        symbols.put(method.getName(), method);
    }

    public Direction getDirection() {
        return direction;
    }

    public Map<String, Method> getMethods() {
        return symbols;
    }

    public boolean isBound() {
        assert(direction == Direction.REQUIRED);
        return boundTo != null;
    }

    public void bindTo(Interface iface) {
        assert(direction == Direction.REQUIRED);
        assert(boundTo == null);

        TBPLibLog.logger.fine("Binding \'" + getName() + "\' --> \'" +
                iface.getName() + "\'");

        boundTo = iface;
    }

    public Interface getBoundInterface() {
        return boundTo;
    }

    @Override
    public Component getParent() {
        return parent;
    }

    
    @Override
    public String toString() {
        return getName();
    }
    
    public String getInstanceName() {
        Component parent = getParent();
        String ps="";
        if (parent!=null){
            ps = parent.getName()+"_";
        }
        return ps + getName();
    }
}
