/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.architecture;

import java.util.*;
import org.ow2.dsrg.fm.tbplib.reference.*;

/**
 * Representation of a method.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class Method extends Container<Reference> implements Reference {

    protected MethodSignature<Reference> methodSignature;
    protected Interface iface;
    protected LinkedHashMap<String, Parameter> params = new LinkedHashMap<String, Parameter>();
    protected LinkedHashMap<String, LocalVariable> locals = new LinkedHashMap<String, LocalVariable>();

    public Method(Interface iface, String name) {
        super(name);

        this.iface = iface;
    }

    public void addLocalVariable(LocalVariable localVariable) {
        addSymbol(localVariable);
        locals.put(localVariable.getName(), localVariable);
    }

    public LinkedHashMap<String, LocalVariable> getLocals() {
        return locals;
    }

    public LinkedHashMap<String, Parameter> getParameters() {
        return params;
    }

    public MethodSignature<Reference> getMethodSignature() {
        return methodSignature;
    }

    @Override
    public Interface getParent() {
        return iface;
    }

    public void setMethodSignature(MethodSignature<Reference> methodSignature) {
        this.methodSignature = methodSignature;
    }

    public void setParameters(List<Parameter> params) {
        setSymbolsIn(this.params, params);
    }

    public String getFullname() {
        return iface.getInstanceName() + "." + name;
    }

    @Override
    public String toString() {
        return name;
    }
}
