/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.ltsa;

/**
 * This interface contains methods common to the both deterministic and
 * nondeterministic finite automata. The states are identified by
 * <code>int</code> indexes, where any index from 0 to getNumberOfStates()-1
 * is a valid index of a state.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public interface Automaton {

    /**
     * @return Id of the single initial state.
     */
    int initialState();

    /**
     * Is the given state an accepting state?
     *
     * @param stateId Identification of the queried state
     * @return <code>true</code> if the given state is an accepting one
     */
    boolean isFinal(int stateId);

    /**
     * Retrieves the number of states this automaton is composed of.
     */
    int getNumberOfState();
}
