/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.ltsa;

import org.ow2.dsrg.fm.tbplib.ltsa.visitor.LTSVisitor;
import org.ow2.dsrg.fm.tbplib.node.TBPNode;
import org.ow2.dsrg.fm.tbplib.node.guard.ITBPCondition;

import java.util.List;

/**
 * Representation of transition of a general Labelled Transition System, 
 * where the type of the labels is specified by the template parameter.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public interface LTS<LABEL_TYPE extends TBPNode<REFERENCE>, REFERENCE> {	
	/**
	 * Gets the guard for this LTS transition.
	 * @return the guard for this transition.
	 */
	public ITBPCondition<REFERENCE> getGuard();
	
	/**
	 * Gets the label for this LTS transition.
	 * @return the label for this transition.
	 */
	public LABEL_TYPE getLabel();
	
	public <RESULT> RESULT visit(LTSVisitor<RESULT, REFERENCE> visitor);
	
	/**
	 * Gets transitions connected to this transition.
	 * @return transitions connected to this transition. 
	 */
	public List<LTS<? extends TBPNode<REFERENCE>, REFERENCE>> getTransitions();
	
	/**
	 * Gets the number of transitions connected to this transition.
	 * @return the number of transitions connected to this transition.
	 */
	public int getTransitionsCount();
	
	/**
	 * Connects given transition with this transition.
	 * @param <T> the type of label of the transition to be connected.
	 * @param transition the transition to be connected to this transition. 
	 */
	public <T extends TBPNode<REFERENCE>> void connectTransition(LTS<T, REFERENCE> transition);
}
