/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2010       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.ltsa;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.ow2.dsrg.fm.tbplib.node.TBPAssignment;
import org.ow2.dsrg.fm.tbplib.node.TBPValue;
import org.ow2.dsrg.fm.tbplib.node.TBPVariableDefinition;

/** Provides base for implementations of {@link LTSAThread} and {@link LTSAMethod}.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public abstract class LTSAAutonomousBase<CODEREF_TYPE, REFERENCE> extends LTSAImpl<CODEREF_TYPE, REFERENCE> {
	
	protected REFERENCE name;
	protected List<TBPVariableDefinition<REFERENCE>> localVariables;
	
	protected LTSAAutonomousBase(CODEREF_TYPE codeReference, REFERENCE name, List<TBPVariableDefinition<REFERENCE>> localVariables) {
		super(null, null, null, codeReference);
		this.name = name;
		this.localVariables =  new LinkedList<TBPVariableDefinition<REFERENCE>>();
		this.localVariables.addAll(localVariables);
	}
	
	/** Implementation for {@link LTSAThread#getName()} and {@link LTSAMethod#getName()}.
	 * 
	 * @return REFERENCE
	 */
	public REFERENCE getName() {
		return name;
	}

	/** Implementation for {@link LTSAThread#getLocalVariables()} and {@link LTSAMethod#getLocalVariables()}.
	 *
	 * @return {@link List} of {@link TBPVariableDefinition}
	 */
	public List<TBPVariableDefinition<REFERENCE>> getLocalVariables() {
		return localVariables;
	}
	
	/** Returns assignments of initial values to {@link #getLocalVariables() local variables}.
	 * 
	 * <p>Note that the assignments are being computed from actual content of {@link #getLocalVariables() local variables list}
	 *    each time this method is called.
	 * </p>
	 */
	@Override
	public List<TBPAssignment<REFERENCE>> getAssignments() {
		List<TBPAssignment<REFERENCE>> assignments = new ArrayList<TBPAssignment<REFERENCE>>(localVariables.size());
		// create assignments of initial values
		for(TBPVariableDefinition<REFERENCE> var : localVariables) {
			assignments.add(new TBPAssignment<REFERENCE>(
					var.getName(), new TBPValue<REFERENCE>(var.getInitialValue())));
		}
		return assignments;		
	}	
}
