/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2010       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.ltsa;

import java.util.LinkedList;
import java.util.List;

import org.ow2.dsrg.fm.tbplib.ltsa.visitor.LTSAVisitor;

/**
 * Contains LTSA transitions of threads and methods of transformed TBP protocol.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class LTSASpecification <CODEREF_TYPE, REFERENCE> {
	
	private List<LTSAThread<CODEREF_TYPE, REFERENCE>> threads
		= new LinkedList<LTSAThread<CODEREF_TYPE,REFERENCE>>();
	private List<LTSAMethod<CODEREF_TYPE, REFERENCE>> methods
		= new LinkedList<LTSAMethod<CODEREF_TYPE,REFERENCE>>();
	
	/** Adds thread to the specification.
	 * 
	 * @param thread {@link LTSAThread}
	 */
	public void addThread(LTSAThread<CODEREF_TYPE, REFERENCE> thread) {
		threads.add(thread);
	}
	
	/** Returns all threads of the specification.
	 * 
	 * @return {@link List} of {@link LTSAThread LTSAThreads}
	 */
	public List<LTSAThread<CODEREF_TYPE, REFERENCE>> getThreads() {
		return threads;
	}
	
	/** Adds method (reaction) to the specification.
	 * 
	 * @param method {@link LTSAMethod} 
	 */
	public void addMethod(LTSAMethod<CODEREF_TYPE, REFERENCE> method) {
		methods.add(method);
	}
	
	/** Returns methods (reactions) of the specification. 
	 * 
	 * @return {@link List} of {@link LTSAMethod LTSAMethods}
	 */
	public List<LTSAMethod<CODEREF_TYPE, REFERENCE>> getMethods() {
		return methods;
	}	
	
	/** Visit the specification.
	 * 
	 * @param <RESULT> visitor result type.
	 * @param visitor visitor
	 * @return visitor's result
	 */
	public <RESULT> RESULT visit(LTSAVisitor<RESULT, CODEREF_TYPE, REFERENCE> visitor) {
		return visitor.visitSpecification(this);
	}
}
