/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2010       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.ltsa;

import java.util.List;

import org.ow2.dsrg.fm.tbplib.node.TBPVariableDefinition;

/** Represents autonomous thread.
 * 
 *  <p>
 *  As a {@link LTSA} transition, <i>this</i> represents initial transition of the thread.
 *  In addition to {@link LTSA}, it contains <b>name</b>, <b>local variables</b> and
 *  <b>replication factor</b> (number of thread copies).
 *  </p> 
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface LTSAThread<CODEREF_TYPE, REFERENCE> extends LTSA<CODEREF_TYPE, REFERENCE> {
	
	/** Infinity (for replication factor). */
	public static final int INFINITY = -1;
	
	/** Returns thread name.
	 * 
	 * @return REFERENCE
	 */
	REFERENCE getName();
	
	/** Returns definitions of local variables.
	 *
	 * @return {@link List} of {@link TBPVariableDefinition}s 
	 */
	List<TBPVariableDefinition<REFERENCE>> getLocalVariables();
	
	/** Returns number of thread copies that may be created.
	 * 
	 * @return number
	 */
	int getReplicationFactor();	
}
