/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2010       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.ltsa;

import java.util.List;

import org.ow2.dsrg.fm.tbplib.ltsa.visitor.LTSAVisitor;
import org.ow2.dsrg.fm.tbplib.node.TBPThreadContainerNode;
import org.ow2.dsrg.fm.tbplib.node.TBPVariableDefinition;

/** Represents {@link LTSA} thread.
 * 
 * <p> Instance of this class represents initial {@link LTSA} transition
 *     of a thread and contains following properties of the thread:
 *     <ul><li>name</li>
 *         <li>list of local variables</li>
 *         <li>replication factor (number of copies of the thread)</li>
 *     </ul>
 * </p>
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class LTSAThreadImpl<CODEREF_TYPE, REFERENCE>
		extends LTSAAutonomousBase<CODEREF_TYPE, REFERENCE>
		implements LTSAThread<CODEREF_TYPE, REFERENCE> {
	
	private int replicationFactor;
	
	/** Creates instance of <i>this</i> by fields.
	 * 
	 * @param codeReference reference to thread declaration
	 * @param name name of the thread
	 * @param localVariables locals variables used within the thread
	 * @param replicationFactor replication factor
	 */
	public LTSAThreadImpl(
			CODEREF_TYPE codeReference,
			REFERENCE name,
			List<TBPVariableDefinition<REFERENCE>> localVariables,
			int replicationFactor) {
		super(codeReference, name, localVariables);
		this.replicationFactor = replicationFactor;
	}
	/** Creates instance of <i>this</i> by fields.
	 * 
	 * @param name name of the thread
	 * @param localVariables locals variables used within the thread
	 * @param replicationFactor replication factor
	 */
	public LTSAThreadImpl(
			REFERENCE name,
			List<TBPVariableDefinition<REFERENCE>> localVariables,
			int replicationFactor) {
		this(null, name, localVariables, replicationFactor);
	}
	/** Creates instance of <i>this</i> from {@link TBPThreadContainerNode}.
	 * 
	 * @param codeReference reference to thread declaration
	 * @param thread {@link TBPThreadContainerNode}
	 * @param replicationFactor replication factor
	 */
	public LTSAThreadImpl(
			CODEREF_TYPE codeReference,
			TBPThreadContainerNode<REFERENCE> thread,
			int replicationFactor) {
		this(codeReference, thread.getName(), thread.getLocalVariables(), replicationFactor);
	}	
	/** Creates instance of <i>this</i> from {@link TBPThreadContainerNode}.
	 * 
	 * @param thread {@link TBPThreadContainerNode}
	 * @param replicationFactor replication factor
	 */
	public LTSAThreadImpl(
			TBPThreadContainerNode<REFERENCE> thread,
			int replicationFactor) {
		this(null, thread, replicationFactor);
	}
			
	@Override
	public int getReplicationFactor() {
		return replicationFactor;
	}
	@Override
	public <RESULT> RESULT visit(
			LTSAVisitor<RESULT, CODEREF_TYPE, REFERENCE> visitor) {
		return visitor.visitThread(this);
	}		
}
