/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2010       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.ltsa;

import org.ow2.dsrg.fm.tbplib.ltsa.visitor.LTSVisitor;
import org.ow2.dsrg.fm.tbplib.node.TBPMethodDeclaration;
import org.ow2.dsrg.fm.tbplib.node.TBPNode;
import org.ow2.dsrg.fm.tbplib.node.TBPThreadContainerNode;
import org.ow2.dsrg.fm.tbplib.node.guard.ITBPCondition;

import java.util.LinkedList;
import java.util.List;

/**
 * Implementation of LTS transition.
 * 
 * @author David Hauzar.
 *
 * @param <LABEL_TYPE> the type of label of this transition.
 */
public class LTSImpl<LABEL_TYPE extends TBPNode<REFERENCE>, REFERENCE> implements LTS<LABEL_TYPE, REFERENCE> {
	private final ITBPCondition<REFERENCE> guard;
	private final LABEL_TYPE label;
	private final List<LTS<? extends TBPNode<REFERENCE>, REFERENCE>> connectedTransitions = new LinkedList<LTS<? extends TBPNode<REFERENCE>, REFERENCE>>();
	
	/**
	 * Creates new LTS transition.
	 * @param label the label of this transition.
	 * @param guard the guard of this transition.
	 */
	protected LTSImpl(LABEL_TYPE label, ITBPCondition<REFERENCE> guard) {
		this.guard = guard;
		this.label = label;
	}
	/**
	 * Creates new LTS transition.
	 * @param label the label of this transition.
	 */
	private LTSImpl(LABEL_TYPE label) {
		this.guard = null;
		this.label = label;
	}
	
	/**
	 * Factory method. Creates instance of LTS transition labeled with
	 * given label guarded by given guard.
	 * @param <LABEL_TYPE> the type of label of the transition to be created.
	 * @param label the label of this transition.
	 * @param guard the guard of this transition.
	 * @return the instance of LTS transition.
	 */
	public static <LABEL_TYPE extends TBPNode<REFERENCE>, REFERENCE> 
		LTS<? extends TBPNode<REFERENCE>, REFERENCE> createInstance(LABEL_TYPE label, ITBPCondition<REFERENCE> guard) {
		if (label instanceof TBPMethodDeclaration<?>) {
			
		}
		if (label instanceof TBPThreadContainerNode<?>) {
			return new LTSThread<REFERENCE>((TBPThreadContainerNode<REFERENCE>) label);
		}
		if (label instanceof TBPMethodDeclaration<?>) {
			return new LTSMethod<REFERENCE>((TBPMethodDeclaration<REFERENCE>) label);
		}
		
		return new LTSImpl<LABEL_TYPE, REFERENCE>(label, guard);
	}
	
	@Override
	public ITBPCondition<REFERENCE> getGuard() {
		return guard;
	}
	
	@Override
	public LABEL_TYPE getLabel() {
		return label;
	}
	
	//@Override
	public <RESULT> RESULT visit(LTSVisitor<RESULT, REFERENCE> visitor) {
		return visitor.visit(this);
	}
	
	@Override
	public List<LTS<? extends TBPNode<REFERENCE>, REFERENCE>> getTransitions() {
		return connectedTransitions;
	}
	
	@Override
	public int getTransitionsCount() {
		return connectedTransitions.size();
	}
	@Override
	public <T extends TBPNode<REFERENCE>> void connectTransition(LTS<T, REFERENCE> transition) {
		connectedTransitions.add(transition);	
	}
}
