/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2010       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.ltsa;

import java.util.LinkedList;
import java.util.List;

/**
 * Contains LTS nodes for methods and threads of transformed TBP protocol.
 * 
 * @author David Hauzar
 *
 */
public class LTSSpecification<REFERENCE> {
	private List< LTSThread<REFERENCE> > threads = new LinkedList< LTSThread<REFERENCE> >(); 
	private List< LTSMethod<REFERENCE> > methods = new LinkedList< LTSMethod<REFERENCE> >();

	/** Creates the instance of LTSSpecification. */
	public LTSSpecification() {}
	
	/**
	 * Gets LTS nodes for threads contained in this LTS specification. 
	 * @return LTS nodes for threads contained in this LTS specification.
	 */
	public List< LTSThread<REFERENCE> > getThreads() {
		return threads;
	}
	
	/**
	 * Gets LTS nodes for methods contained in this LTS specification. 
	 * @return LTS nodes for methods contained in this LTS specification.
	 */
	public List< LTSMethod<REFERENCE> > getMethods() {
		return methods;
	}
	
	/**
	 * Adds LTS node for a thread to this LTS specification.
	 * @param thread the LTS node for a thread to be added.
	 */
	public void addThread(LTSThread<REFERENCE> thread) {
		threads.add(thread);
	}
	
	/**
	 * Adds LTS node for a method to this LTS specification.
	 * @param method the LTS node for a method to be added.
	 */
	public void addMethod(LTSMethod<REFERENCE> method) {
		methods.add(method);
	}
}
