/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.ltsa;

import java.util.List;
import java.util.Map;

/**
 * Interface for a represenation of a NFA. States are identified by
 * <code>int</code> codes. Events are expected to be <code>long</code>.
 *
 * Implementations may differ on the level of compaction and also insight into
 * the event structure.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public interface NFA extends Automaton {

    /**
     * Accepts the given event and returns a collection of next states.
     *
     * @param stateId Identification of the current state
     * @param event Event to be accepted
     * @return next state if the event is accepted by the provision,
     *      <code>ERROR_STATE</code> otherwise.
     */
    List<Integer> nextStates(int stateId, long event);

    /**
     * Get edges of the specified state. This method might be slow in some
     * DFA implementations, and should be used only for debuging and printing
     * purposes where speed is not essential.
     *
     * @param state State whise edges should be returned.
     * @return Map from events to indices of target states. Do not alter this map.
     */
    Map<Long, List<Integer>> getEdges(int state);
}
