/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.ltsa;

/**
 * Representation of an observer, i.e., a passive part of the behavior that
 * observes a subset of events and signalizes any violation. Such an observer
 * has to be deterministic.
 * 
 * The observer is aware of the event encoding and thus might optimize its 
 * internal representation (e.g., a transition table).
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public interface Observer extends DFA {

    /**
     * Does the observer observe this event?
     *
     * @param event Integer encoding of an event
     * @return <code>true</code> if the given event is observed by this
     *      observer, <code>false</code> when the observer ignores the event.
     */
    boolean observesEvent(long event);

    /**
     * Does the observer observe events associated with the given method?
     *
     * @param method Integer encoding of the method
     * @return <code>true</code> if the given event is observed by this
     *      observer, <code>false</code> when the observer ignores the event.
     */
    boolean observesMethod(int method);
}
