package org.ow2.dsrg.fm.tbplib.ltsa.visitor;

import org.ow2.dsrg.fm.tbplib.node.TBPIf;
import org.ow2.dsrg.fm.tbplib.node.TBPImperativeSequence;
import org.ow2.dsrg.fm.tbplib.node.TBPProvisionNull;
import org.ow2.dsrg.fm.tbplib.node.TBPSpecification;
import org.ow2.dsrg.fm.tbplib.node.TBPSwitch;
import org.ow2.dsrg.fm.tbplib.node.TBPSync;
import org.ow2.dsrg.fm.tbplib.node.TBPWait;
import org.ow2.dsrg.fm.tbplib.node.TBPWhile;
import org.ow2.dsrg.fm.tbplib.node.visitor.TBPVisitor;

/**
 * Provides implementations for visiting TBP nodes that cannot be labels
 * of LTS transitions.
 * @author David Hauzar
 *
 */
public abstract class LTSLabelsVisitorAdapter<RESULT, REFERENCE> implements TBPVisitor<RESULT, REFERENCE> {

	@Override
	public RESULT visitParsedIf(TBPIf<REFERENCE> node) {
		throw new UnsupportedOperationException("LTS transition should not contain label of this type.");
	}

	@Override
	public RESULT visitParsedNull(TBPProvisionNull<REFERENCE> node) {
		throw new UnsupportedOperationException("LTS transition should not contain label of this type.");
	}

	@Override
	public RESULT visitParsedSwitch(TBPSwitch<REFERENCE> node) {
		throw new UnsupportedOperationException("LTS transition should not contain label of this type.");
	}

	@Override
	public RESULT visitParsedSync(TBPSync<REFERENCE> node) {
		throw new UnsupportedOperationException("LTS transition should not contain label of this type.");
	}
        
        @Override
	public RESULT visitParsedWait(TBPWait<REFERENCE> node) {
		throw new UnsupportedOperationException("LTS transition should not contain label of this type.");
	}

	@Override
	public RESULT visitParsedWhile(TBPWhile<REFERENCE> node) {
		throw new UnsupportedOperationException("LTS transition should not contain label of this type.");
	}
	
	@Override
	public RESULT visitSpecification(TBPSpecification<REFERENCE> node) {
		throw new UnsupportedOperationException("LTS transition should not contain label of this type.");
	}
	
	@Override
	public RESULT visitParsedImperativeSequence(
			TBPImperativeSequence<REFERENCE> node) {
		throw new UnsupportedOperationException("LTS transition should not contain label of this type.");
	}
}
