/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.node;

import java.util.*;

/**
 * This class represents composition of components.
 * 
 * @author caitt3am
 *
 */
public class TBPComposition<REFERENCE> {
    
	private List<TBPSpecification<REFERENCE>> specs = new ArrayList<TBPSpecification<REFERENCE>>();

    /**
     * Add a TBP specification into the current composition.
     *
     * @param spec A TBP specification to be added.
     */
	public void addTBP(TBPSpecification<REFERENCE> spec) {
		specs.add(spec);
	}

    /**
     * Add multiple TBP specifications into the current composition. 
     * 
     * @param specs A list of TBP specifications to be added.
     */
	public void addMultipleTBP(List<TBPSpecification<REFERENCE>> specs) {
        for (TBPSpecification<REFERENCE> spec : specs) {
            this.specs.add(spec);
        }
	}
	
	/**
	 * Creates new TBP specification composed from specifications
	 * added through {@link #addTBP(TBPSpecification)}.
	 * Parts of components are put together. Component variables are prefixed
	 * with names of component to avoid name clashes.
	 * The added specifications are consumed during the process.
	 * 
	 * @return Resulting TBP specification (merge of the given specifications).
	 */
	public TBPSpecification<REFERENCE> getComposition(REFERENCE name) {

		List<TBPVariableDefinition<REFERENCE>> defs =
                new ArrayList<TBPVariableDefinition<REFERENCE>>();
		List<TBPProvisionContainerNode<REFERENCE>> provisions =
                new ArrayList<TBPProvisionContainerNode<REFERENCE>>();
		List<TBPMethodDeclaration<REFERENCE>> reactions =
                new ArrayList<TBPMethodDeclaration<REFERENCE>>();
		List<TBPThreadContainerNode<REFERENCE>> threads =
                new ArrayList<TBPThreadContainerNode<REFERENCE>>();
		
		for(TBPSpecification<REFERENCE> spec : specs){
            defs.addAll(spec.getVariables());
			provisions.addAll(spec.getProvisions());
			reactions.addAll(spec.getReactions());
			threads.addAll(spec.getThreads());
		}
		
        specs.clear();

		return new TBPSpecification<REFERENCE>(name, defs, provisions, reactions,
					threads);
	}
}
