/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.node;

import org.ow2.dsrg.fm.tbplib.node.guard.ITBPCondition;
import org.ow2.dsrg.fm.tbplib.node.visitor.TBPVisitor;

/**
 * @author caitt3am
 *
 */
public class TBPIf<REFERENCE> extends TBPImperativeNaryNode<REFERENCE> {

	private final ITBPCondition<REFERENCE> cond;

	/**
	 * Creates new if.
	 * @param cond - branching condition
	 * @param ifbranch - branch for if path 
	 * @param elsebranch - branch for else path - can be null
	 */
	public TBPIf(ITBPCondition<REFERENCE> cond, TBPImperativeNode<REFERENCE> ifbranch,
			TBPImperativeNode<REFERENCE> elsebranch) {
		this.cond = cond;
		addChild(ifbranch);
		if(elsebranch!=null)
			addChild(elsebranch);
	}
	
	public ITBPCondition<REFERENCE> getCondition() {
		return cond;
	}

    public boolean hasElse() {
        return getChildCount() > 1;
    }

	@Override
	public <RESULT> RESULT visit(TBPVisitor<RESULT, REFERENCE> visitor) {
		return visitor.visitParsedIf(this);
	}
}
