/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.node;

import java.util.List;

import org.ow2.dsrg.fm.tbplib.node.visitor.TBPVisitor;

/**
 * @author caitt3am
 *
 */
public class TBPSwitch<REFERENCE> extends TBPImperativeNaryNode<REFERENCE> {

	private final List<REFERENCE> cases;
	private final TBPImperativeNode<REFERENCE> defaultBranch;
	private final TBPValue<REFERENCE> value;

	public TBPSwitch(List<TBPImperativeNode<REFERENCE>> branches) {

		this.cases = null;
		this.defaultBranch = null;
		this.value = null;
		for(TBPImperativeNode<REFERENCE> branch : branches){
			addChild(branch);
		}
	}
	
	public TBPSwitch(
			TBPValue<REFERENCE> value,
			List<TBPImperativeNode<REFERENCE>> branches,
			List<REFERENCE> cases) {
		
				this.value = value;
				int csize = cases.size();
				int bsize = branches.size();
				assert csize == bsize || bsize==csize+1;
				
				this.cases = cases;
				if(bsize-1 == csize){
					this.defaultBranch = branches.get(csize);
				} else {
					this.defaultBranch = null;
				}
				for(TBPImperativeNode<REFERENCE> branch : branches){
					assert branch != null;
					addChild(branch);
				}
	}

	public boolean isNondeterministic(){
		return cases == null;
	}

	public boolean isDeterministic(){
		return cases != null;
	}

	public List<REFERENCE> getCases() {
		return cases;
	}
	
	public TBPValue<REFERENCE> getValue() {
		return value;
	}
	
	public TBPImperativeNode<REFERENCE> getDefaultBranch() {
		return defaultBranch;
	}
	
	@Override
	public <RESULT> RESULT visit(TBPVisitor<RESULT, REFERENCE> visitor) {
		return visitor.visitParsedSwitch(this);
	}
}
