/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.node;

import org.ow2.dsrg.fm.tbplib.node.visitor.TBPVisitor;

/**
 * Variable definition.
 * @author caitt3am
 *
 */
public class TBPVariableDefinition<REFERENCE> extends TBPImperativeLeafNode<REFERENCE> {

	boolean mutex;
	REFERENCE name;
	REFERENCE type;
	REFERENCE init;

	public TBPVariableDefinition(REFERENCE type, REFERENCE name, REFERENCE init) {
		this.mutex = false;
		this.type = type;
		this.name = name;
		this.init = init;
	}

	public TBPVariableDefinition(REFERENCE name) {
		this.mutex = true;
		this.name = name;
	}

	public REFERENCE getName() {
		return name;
	}

	public void setName(REFERENCE name) {
		this.name = name;
	}
	
	public REFERENCE getTypename() {
		return type;
	}

	public REFERENCE getInitialValue() {
		return init;
	}

	public boolean isMutex() {
		return mutex;
	}
	
	@Override
	public String toString() {
		if(mutex){
			return "mutex " + name;
		} else {
			return type + " " + name + " = " + init;
		}
	}

	@Override
	public <RESULT> RESULT visit(TBPVisitor<RESULT, REFERENCE> visitor) {
		return visitor.visitVariableDefinition(this);
	}
}