/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.node.visitor;

import org.ow2.dsrg.fm.tbplib.node.*;
import org.ow2.dsrg.fm.tbplib.node.TBPNode;

/**
 * This visitor checks if all path contains return.
 * It is quite conservative.
 *  
 * @author caitt3am
 *
 */
public class CheckReturnsVisitor<REFERENCE> extends ImperativeVisitor<Boolean, REFERENCE> {

    @Override
    public Boolean visitParsedImperativeSequence(TBPImperativeSequence<REFERENCE> node) {
		return node.getLeft().visit(this) || node.getRight().visit(this);
    }

    @Override
    public Boolean visitVariableDefinition(TBPVariableDefinition<REFERENCE> node) {
        return Boolean.FALSE;
    }

    @Override
    public Boolean visitParsedSwitch(TBPSwitch<REFERENCE> node) {
		for (TBPNode<REFERENCE> child : node.getChildren()) {
			if(child.visit(this).equals(Boolean.FALSE))
				return Boolean.FALSE;
		}
		return Boolean.TRUE;
    }

    @Override
    public Boolean visitParsedIf(TBPIf<REFERENCE> node) {

		if(!node.hasElse()){
			return Boolean.FALSE;
		}

        return (node.getChild(0).visit(this)) && (node.getChild(1).visit(this));
    }

    @Override
    public Boolean visitParsedValue(TBPValue<REFERENCE> node) {
		return Boolean.FALSE;
    }

    @Override
    public Boolean visitParsedReturn(TBPReturn<REFERENCE> node) {
		return Boolean.TRUE;
    }

    @Override
    public Boolean visitParsedWhile(TBPWhile<REFERENCE> node) {
		return Boolean.FALSE;
    }

    @Override
    public Boolean visitParsedAssignment(TBPAssignment<REFERENCE> node) {
		return Boolean.FALSE;
    }

    @Override
    public Boolean visitParsedMethodDeclaration(TBPMethodDeclaration<REFERENCE> node) {
		throw new RuntimeException("Unexpected type of node: " + node.getClass());
    }

    @Override
    public Boolean visitParsedSync(TBPSync<REFERENCE> node) {
		return node.getChild().visit(this);
    }
    
    @Override
    public Boolean visitParsedWait(TBPWait<REFERENCE> node) {
		return false;
    }


    @Override
    public Boolean visitParsedImperativeNull(TBPImperativeNull<REFERENCE> node) {
		return Boolean.FALSE;
    }

    @Override
    public Boolean visitParsedThreadContainerNode(TBPThreadContainerNode<REFERENCE> node) {
		return node.getChild().visit(this);
    }

    @Override
    public Boolean visitParsedEmit(TBPEmit<REFERENCE> node) {
		return Boolean.FALSE;
    }

    @Override
    public Boolean visitSpecification(TBPSpecification<REFERENCE> node) {
        for (TBPMethodDeclaration<REFERENCE> reaction : node.getReactions()) {
            if (!reaction.visit(this))
                throw new RuntimeException("Not all code paths return value in reaction: " + reaction.getMethod().toString());
        }
        return Boolean.TRUE;
    }
}
