package org.ow2.dsrg.fm.tbplib.node.visitor;

import org.ow2.dsrg.fm.tbplib.node.guard.*;
import org.ow2.dsrg.fm.tbplib.reference.ReferenceTransformer;

import java.util.HashSet;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: tom
 * Date: 19.7.2010
 * Time: 11:20:40
 * To change this template use File | Settings | File Templates.
 */
public class ConditionResolvingVisitor<FROM, TO> implements ConditionVisitor<ITBPCondition<TO>, FROM> {

    private ReferenceTransformer<FROM, TO> referenceTransformer;

    protected Set<TO> resolveSet(Set<FROM> inSet) {
            Set<TO> dest = new HashSet<TO>();

            for (FROM element : inSet) {
                TO to = referenceTransformer.resolveIdentifier(element);
                dest.add(to);
            }
            return dest;
        }

    public ConditionResolvingVisitor(ReferenceTransformer<FROM, TO> referenceTransformer) {
        this.referenceTransformer = referenceTransformer;
    }

    @Override
    public ITBPCondition<TO> visitTBPInSet(TBPInSet<FROM> node) {
        TO lref = referenceTransformer.resolveIdentifier(node.getLeft());
        Set<TO> dest = resolveSet(node.getSet());
        return new TBPInSet<TO>(lref,dest);
    }

    @Override
    public ITBPCondition<TO> visitTBPNotInSet(TBPNotInSet<FROM> node) {
        TO lref = referenceTransformer.resolveIdentifier(node.getLeft());
        Set<TO> dest = resolveSet(node.getSet());
        return new TBPNotInSet<TO>(lref,dest);
    }

    @Override
    public ITBPCondition<TO> visitTBPEquals(TBPEquals<FROM> node) {
        TO lref = referenceTransformer.resolveIdentifier(node.getLeft());
        TO rref = referenceTransformer.resolveIdentifier(node.getRight());
        return new TBPEquals<TO>(lref,rref);
    }

    @Override
    public ITBPCondition<TO> visitTBPNotEquals(TBPNotEquals<FROM> node) {
        TO lref = referenceTransformer.resolveIdentifier(node.getLeft());
        TO rref = referenceTransformer.resolveIdentifier(node.getRight());
        return new TBPNotEquals<TO>(lref,rref);
    }

    @Override
    public ITBPCondition<TO> visitTBPNondetCondition(TBPNondetCondition<FROM> node) {
        return new TBPNondetCondition<TO>();
    }

        @Override
    public ITBPCondition<TO> visitTBPNot(TBPNot<FROM> node) {
        ITBPCondition<TO> op = node.getOperand().visit(this);
        return new TBPNot<TO>(op);
    }

    @Override
    public ITBPCondition<TO> visitTBPAnd(TBPAnd<FROM> node) {
        ITBPCondition<TO> l = node.getLeft().visit(this);
        ITBPCondition<TO> r = node.getRight().visit(this);
        return new TBPAnd<TO>(l,r);
    }

    @Override
    public ITBPCondition<TO> visitTBPOr(TBPOr<FROM> node) {
        ITBPCondition<TO> l = node.getLeft().visit(this);
        ITBPCondition<TO> r = node.getRight().visit(this);
        return new TBPOr<TO>(l,r);
    }

}
