/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.node.visitor;

import org.ow2.dsrg.fm.tbplib.node.*;
import org.ow2.dsrg.fm.tbplib.reference.MethodCall;

import java.util.ArrayList;
import java.util.List;

/**
 * Copies tree of code nodes.
 * 
 * @author caitt3am
 *
 */
public class CopyVisitor<REFERENCE>
	extends ImperativeVisitor<TBPImperativeNode<REFERENCE>, REFERENCE> {

    /* FIXME: Remove this code!!!
	@Override
	public TBPResolvedImperativeNode visitResolvedSwitch(
			TBPResolvedSwitch resolvedSwitch) {
		
		List<TBPResolvedImperativeNode> branches = new ArrayList<TBPResolvedImperativeNode>();
		
		for (TBPNode b : resolvedSwitch.getChildren()) {
			branches.add(((TBPResolvedImperativeNode)b).visit(this));
		}
		
		TBPResolvedImperativeNode result;
		if(resolvedSwitch.isNondeterministic()){
			result = new TBPResolvedSwitch(branches);
		} else {
		    result = new TBPResolvedSwitch(resolvedSwitch.getCases(),
		    		(TBPResolvedValue) resolvedSwitch.getValue().visit(this),
		    		branches);
		}
		result.setAnnotation(resolvedSwitch.getAnnotation());
		return result;
	}
	
	@Override
	public TBPResolvedImperativeNode visitResolvedValue(
			TBPResolvedValue resolvedValue) {
		TBPResolvedImperativeNode result;
		if(resolvedValue.isReference()){
			result =  new TBPResolvedValue(resolvedValue.getReference());	 
		} else {
			TBPResolvedImperativeNode child = (TBPResolvedImperativeNode) resolvedValue.getChild(0);
			result =  new TBPResolvedValue((TBPResolvedEmit)child.visit(this));
		}
		result.setAnnotation(resolvedValue.getAnnotation());
		return result;
	}
     */


    @Override
    public TBPImperativeNode<REFERENCE> visitParsedImperativeSequence(TBPImperativeSequence<REFERENCE> node) {
        TBPImperativeSequence<REFERENCE> result = new TBPImperativeSequence<REFERENCE>(
                node.getLeft().visit(this), node.getRight().visit(this));
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitVariableDefinition(TBPVariableDefinition<REFERENCE> node) {
        TBPVariableDefinition<REFERENCE> result;

        if (node.isMutex()) {
            result = new TBPVariableDefinition<REFERENCE>(node.getName());
        } else {
            result = new TBPVariableDefinition<REFERENCE>(node.getTypename(), node.getName(), node.getInitialValue());
        }
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedSwitch(TBPSwitch<REFERENCE> node) {
		List<TBPImperativeNode<REFERENCE>> branches = new ArrayList<TBPImperativeNode<REFERENCE>>();

		for (TBPNode<REFERENCE> b : node.getChildren()) {
			branches.add(b.visit(this));
		}

		TBPImperativeNode<REFERENCE> result;
		if(node.isNondeterministic()){
			result = new TBPSwitch<REFERENCE>(branches);
		} else {
		    result = new TBPSwitch<REFERENCE>(
		    		(TBPValue<REFERENCE>)node.getValue().visit(this),
		    		branches,
                    node.getCases());
		}
		result.setAnnotation(node.getAnnotation());
		return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedIf(TBPIf<REFERENCE> node) {
        TBPIf<REFERENCE> result = new TBPIf<REFERENCE>(node.getCondition().clone(),
                node.getChild(0).visit(this),
                node.hasElse() ? node.getChild(1).visit(this) : null);
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedValue(TBPValue<REFERENCE> node) {
		TBPImperativeNode<REFERENCE> result;
        MethodCall<REFERENCE> mc = node.getMethodCall();

		if(mc != null){
            result = new TBPValue<REFERENCE>(mc.clone());
		} else {
			result =  new TBPValue(node.getVarname());
		}
		result.setAnnotation(node.getAnnotation());
		return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedReturn(TBPReturn<REFERENCE> node) {
        TBPReturn<REFERENCE> result = new TBPReturn<REFERENCE>(node.getIdf());
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedWhile(TBPWhile<REFERENCE> node) {
        TBPWhile<REFERENCE> result = new TBPWhile<REFERENCE>(
                node.getCondition().clone(),
                node.getChild().visit(this));
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedAssignment(TBPAssignment<REFERENCE> node) {
        TBPAssignment<REFERENCE> result = new TBPAssignment<REFERENCE>(
                node.getIdf(), (TBPValue<REFERENCE>)node.getChild().visit(this));
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedMethodDeclaration(TBPMethodDeclaration<REFERENCE> node) {
        TBPMethodDeclaration<REFERENCE> result = new TBPMethodDeclaration<REFERENCE>(
                node.getInterface(), node.getMethod(),
                node.getSignature(), node.getChild().visit(this),
                node.getLocalVariables());
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedSync(TBPSync<REFERENCE> node) {
        TBPSync<REFERENCE> result = new TBPSync<REFERENCE>(
                node.getMutex(), node.getChild().visit(this));
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedWait(TBPWait<REFERENCE> node) {
        TBPWait<REFERENCE> result = new TBPWait<REFERENCE>(node.getMutex());
	result.setAnnotation(node.getAnnotation());

        return result;
    }

    
    
    
    @Override
    public TBPImperativeNode<REFERENCE> visitParsedImperativeNull(TBPImperativeNull<REFERENCE> node) {
        TBPImperativeNull<REFERENCE> result = new TBPImperativeNull<REFERENCE>();
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedEmit(TBPEmit<REFERENCE> node) {
        TBPEmit<REFERENCE> result = new TBPEmit<REFERENCE>(
                node.getMethodCall().clone());
		result.setAnnotation(node.getAnnotation());

        return result;
    }

    @Override
    public TBPImperativeNode<REFERENCE> visitParsedThreadContainerNode(TBPThreadContainerNode<REFERENCE> node) {
        TBPThreadContainerNode<REFERENCE> result = new TBPThreadContainerNode<REFERENCE>(
                node.getName(), node.getChild().visit(this),
                node.getLocalVariables());
		result.setAnnotation(node.getAnnotation());

        return result;
    }
}
