/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.node.visitor;

import org.ow2.dsrg.fm.tbplib.TBPResolvingException;
import org.ow2.dsrg.fm.tbplib.node.*;

/**
 * This abstract visitor throws exception on provision nodes. Extends this
 * if you need visitor that works on trees of imperative nodes.
 * It works on all {@link TBPResolvedImperativeNode} nodes with the exception of
 * {@link TBPResolvedMethodDefinition}
 * 
 * @author caitt3am
 *
 */
public abstract class ImperativeVisitor<RESULT, REFERENCE>
        implements TBPVisitor<RESULT, REFERENCE> {
	

	private static TBPResolvingException exc = 
		new TBPResolvingException("This visitor works only on imperative nodes");

	@Override
	public RESULT visitParsedAlternative(TBPAlternative<REFERENCE> node) {
		throw exc;
	}

	@Override
	public RESULT visitParsedParallel(TBPParallel<REFERENCE> node) {
		throw exc;
	}

	@Override
	public RESULT visitParsedParallelOr(TBPParallelOr<REFERENCE> node) {
		throw exc;
	}

	@Override
	public RESULT visitParsedSequence(TBPSequence<REFERENCE> node) {
		throw exc;
	}

	@Override
	public RESULT visitParsedAccept(TBPAccept<REFERENCE> node) {
		throw exc;
	}

	@Override
	public RESULT visitParsedNull(TBPProvisionNull<REFERENCE> node) {
		throw exc;
	}

	@Override
	public RESULT visitLimitedReentrancy(TBPLimitedReentrancy<REFERENCE> node) {
		throw exc;
	}

	@Override
	public RESULT visitParsedRepetition(TBPRepetition<REFERENCE> node) {
		throw exc;
	}

	@Override
	public RESULT visitUnlimitedReentrancy(TBPUnlimitedReentrancy<REFERENCE> node) {
		throw exc;
	}

	@Override
	public RESULT visitParsedProvisionContainerNode(
			TBPProvisionContainerNode<REFERENCE> node) {
		throw exc;
	}

    @Override
    public RESULT visitSpecification(TBPSpecification<REFERENCE> node) {
		throw exc;
    }
}
