/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.node.visitor;

import org.ow2.dsrg.fm.tbplib.node.*;
import org.ow2.dsrg.fm.tbplib.node.guard.ITBPCondition;
import org.ow2.dsrg.fm.tbplib.reference.MethodCall;

import java.util.List;

public class PrefixingVisitor extends TBPDefaultVisitor<Object, String> {
	
	private final String prefix;
	private final List<String> varnames;
	
	public PrefixingVisitor(String prefix, List<String> varnames) {
		this.prefix = prefix;
		this.varnames = varnames;
	}

	private String prefixName(String name){
		if(varnames.contains(name))
			return prefix + name;
		else 
			return name;
	}
	
	@Override
	public Object visitParsedSwitch(TBPSwitch<String> node) {
		if(!node.isNondeterministic())
			node.getValue().visit(this);
		return null; 
	}
	
	@Override
	public Object visitParsedWhile(TBPWhile<String> node) {
		visitCondition(node.getCondition());
		return null;
	}
	
	@Override
	public Object visitParsedIf(TBPIf<String> node) {
		visitCondition(node.getCondition());
		return null;
	}
	
	@Override
	public Object visitParsedSync(TBPSync<String> node) {
		node.setMutex(prefixName(node.getMutex()));
		return null;
	}

        @Override
	public Object visitParsedWait(TBPWait<String> node) {
		node.setMutex(prefixName(node.getMutex()));
		return null;
	}
        
	@Override
	public Object visitParsedAssignment(TBPAssignment<String> node) {
		node.setIdf(prefixName(node.getIdf()));
		return null;
	}
	
	@Override
	public Object visitParsedValue(TBPValue<String> node) {
		String varname = node.getVarname();
		if(varname != null){
			node.setVarname(prefixName(varname));
		} else {
			visitMethodCall(node.getMethodCall());
		}
		return null;
	}

	@Override
	public Object visitParsedEmit(TBPEmit<String> node) {
		visitMethodCall(node.getMethodCall());
		return null;
	}
	
	private void visitMethodCall(MethodCall methodCall) {
		List<String> paramDecl = methodCall.getParamDecl();
		final int s = paramDecl.size();
		
		for(int i = 0; i < s; i++){
			paramDecl.set(i, prefixName(paramDecl.get(i)));
		}
		
	}

	private void visitCondition(ITBPCondition<String> condition) {
        assert false && "unused - implement when needed"==null;
		/*if(condition.getLeft() != null){
			condition.setLeft(prefixName(condition.getLeft()));
			condition.setRight(prefixName(condition.getRight()));
		}*/
	}
	
}