/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.node.visitor;

import org.ow2.dsrg.fm.tbplib.node.TBPAccept;
import org.ow2.dsrg.fm.tbplib.node.TBPAlternative;
import org.ow2.dsrg.fm.tbplib.node.TBPLimitedReentrancy;
import org.ow2.dsrg.fm.tbplib.node.TBPNode;
import org.ow2.dsrg.fm.tbplib.node.TBPParallel;
import org.ow2.dsrg.fm.tbplib.node.TBPParallelOr;
import org.ow2.dsrg.fm.tbplib.node.TBPProvisionContainerNode;
import org.ow2.dsrg.fm.tbplib.node.TBPProvisionNull;
import org.ow2.dsrg.fm.tbplib.node.TBPRepetition;
import org.ow2.dsrg.fm.tbplib.node.TBPSequence;
import org.ow2.dsrg.fm.tbplib.node.TBPUnlimitedReentrancy;

/** Builds a string representation of provision expression.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class ReconstructProvisionExpressionStringVisitor<REFERENCE> extends
		TBPCheckingVisitor<StringBuilder, REFERENCE> {
		
    @Override
    public StringBuilder visitParsedAlternative(TBPAlternative<REFERENCE> node) {
    	return visitBinaryOperator(node.getLeft(), node.getRight(), "+");
    }

    @Override
    public StringBuilder visitParsedParallel(TBPParallel<REFERENCE> node) {
    	return visitBinaryOperator(node.getLeft(), node.getRight(), "|");
    }

    @Override
    public StringBuilder visitParsedParallelOr(TBPParallelOr<REFERENCE> node) {
    	return visitBinaryOperator(node.getLeft(), node.getRight(), "||");
    }

    @Override
    public StringBuilder visitParsedSequence(TBPSequence<REFERENCE> node) {
    	return visitBinaryOperator(node.getLeft(), node.getRight(), ";");
    }

    @Override
    public StringBuilder visitParsedAccept(TBPAccept<REFERENCE> node) {
    	return new StringBuilder(node.getFullname());
    }

    @Override
    public StringBuilder visitParsedNull(TBPProvisionNull<REFERENCE> node) {
        return new StringBuilder("NULL");
    }

    @Override
    public StringBuilder visitLimitedReentrancy(TBPLimitedReentrancy<REFERENCE> node) {
        return visitUnaryOperator(node.getChild(), String.format("|%d", node.getLimit()));
    }

    @Override
    public StringBuilder visitParsedRepetition(TBPRepetition<REFERENCE> node) {
        return visitUnaryOperator(node.getChild(), "*");
    }

    @Override
    public StringBuilder visitUnlimitedReentrancy(TBPUnlimitedReentrancy<REFERENCE> node) {
    	return visitUnaryOperator(node.getChild(), "|?");
    }

    @Override
    public StringBuilder visitParsedProvisionContainerNode(TBPProvisionContainerNode<REFERENCE> node) {
        return node.getChild().visit(this);
    }
    
	private StringBuilder visitBinaryOperator(TBPNode<REFERENCE> left, TBPNode<REFERENCE> right, String operator) {
    	StringBuilder leftSB = left.visit(this);
    	StringBuilder rightSB = right.visit(this);
    	leftSB.insert(0, "(").append(")").append(operator).append("(").append(rightSB).append(")");
    	return leftSB;
	}
	
	private StringBuilder visitUnaryOperator(TBPNode<REFERENCE> child, String operator) {
		StringBuilder childSB = child.visit(this);
		childSB.insert(0, "(").append(")").append(operator);
		return childSB;
	}    
}
