package org.ow2.dsrg.fm.tbplib.node.visitor;

import org.ow2.dsrg.fm.tbplib.architecture.Method;
import org.ow2.dsrg.fm.tbplib.node.*;
import org.ow2.dsrg.fm.tbplib.reference.*;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: tom
 * Date: 21.7.2010
 * Time: 15:45:28
 * To change this template use File | Settings | File Templates.
 */
public class ReplaceArbitraryParameterVisitor extends CopyVisitor<Reference>{

    private TBPEmit<Reference> copyEmit(MethodCall<Reference> mc, List<Reference> params){
        MethodCall<Reference> mcCopy = new MethodCall<Reference>(mc.getInterface(),mc.getMethod(),params);
        return new TBPEmit<Reference>(mcCopy);
    }

    @Override
    public TBPImperativeNode<Reference> visitParsedEmit(TBPEmit<Reference> node) {
        MethodCall<Reference> mc = node.getMethodCall();
        List<List<Reference>> alternativeParams = generateAlternatives(mc);

        if (alternativeParams.size()==1){
            return super.visitParsedEmit(node);
        }

        List<TBPImperativeNode<Reference>> branches = new ArrayList<TBPImperativeNode<Reference>>(alternativeParams.size());
        for (List<Reference> alternativeParam:alternativeParams){
            TBPEmit<Reference> branch = copyEmit(mc,alternativeParam);
            branches.add(branch);
        }
        return new TBPSwitch<Reference>(branches);

    }


    private TBPAssignment<Reference> copyAssignment(TBPAssignment<Reference> node, MethodCall<Reference> mc, List<Reference> params) {
        MethodCall<Reference> mcCopy = new MethodCall<Reference>(mc.getInterface(),mc.getMethod(),params);
        TBPValue<Reference> val = new TBPValue<Reference>(mcCopy);
        TBPAssignment<Reference> ret = new TBPAssignment<Reference>(node.getIdf(),val);
        return ret;
    }

    @Override
    public TBPImperativeNode<Reference> visitParsedAssignment(TBPAssignment<Reference> node) {
        TBPValue<Reference> child = (TBPValue<Reference>) node.getChild();
        if (child.isMethodCall()){
            MethodCall<Reference> mc = child.getMethodCall();
            List<List<Reference>> alternativeParams = generateAlternatives(mc);

            if (alternativeParams.size()==1){
                return super.visitParsedAssignment(node);
            }

            List<TBPImperativeNode<Reference>> branches = new ArrayList<TBPImperativeNode<Reference>>(alternativeParams.size());
            for (List<Reference> alternativeParam:alternativeParams){
                TBPAssignment<Reference> branch = copyAssignment(node,mc,alternativeParam);
                branches.add(branch);
            }
            return new TBPSwitch<Reference>(branches);
        }
        return super.visitParsedAssignment(node);
    }


    private List<List<Reference>> generateAlternatives(MethodCall<Reference> mc) {
        List<Reference> params = mc.getParamDecl();
        Method m = (Method)mc.getMethod();
        MethodSignature<Reference> ms = m.getMethodSignature();

        List<List<Reference>> alternativeParams = new ArrayList<List<Reference>>(1);
        alternativeParams.add(params);

        Collection<Reference> refs = ms.getParams().values();
        Iterator<Reference> irefs = refs.iterator();
        int index = 0;
        for (Reference param:params){
            if (!irefs.hasNext()) {
                break;
            }
            Reference typeDef = irefs.next();

            if (param instanceof ArbitraryValue){
                alternativeParams = multiplyParams(alternativeParams, index, typeDef);
            }
            index++;
        }
        return alternativeParams;
    }




    private List<List<Reference>> multiplyParams(List<List<Reference>> alternativeParams, int index, Reference typeDef) {
        EnumerationType type = (EnumerationType) typeDef;
        int dim =type.getEnums().size();
        List<List<Reference>> newAlternativeParams = new ArrayList<List<Reference>>(dim*alternativeParams.size());
        for (List<Reference> alternative:alternativeParams) {
            for (Constant c:type.getEnums().values()) {
                ArrayList<Reference> newAlternative = new ArrayList<Reference>(alternative);
                assert(newAlternative.get(index) instanceof ArbitraryValue);
                newAlternative.set(index,c);
                newAlternativeParams.add(newAlternative);
            }
        }
        alternativeParams = newAlternativeParams;
        return alternativeParams;
    }
}
