/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License aTBPAccept<REFERENCE> with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.node.visitor;

import org.ow2.dsrg.fm.tbplib.node.TBPAccept;
import org.ow2.dsrg.fm.tbplib.node.TBPAlternative;
import org.ow2.dsrg.fm.tbplib.node.TBPEmit;
import org.ow2.dsrg.fm.tbplib.node.TBPLimitedReentrancy;
import org.ow2.dsrg.fm.tbplib.node.TBPParallel;
import org.ow2.dsrg.fm.tbplib.node.TBPParallelOr;
import org.ow2.dsrg.fm.tbplib.node.TBPProvisionContainerNode;
import org.ow2.dsrg.fm.tbplib.node.TBPProvisionNull;
import org.ow2.dsrg.fm.tbplib.node.TBPRepetition;
import org.ow2.dsrg.fm.tbplib.node.TBPSequence;
import org.ow2.dsrg.fm.tbplib.node.TBPUnlimitedReentrancy;
import org.ow2.dsrg.fm.tbplib.provision.actuator.ActionSeqNode;
import org.ow2.dsrg.fm.tbplib.provision.actuator.BranchAction;
import org.ow2.dsrg.fm.tbplib.provision.actuator.EmitEventAction;
import org.ow2.dsrg.fm.tbplib.provision.actuator.EmitThreadsAction;
import org.ow2.dsrg.fm.tbplib.provision.actuator.NullAction;
import org.ow2.dsrg.fm.tbplib.provision.actuator.RepeatAction;
import org.ow2.dsrg.fm.tbplib.reference.MethodCall;

/**
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 *
 */
public class TranslateProvisionToSymbolicActuatorVisitor<REFERENCE> extends
		TBPCheckingVisitor<ActionSeqNode<REFERENCE>, REFERENCE> {
	
    @Override
    public ActionSeqNode<REFERENCE> visitParsedAlternative(TBPAlternative<REFERENCE> node) {
    	return new BranchAction<REFERENCE>(node.getLeft().visit(this), node.getRight().visit(this));
    }

    @Override
    public ActionSeqNode<REFERENCE> visitParsedParallel(TBPParallel<REFERENCE> node) {
    	ActionSeqNode<REFERENCE> left = node.getLeft().visit(this);
    	ActionSeqNode<REFERENCE> right = node.getRight().visit(this);
    	EmitThreadsAction<REFERENCE> result = new EmitThreadsAction<REFERENCE>();
    	result.addEntryPointOfThread(left);
    	result.addEntryPointOfThread(right);
    	result.setMinThreadLaunches(2);
    	return result;
    }

    @Override
    public ActionSeqNode<REFERENCE> visitParsedParallelOr(TBPParallelOr<REFERENCE> node) {
    	ActionSeqNode<REFERENCE> left = node.getLeft().visit(this);
    	ActionSeqNode<REFERENCE> right = node.getRight().visit(this);
    	EmitThreadsAction<REFERENCE> result = new EmitThreadsAction<REFERENCE>();
    	result.addEntryPointOfThread(left);
    	result.addEntryPointOfThread(right);
    	result.setMinThreadLaunches(1);
    	return result;
    }

    @Override
    public ActionSeqNode<REFERENCE> visitParsedSequence(TBPSequence<REFERENCE> node) {
    	ActionSeqNode<REFERENCE> left = node.getLeft().visit(this);
    	ActionSeqNode<REFERENCE> right = node.getRight().visit(this);
    	// find list of end-points of right
    	left.setSuccessor(right);
    	return left;
    }

    @Override
    public ActionSeqNode<REFERENCE> visitParsedAccept(TBPAccept<REFERENCE> node) {
    	MethodCall<REFERENCE> mc = node.getMethodCall();
    	TBPEmit<REFERENCE> emit = new TBPEmit<REFERENCE>(mc);
    	return new EmitEventAction<REFERENCE>(emit, node.getReturnValue());
    }

    @Override
    public ActionSeqNode<REFERENCE> visitParsedNull(TBPProvisionNull<REFERENCE> node) {
    	return new NullAction<REFERENCE>();
    }

    @Override
    public ActionSeqNode<REFERENCE> visitLimitedReentrancy(TBPLimitedReentrancy<REFERENCE> node) {
    	ActionSeqNode<REFERENCE> left = node.getChild().visit(this);
    	EmitThreadsAction<REFERENCE> result = new EmitThreadsAction<REFERENCE>();
    	result.addEntryPointOfThread(left);
    	result.setMinThreadLaunches(1);
    	result.setMaxThreadReplicas(node.getLimit());
    	return result;
    }

    @Override
    public ActionSeqNode<REFERENCE> visitParsedRepetition(TBPRepetition<REFERENCE> node) {
    	return new RepeatAction<REFERENCE>(node.getChild().visit(this));
    }

    @Override
    public ActionSeqNode<REFERENCE> visitUnlimitedReentrancy(TBPUnlimitedReentrancy<REFERENCE> node) {
    	ActionSeqNode<REFERENCE> left = node.getChild().visit(this);
    	EmitThreadsAction<REFERENCE> result = new EmitThreadsAction<REFERENCE>();
    	result.addEntryPointOfThread(left);
    	result.setMinThreadLaunches(1);
    	result.setMaxThreadReplicas(EmitThreadsAction.INFINITY);
    	return result;
    }

    @Override
    public ActionSeqNode<REFERENCE> visitParsedProvisionContainerNode(TBPProvisionContainerNode<REFERENCE> node) {
        return node.getChild().visit(this);
    }
}
