/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.node.visitor;

import org.ow2.dsrg.fm.tbplib.event.EventTable;
import org.ow2.dsrg.fm.tbplib.ltsa.DynamicNFA;
import org.ow2.dsrg.fm.tbplib.node.TBPLimitedReentrancy;
import org.ow2.dsrg.fm.tbplib.node.TBPParallel;
import org.ow2.dsrg.fm.tbplib.node.TBPParallelOr;
import org.ow2.dsrg.fm.tbplib.node.TBPUnlimitedReentrancy;
import org.ow2.dsrg.fm.tbplib.node.visitor.TranslateProvisionVisitor;
import org.ow2.dsrg.fm.tbplib.reference.Reference;

/** Does the same job as {@link TranslateProvisionVisitor} except this throws exception on any parallel operator.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 *
 */
public class TranslateRegularProvisionVisitor<REFERENCE extends Reference> extends
		TranslateProvisionVisitor<REFERENCE> {
	public TranslateRegularProvisionVisitor(EventTable eventTable)
	{
		super(eventTable);
	}
	
    @Override
    public DynamicNFA visitParsedParallel(TBPParallel<REFERENCE> node) {
    	throw new UnsupportedOperationException("Non-regular operator found in a regular expression.");
    }

    @Override
    public DynamicNFA visitParsedParallelOr(TBPParallelOr<REFERENCE> node) {
    	throw new UnsupportedOperationException("Non-regular operator found in a regular expression.");    	
    }
    
    @Override
    public DynamicNFA visitLimitedReentrancy(TBPLimitedReentrancy<REFERENCE> node) {
    	throw new UnsupportedOperationException("Non-regular operator found in a regular expression.");    	
    }
    
    @Override
    public DynamicNFA visitUnlimitedReentrancy(TBPUnlimitedReentrancy<REFERENCE> node) {
    	throw new UnsupportedOperationException("Non-regular operator found in a regular expression.");    	
    }
}
