/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.provision;

import java.util.Map;

import org.ow2.dsrg.fm.tbplib.ltsa.DFA;

/** Basic interface for deterministic provision automaton.
 * 
 * <p>The automaton is similar to combination of Mealy and Moore machines.
 *    It may emit symbols on edges and there are symbols assigned to states.
 * </p>  
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 *
 */
public interface ProvisionAutomaton<OUTPUTSYMBOL> extends DFA {
	public Transition<OUTPUTSYMBOL> getTransition(int state, long event);
	public Map<Long, Transition<OUTPUTSYMBOL> > getTransitions(int state);
	public OUTPUTSYMBOL getStateSymbol(int state);
}
