/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2010       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.provision;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/** Transition with output symbols coupled with direction (UP/DOWN).
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 *
 */
public class Transition<SYMBOL> {
	public static enum OutputSymbolDirection { Up, Down }
	public static final class OutputSymbol<SYMBOLBASE> {
		private SYMBOLBASE symbol;
		private OutputSymbolDirection direction;
		public OutputSymbol(SYMBOLBASE symbol, OutputSymbolDirection direction) {
			this.symbol = symbol;
			this.direction = direction;
		}
		public SYMBOLBASE getSymbol() {
			return symbol;
		}
		public OutputSymbolDirection getDirection() {
			return direction;
		}		
	}
	public Transition(int targetState, Collection<OutputSymbol<SYMBOL>> symbols) {
		this(targetState, symbols != null ? new ArrayList<OutputSymbol<SYMBOL>>(symbols) : null);
	}
	public Transition(int targetState, ArrayList<OutputSymbol<SYMBOL>> symbols) {
		this.targetState = targetState;
		if (symbols == null) {
			this.symbols = Collections.emptyList();
		} else {
			this.symbols = Collections.unmodifiableList(symbols);	
		}
	}
	public int getTargetState() {
		return this.targetState;
	}
	public Collection<OutputSymbol<SYMBOL>> getOutputSymbols() {
		return this.symbols;
	}
	
	private int targetState;
	private List<OutputSymbol<SYMBOL>> symbols;
}
