/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.provision.actuator;

import org.ow2.dsrg.fm.tbplib.ltsa.LTS;
import org.ow2.dsrg.fm.tbplib.node.TBPNode;
import org.ow2.dsrg.fm.tbplib.provision.actuator.visitor.ActionSeqNodeVisitor;

/** Essential implementation of {@link ActionSeqNode}.
 * 
 * @author Ondej Cerny &lt;ondrej.cerny@email.cz&gt;
 *
 */
public abstract class ActionSeqNodeImpl<REFERENCE> implements ActionSeqNode<REFERENCE> {
	private ActionSeqNode<REFERENCE> successor;
	private LTS<? extends TBPNode<REFERENCE>, REFERENCE> ltsTransition;
	
	public ActionSeqNodeImpl() {
		successor = null;
	}
	
	public ActionSeqNodeImpl(ActionSeqNode<REFERENCE> successor) {
		this();
		this.successor = successor;
	}
	
	@Override
	public ActionSeqNode<REFERENCE> getSuccessor() {
		return successor;
	}

	@Override
	public void setSuccessor(ActionSeqNode<REFERENCE> newValue) {
		successor = newValue;
	}

	@Override
	public LTS<? extends TBPNode<REFERENCE>, REFERENCE> getInitialLTSTransition() {
		return ltsTransition;
	}

	@Override
	public void setInitialLTSTransition(LTS<? extends TBPNode<REFERENCE>, REFERENCE> transition) {
		this.ltsTransition = transition;
	}
	
	abstract public<RESULT> RESULT visit(ActionSeqNodeVisitor<RESULT, REFERENCE> visitor); 
	
}
