/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.provision.actuator;

import java.util.Collection;
import java.util.LinkedList;

import org.ow2.dsrg.fm.tbplib.provision.actuator.visitor.ActionSeqNodeVisitor;

/**
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 *
 */
public class EmitThreadsAction<REFERENCE> extends ActionSeqNodeImpl<REFERENCE> {
	
	private Collection<ActionSeqNode<REFERENCE>> entryPointsOfThreads;
	private int minimalThreadLaunches;
	private int maximalThreadReplicas;
	public static final int INFINITY = -1;
	
	public EmitThreadsAction() {
		minimalThreadLaunches = 0;
		maximalThreadReplicas = 1;
	}
	public EmitThreadsAction(Collection<ActionSeqNode<REFERENCE>> entryPointsOfThreads, int minThreadLaunches, int maxThreadReplicas) {
		this();
		this.entryPointsOfThreads = entryPointsOfThreads;
		this.minimalThreadLaunches = minThreadLaunches;
		this.maximalThreadReplicas = maxThreadReplicas;
	}
	
	
	public Collection<ActionSeqNode<REFERENCE>> getEntryPointsOfThreads() {
		consolidateThreadEntryPointsCollection();
		return entryPointsOfThreads;
	}
	public void setEntryPointsOfThreads(Collection<ActionSeqNode<REFERENCE>> entryPointsOfThreads) {
		this.entryPointsOfThreads = entryPointsOfThreads; 
	}
	public void addEntryPointOfThread(ActionSeqNode<REFERENCE> anotherEntryPointOfThread) {
		consolidateThreadEntryPointsCollection();
		entryPointsOfThreads.add(anotherEntryPointOfThread);
	}
	
	public int getMinThreadLaunches() {
		return minimalThreadLaunches;
	}
	public void setMinThreadLaunches(int newValue) {
		this.minimalThreadLaunches = newValue;
	}
	
	public int getMaxThreadReplicas() {
		return maximalThreadReplicas;
	}
	public void setMaxThreadReplicas(int newValue) {
		this.maximalThreadReplicas = newValue;
	}
	
	
	private void consolidateThreadEntryPointsCollection() {
		if (entryPointsOfThreads == null) {
			entryPointsOfThreads = new LinkedList<ActionSeqNode<REFERENCE>>();
		}
	}
	@Override
	public <RESULT> RESULT visit(
			ActionSeqNodeVisitor<RESULT, REFERENCE> visitor) {
		return visitor.visitEmitThreadsAction(this);
	}	
}
