/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.provision.actuator.visitor;

import java.util.List;

import org.ow2.dsrg.fm.tbplib.ltsa.LTSA;
import org.ow2.dsrg.fm.tbplib.ltsa.LTS;
/** 
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public interface ThreadCooperationScheme {
	
	public static final String EXIT_FLAG_NAME = "_e";
	
	/** Instantiates thread control scheme into {@link LTS}.
	 * 
	 * <p>Creates synchronization infrastructure around given threads (<i>threadBodies</i>)
	 *    which is then returned within output parameters (<i>controllerLTS</i> and <i>threads</i>).
	 * </p>
	 *    
	 * @param <REFERENCE> Generic parameter.
	 * @param variableFactory for creating variables necessary for the scheme
	 * @param threadBodies {@link List list} of threads (their effective bodies)
	 * @param atLeastExecuted minimal number of threads that actually execute their body
	 * @param controllerLTS <i>[out]</i> the main thread (controller)<br/>
	 *                                   (an empty {@link LTSHandler} expected) 
	 * @param threads <i>[out]</i> given threads wrapped within synch. infrastructure<br/>
	 *                             (an empty {@link List} expected)
	 * @deprecated replace by {@link #instanciateScheme(VariableFactory, List, int, LTSAHandler, List)}
	 */
	public <REFERENCE> void instanciateScheme(	VariableFactory<REFERENCE> variableFactory,
												List<LTSHandler<REFERENCE>> threadBodies,
												int atLeastExecuted,
												LTSHandler<REFERENCE> controllerLTS,
												List<LTSHandler<REFERENCE>> threads);
	
	/** Instantiates thread control scheme into {@link LTSA}.
	 * 
	 * <p>Creates synchronization infrastructure around given threads (<i>threadBodies</i>)
	 *    which is then returned within output parameters (<i>controllerLTS</i> and <i>threads</i>).
	 * </p>
	 *    
	 * @param <REFERENCE> Generic parameter.
	 * @param variableFactory for creating variables necessary for the scheme
	 * @param threadBodies {@link List list} of threads (their effective bodies)
	 * @param atLeastExecuted minimal number of threads that actually execute their effective code
	 * @param controllerLTSA <i>[out]</i> the main thread (controller)<br/>
	 *                                   (an empty {@link LTSAHandler} expected) 
	 * @param threads <i>[out]</i> given threads wrapped within synch. infrastructure<br/>
	 *                             (an empty {@link List} expected) 
	 */
	public <CODE_REFERENCE, REFERENCE> void instanciateScheme(
												VariableFactory<REFERENCE> variableFactory,
												List<LTSAHandler<CODE_REFERENCE, REFERENCE>> threadBodies,
												int atLeastExecuted,
												LTSAHandler<CODE_REFERENCE, REFERENCE> controllerLTSA,
												List<LTSAHandler<CODE_REFERENCE, REFERENCE>> threads);
	
}
