/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.provision.actuator.visitor;

import java.util.LinkedList;
import java.util.List;

import org.ow2.dsrg.fm.tbplib.ltsa.LTS;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSImpl;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSSpecification;
import org.ow2.dsrg.fm.tbplib.ltsa.LTSThread;
import org.ow2.dsrg.fm.tbplib.node.TBPImperativeNull;
import org.ow2.dsrg.fm.tbplib.node.TBPNode;
import org.ow2.dsrg.fm.tbplib.node.TBPThreadContainerNode;
import org.ow2.dsrg.fm.tbplib.node.TBPVariableDefinition;
import org.ow2.dsrg.fm.tbplib.provision.MalformedProvisionException;
import org.ow2.dsrg.fm.tbplib.provision.actuator.ActionSeqNode;
import org.ow2.dsrg.fm.tbplib.provision.actuator.BranchAction;
import org.ow2.dsrg.fm.tbplib.provision.actuator.EmitEventAction;
import org.ow2.dsrg.fm.tbplib.provision.actuator.EmitThreadsAction;
import org.ow2.dsrg.fm.tbplib.provision.actuator.NullAction;
import org.ow2.dsrg.fm.tbplib.provision.actuator.RepeatAction;

/**
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 * @deprecated Replaced by {@link TranslateToLTSAVisitor}
 */
public class TranslateToLTSVisitor<REFERENCE> implements
		ActionSeqNodeVisitor<LTSHandler<REFERENCE>, REFERENCE> {
	
	/** Representation of infinity */
	public static final int INFINITY = -1;
	private List<LTSThread<REFERENCE>> threads;
	private VariableFactory<REFERENCE> variableFactory;
	private boolean threadsAllowed; 
	
	/** Creates this visitor.
	 * 
	 * @param factory for creation of variables for thread synchronization 
	 */
	public TranslateToLTSVisitor(VariableFactory<REFERENCE> factory) {
		this.variableFactory = factory;
		this.threads = new LinkedList<LTSThread<REFERENCE>>();
		this.threadsAllowed = true;
	}
	
	/** Translates graph of {@link ActionSeqNode} into {@link LTSSpecification}.
	 * 
	 * @param node initial node of the graph
	 * @return resulting {@link LTSSpecification}
	 */
	public LTSSpecification<REFERENCE> translate(ActionSeqNode<REFERENCE> node) {
		LTSThread<REFERENCE> mainThread = createThread(node.visit(this), "main");
		LTSSpecification<REFERENCE> s = new LTSSpecification<REFERENCE>();
		
		// add threads
		s.addThread(mainThread);
		for(LTSThread<REFERENCE> auxThread: threads) {
			s.addThread(auxThread);
		}
		
		return s;
	}

	@Override
	public LTSHandler<REFERENCE> visitBranchAction(
			BranchAction<REFERENCE> node) {
		LTSHandler<REFERENCE> alternative = new LTSHandler<REFERENCE>();
		
		LTSHandler<REFERENCE> succ = node.getSuccessor() != null ? node.getSuccessor().visit(this) : null;
		LTSHandler<REFERENCE> leftBranch = node.getLeftBranch().visit(this);
		LTSHandler<REFERENCE> rightBranch = node.getRightBranch().visit(this);
		
		alternative.addEntryPoints(leftBranch.getEntryPoints());
		alternative.addEndPoints(leftBranch.getEndPoints());
		
		alternative.addEntryPoints(rightBranch.getEntryPoints());
		alternative.addEndPoints(rightBranch.getEndPoints());
		
		return concat(alternative, succ);
	}

	@Override
	public LTSHandler<REFERENCE> visitEmitEventAction(
			EmitEventAction<REFERENCE> node) {
		LTSHandler<REFERENCE> tmp = new LTSHandler<REFERENCE>();		
		LTSHandler<REFERENCE> succ = node.getSuccessor() != null ? node.getSuccessor().visit(this) : null;
		
		// create LTS transition
		LTS<? extends TBPNode<REFERENCE>, REFERENCE> transition = LTSImpl.createInstance(node.getEmit(), null);
		
		tmp.addEntryPoint(transition);
		tmp.addEndPoint(transition);
						
		return concat(tmp, succ);
	}

	@Override
	public LTSHandler<REFERENCE> visitEmitThreadsAction(
			EmitThreadsAction<REFERENCE> node) {
		if (!threadsAllowed) {
			throw new MalformedProvisionException("No parallel operator allowed within unlimited parallel section.");
		}
		LTSHandler<REFERENCE> tmp = new LTSHandler<REFERENCE>();	
		LTSHandler<REFERENCE> succ = node.getSuccessor() != null ? node.getSuccessor().visit(this) : null;
		
		// create temporary wrapper
		ThreadLauncherWrapper tWrapper = new ThreadLauncherWrapper();
		tWrapper.threadLauncher = node;
		tWrapper.rightOpen = succ == null;
				
		// create thread controller
		
		// collect thread bodies
		List<LTSHandler<REFERENCE>> threadBodies = new LinkedList<LTSHandler<REFERENCE>>();
		if (tWrapper.isUnlimited()) {
			boolean wereThreadsAllowed = threadsAllowed;
			threadsAllowed = false;
			for(ActionSeqNode<REFERENCE> t : node.getEntryPointsOfThreads()) {
				threadBodies.add(t.visit(this));
			}
			threadsAllowed = wereThreadsAllowed;
		} else {
			assert(tWrapper.threadLauncher.getMaxThreadReplicas() > 0);
			
			for(ActionSeqNode<REFERENCE> t : node.getEntryPointsOfThreads()) {
				for(int i = 0; i < tWrapper.threadLauncher.getMaxThreadReplicas(); ++i) {
					threadBodies.add(t.visit(this));					
				}
			}
		}
		
		List<LTSHandler<REFERENCE>> resultingThreadBodies = new LinkedList<LTSHandler<REFERENCE>>();
		ThreadCooperationScheme scheme = getScheme(tWrapper);
		scheme.instanciateScheme(variableFactory, threadBodies, tWrapper.threadLauncher.getMinThreadLaunches(), tmp, resultingThreadBodies);
		
		for(LTSHandler<REFERENCE> resultingThreadBody: resultingThreadBodies) {
			threads.add(createThread(resultingThreadBody, "worker"));
		}
		
		
		return concat(tmp, succ);
	}

	@Override
	public LTSHandler<REFERENCE> visitNullAction(NullAction<REFERENCE> node) {
		LTSHandler<REFERENCE> tmp = new LTSHandler<REFERENCE>();		
		LTSHandler<REFERENCE> succ = node.getSuccessor() != null ? node.getSuccessor().visit(this) : null;
		
		// create LTS transition
		LTS<? extends TBPNode<REFERENCE>, REFERENCE> transition = createNullTransition();
		
		tmp.addEntryPoint(transition);
		tmp.addEndPoint(transition);
						
		return concat(tmp, succ);
	}

	@Override
	public LTSHandler<REFERENCE> visitRepeatAction(
			RepeatAction<REFERENCE> node) {		
		LTSHandler<REFERENCE> tmp = new LTSHandler<REFERENCE>();
		LTSHandler<REFERENCE> repeated = node.getRepeatedSequenceEntryPoint().visit(this);		
		LTSHandler<REFERENCE> succ = node.getSuccessor() != null ? node.getSuccessor().visit(this) : null;
		
		// create LTS transition
		//LTS<? extends TBPNode<REFERENCE>, REFERENCE> repeatTransition = createNullTransition();
		
		assert(repeated != null && repeated.getEntryPoints().size() >= 1 && repeated.getEndPoints().size() >= 1);
		
		// end of backward edge
		LTS<? extends TBPNode<REFERENCE>, REFERENCE> repeatTransition =  createNullTransition();
		for(LTS<?, REFERENCE> e : repeated.getEntryPoints()) {
			repeatTransition.connectTransition(e);
		}
		
		// start of backward edge, end of repetition
		LTS<? extends TBPNode<REFERENCE>, REFERENCE> repetitionEnd = createNullTransition();		
		for(LTS<?, REFERENCE> e : repeated.getEndPoints()) {
			e.connectTransition(repetitionEnd);
			e.connectTransition(repeatTransition);
		}
				
		tmp.addEntryPoints(repeated.getEntryPoints());
		tmp.addEndPoint(repetitionEnd);
						
		return concat(tmp, succ);
	}
	
	
	//
	// Auxiliary types
	//
	private class ThreadLauncherWrapper {
		public EmitThreadsAction<REFERENCE> threadLauncher;
		public boolean rightOpen = false;
		
		public boolean isUnlimited() {
			return threadLauncher.getMaxThreadReplicas() == EmitThreadsAction.INFINITY;
		}
	}
	
	//
	// Auxiliary methods
	//
	
	private LTS<?, REFERENCE> createNullTransition() {
		return LTSImpl.createInstance(new TBPImperativeNull<REFERENCE>(), null);
	}
	
	private LTSThread<REFERENCE> createThread(LTSHandler<REFERENCE> body, String name) {
		
		Object obj = LTSImpl.createInstance(
				new TBPThreadContainerNode<REFERENCE>(
						variableFactory.createThreadName(name), /* name */
						new TBPImperativeNull<REFERENCE>(), /* code */
						new LinkedList<TBPVariableDefinition<REFERENCE>>()), /* local variables */
				null /* guard */);
		
		LTSThread<REFERENCE> thr = null;
		if (obj instanceof LTSThread<?>) {
			thr = (LTSThread<REFERENCE>)obj;
		}
		
		if (thr != null) {
			LTSAManipulationHelper.connectAllLTSTransitions(thr, body.getEntryPoints());
		} else {
			throw new RuntimeException("Unable to create a thread from LTS graph.");
		}
		
		return thr;
	}
	
	private ThreadCooperationScheme getScheme(ThreadLauncherWrapper tWrapper) {
		if (!tWrapper.isUnlimited()) {
			// limited scheme
			// currently we have just closed limited scheme,
			// which is however sufficient for both; closed and open limited cases
			// (but it is not really optimal for closed case)
			return new ClosedLimitedScheme();
			
		} else if (tWrapper.rightOpen){
			// unlimited scheme
			// we have just a scheme for (at least right)closed unlimited case
			// unfortunately, the open unlimited scheme seems to be not realizable
			return new OpenUnlimitedScheme();
			
		} else {
			throw new RuntimeException("Unable to create thread-cooperation meachanism for unlimited parallel followed by non-empty suffix.");
		}
	}	
	
	private LTSHandler<REFERENCE> concat(LTSHandler<REFERENCE> head, LTSHandler<REFERENCE> tail) {
		assert(head != tail);
		
		if (head == null) {
			return tail;
		}
		if (tail == null) {
			return head;
		}
		
		assert(head != null && head.getEntryPoints().size() >= 1 && head.getEndPoints().size() >= 1);
		assert(tail != null && tail.getEntryPoints().size() >= 1 && tail.getEndPoints().size() >= 1);
		
		LTSHandler<REFERENCE> result = new LTSHandler<REFERENCE>();
		
		// entry
		result.addEntryPoints(head.getEntryPoints());
		
		// connect
		LTSAManipulationHelper.connectAllLTSTransitions(head.getEndPoints(), tail.getEntryPoints());
		
		// end
		result.addEndPoints(tail.getEndPoints());
		
		return result;
	}
}