/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.provision.actuator.visitor;

import java.util.LinkedList;
import java.util.List;

/**
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 *
 */
public class VariableDescriptor<REFERENCE> {
	private List<REFERENCE> values;
	private REFERENCE identifier;
	
	/** Creates empty (unnamed) variable descriptor.
	  */
	public VariableDescriptor() {
		this(null);
	}
	
	/** Creates empty variable descriptor with given identifier <b>idf</b>.
	 * 
	 * @param idf variable identifier
	 */
	public VariableDescriptor(REFERENCE idf) {
		this.identifier = idf;
		this.values = new LinkedList<REFERENCE>();
	}
	
	/** Adds variable value (extends domain with given <i>value</i>).
	 * 
	 * @param value reference to the value
	 */
	public void addValue(REFERENCE value) {
		values.add(value);
	}
	/** Returns variable values.
	 * 
	 * @return {@link List} of variable values (domain).
	 */
	public List<REFERENCE> getValues() {
		return values;
	}
	
	/** Sets variable identifier.
	 * 
	 * @param idf identifier
	 */
	public void setIdentifier(REFERENCE idf) {
		this.identifier = idf;
	}
	
	/** Gets variable identifier.
	 * 
	 * @return variable identifier
	 */
	public REFERENCE getIdentifier() {
		return this.identifier;
	}
}	
