/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.reference;

/**
 * Reference to a constant, i.e., an item of an enumeration type.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class Constant implements Reference {

    private byte code;
    private String name;
    private EnumerationType type;

    public Constant(EnumerationType type, String name, byte code) {
        this.name = name;
        this.type = type;
        this.code = code;
    }

    @Override
    public String getName() {
        return name;
    }

    public EnumerationType getType() {
        return type;
    }

    public byte getCode() {
        return code;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Constant) {
            Constant o = (Constant)obj;
            return o.name.equals(name) &&
                    o.type.getName().equals(type.getName());
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        return name.hashCode();
    }

    @Override
    public String toString() {
        return name;
    }
}
