/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.reference;

import java.util.List;
import org.ow2.dsrg.fm.tbplib.architecture.Method;

/**
 * Helper class holding call to a reaction.
 * 
 * @author caitt3am
 *
 */
public class MethodCall<REFERENCE> implements Cloneable{
	private REFERENCE ifaceName;
	private REFERENCE methodName;
	private String fullname;
	private List<REFERENCE> paramDecl;
	
	public MethodCall(REFERENCE ifaceName, REFERENCE methodName,
			List<REFERENCE> paramDecl) {

		this.ifaceName = ifaceName;
		this.methodName = methodName;
		this.paramDecl = paramDecl;
		recompute_fullname();
	}

	public List<REFERENCE> getParamDecl() {
		return paramDecl;
	}
	
	public REFERENCE getInterface() {
		return ifaceName;
	}


	public void setInterface(REFERENCE ifaceName) {
		this.ifaceName = ifaceName;
		recompute_fullname();
	}
	
	public REFERENCE getMethod() {
		return methodName;
	}

	public void setMethod(REFERENCE methodName) {
		this.methodName = methodName;
		recompute_fullname();
	}
	
	public String getFullname() {
		return fullname;
	}

	@Override
	public MethodCall clone() {
		try {
			return (MethodCall) super.clone();
		} catch (CloneNotSupportedException e) {
			throw new RuntimeException(e);
		}
	}
	
	private void recompute_fullname() {
            if (methodName instanceof Method) {
                fullname = ((Method)methodName).getFullname();
            }else {
		fullname = ifaceName.toString() + "." + methodName.toString();
            }
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append(getFullname());
		sb.append('(');
		boolean first = true;
		for(REFERENCE par : getParamDecl()){
			if(!first)
				sb.append(", ");
			sb.append(par);
			first = false;
		}
		sb.append(')');
		return sb.toString();
	}
}
