/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.reference;

import java.util.*;

/**
 * Describes signature of a method.
 * 
 * @author caitt3am
 *
 */

public class MethodSignature<REFERENCE> {

	private final LinkedHashMap<REFERENCE, REFERENCE> params = new LinkedHashMap<REFERENCE, REFERENCE>();
	private final REFERENCE return_type;
	
	
	public MethodSignature(Map<? extends REFERENCE, ? extends REFERENCE> params,
            REFERENCE return_type) {
		this.return_type = return_type;
		this.params.putAll(params);
	}

	public LinkedHashMap<REFERENCE, REFERENCE> getParams() {
		return params;
	}
	
	/**
	 * Return type of method; <code>null</code> if type is void.
	 */
	public REFERENCE getReturnType() {
		return return_type;
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append('(');
		boolean first = true;
		for(Map.Entry<REFERENCE, REFERENCE> entry : getParams().entrySet()){
			if(!first)
				sb.append(", ");
			sb.append(entry.getValue());
			sb.append(" ");
			sb.append(entry.getKey());
			first = false;
		}
		sb.append(')');
		if(return_type != null){
			sb.append(" : ");
			sb.append(return_type);
		}
		return sb.toString();
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((params == null) ? 0 : params.hashCode());
		result = prime * result
				+ ((return_type == null) ? 0 : return_type.hashCode());
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof MethodSignature))
			return false;
		final MethodSignature other = (MethodSignature) obj;
		if (params == null) {
			if (other.params != null)
				return false;
		} else if (!params.equals(other.params))
			return false;
		if (return_type == null) {
			if (other.return_type != null)
				return false;
		} else if (!return_type.equals(other.return_type))
			return false;
		return true;
	}

}

