/*
 *  $Id$
 *
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 *
 */

package org.ow2.dsrg.fm.tbplib.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

/**
 * An indenting extension of PrintWriter.
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class CodeWriter extends PrintWriter {

    private int currentIndent = 0;
    private int indentStep = 4;
    private char indentChar = ' ';
    private String indentString = "";
    private boolean needsIndent = true;

    public CodeWriter(File file, String csn) throws FileNotFoundException, UnsupportedEncodingException {
        super(file, csn);
    }

    public CodeWriter(File file) throws FileNotFoundException {
        super(file);
    }

    public CodeWriter(String fileName, String csn) throws FileNotFoundException, UnsupportedEncodingException {
        super(fileName, csn);
    }

    public CodeWriter(String fileName) throws FileNotFoundException {
        super(fileName);
    }

    public CodeWriter(OutputStream out, boolean autoFlush) {
        super(out, autoFlush);
    }

    public CodeWriter(OutputStream out) {
        super(out);
    }

    public CodeWriter(Writer out, boolean autoFlush) {
        super(out, autoFlush);
    }

    public CodeWriter(Writer out) {
        super(out);
    }

    public CodeWriter(File file, String csn, int indentStep, char indentChar) throws FileNotFoundException, UnsupportedEncodingException {
        super(file, csn);
        this.indentStep = indentStep;
        this.indentChar = indentChar;
    }

    public CodeWriter(File file, int indentStep, char indentChar) throws FileNotFoundException {
        super(file);
        this.indentStep = indentStep;
        this.indentChar = indentChar;
    }

    public CodeWriter(String fileName, String csn, int indentStep, char indentChar) throws FileNotFoundException, UnsupportedEncodingException {
        super(fileName, csn);
        this.indentStep = indentStep;
        this.indentChar = indentChar;
    }

    public CodeWriter(String fileName, int indentStep, char indentChar) throws FileNotFoundException {
        super(fileName);
        this.indentStep = indentStep;
        this.indentChar = indentChar;
    }

    public CodeWriter(OutputStream out, boolean autoFlush, int indentStep, char indentChar) {
        super(out, autoFlush);
        this.indentStep = indentStep;
        this.indentChar = indentChar;
    }

    public CodeWriter(OutputStream out, int indentStep, char indentChar) {
        super(out);
        this.indentStep = indentStep;
        this.indentChar = indentChar;
    }

    public CodeWriter(Writer out, boolean autoFlush, int indentStep, char indentChar) {
        super(out, autoFlush);
        this.indentStep = indentStep;
        this.indentChar = indentChar;
    }

    public CodeWriter(Writer out, int indentStep, char indentChar) {
        super(out);
        this.indentStep = indentStep;
        this.indentChar = indentChar;
    }

    public void increaseIndent() {
        currentIndent++;
        updateIndentString();
    }

    public void decreaseIndent() {
        assert(currentIndent > 0);
        currentIndent--;
        updateIndentString();
    }

    private void updateIndentString() {
        StringBuilder bfr = new StringBuilder(indentStep * currentIndent);

        for (int i=0; i<indentStep * currentIndent; i++) {
            bfr.append(indentChar);
        }

        indentString = bfr.toString();
    }

    @Override
    public void println() {
        super.println();
        needsIndent = true;
    }

    @Override
    public void write(int c) {
        if (needsIndent && currentIndent != 0) {
            super.write(indentString, 0, indentString.length());
            needsIndent = false;
        }
        super.write(c);
    }

    @Override
    public void write(char[] buf, int off, int len) {
        if (needsIndent && currentIndent != 0) {
            super.write(indentString, 0, indentString.length());
            needsIndent = false;
        }
        super.write(buf, off, len);
    }

    @Override
    public void write(char[] buf) {
        if (needsIndent && currentIndent != 0) {
            super.write(indentString, 0, indentString.length());
            needsIndent = false;
        }
        super.write(buf);
    }

    @Override
    public void write(String s, int off, int len) {
        if (needsIndent && currentIndent != 0) {
            super.write(indentString, 0, indentString.length());
            needsIndent = false;
        }
        super.write(s, off, len);
    }

    @Override
    public void write(String s) {
        if (needsIndent && currentIndent != 0) {
            super.write(indentString, 0, indentString.length());
            needsIndent = false;
        }
        super.write(s);
    }
}
