/*
 *  $Id$
 * 
 *   Threaded Behavior Protocols  - Parsers, Transformations
 *   Copyright (C) 2009       DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */

package org.ow2.dsrg.fm.tbplib.util;

import java.io.FileNotFoundException;
import java.io.OutputStream;

/** Extension of {@link DOTPrintStream} that allows creation of subgraphs.
 * 
 * @author Ondrej Cerny &lt;ondrej.cerny@email.cz&gt;
 */
public class DOTPrintStreamWithSubgraphs extends DOTPrintStream {
	
	public enum NodeShape {
		DEFAULT, POINT, BOX
	}
	
	private NodeShape currentNodeShape = NodeShape.DEFAULT;
	private int clusterNo = 0;
	private int openSubgraphs = 0;
		
	public DOTPrintStreamWithSubgraphs(OutputStream out) {
		super(out);
	}
	public DOTPrintStreamWithSubgraphs(String fileName) throws FileNotFoundException {
		super(fileName);
	}
	
	public void openSubGraph(String label) {
		this.printf("subgraph cluster_%d {\n  label = \"%s\";\n", clusterNo, label);
		clusterNo++;
		openSubgraphs++;
	}
	
	public void closeSubGraph() {
		this.printf("}\n");
		openSubgraphs--;
	}
	
	public void changeNodeShape(NodeShape s) {
		this.currentNodeShape = s;
		this.shape = nodeShapeToString(this.currentNodeShape);
	}
	
	public NodeShape getCurrentNodeShape() {
		return this.currentNodeShape;
	}
	
	@Override
	public void DOTclose() {
		assert(openSubgraphs == 0);
		super.DOTclose();
	}
		
	private String nodeShapeToString(NodeShape s) {
		switch (s) {
		case POINT: return "point";
		case BOX: return "box";
		default: return null;
		}
	}
}
