/*
 *  $Id$
 * 
 *   Badger - Threaded Behavior Protocols analysis toolchain
 *   Copyright (C) 2008-2010  DSRG, Charles University in Prague
 *                            http://dsrg.mff.cuni.cz/
 * 
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 * 
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *   MA 02110-1301 USA
 * 
 */
package org.ow2.dsrg.fm.tbplib.event;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Ondrej Sery <ondrej.sery@dsrg.mff.cuni.cz>
 */
public class EventTableImplTest {

    static EventTable.MethodEncoder encoder1;
    static EventTable.MethodEncoder encoder2;
    static EventTableImpl instance;

    public EventTableImplTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
        encoder1 =
                new EventTableImpl.MethodEncoderImpl("Logger.log", 5,
                        3, 4, 5);
        encoder2 =
                new EventTableImpl.MethodEncoderImpl("Clock.tick", 0);

        instance = new EventTableImpl();

        assertEquals(0, instance.addMethodEncoder(encoder1));
        assertEquals(1, instance.addMethodEncoder(encoder2));
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    /**
     * Test of addMethodEncoder method, of class EventTableImpl.
     */
    @Test(expected = IllegalArgumentException.class)
    public void testDuplicateAddition() {
        instance.addMethodEncoder(encoder1);
    }

    /**
     * Test encoding
     */
    @Test
    public void testEncoding() {
        String m1 = "Logger.log";
        String m2 = "Clock.tick";
        assertTrue(instance.containsMethod(m1));
        assertFalse(instance.containsMethod("xxx"));

        long eCall1 = instance.encodeMethodCall(
                instance.encodeMethod(m1), (byte)2, (byte)3, (byte)1);
        long eCall2 = instance.encodeMethodCall(
                instance.encodeMethod(m2));
        long eRet1 = instance.encodeMethodReturn(
                instance.encodeMethod(m1), (byte)3);
        long eRet2 = instance.encodeMethodReturn(
                instance.encodeMethod(m2), EventCode.VOID_VALUE);

        System.out.println("Call (" + Long.toHexString(eCall1) + "): \"" + instance.decodeEventName(eCall1) + "\"");
        System.out.println("Return (" + Long.toHexString(eRet1) + "): \"" + instance.decodeEventName(eRet1) + "\"");
        System.out.println("Call (" + Long.toHexString(eCall2) + "): \"" + instance.decodeEventName(eCall2) + "\"");
        System.out.println("Return (" + Long.toHexString(eRet2) + "): \"" + instance.decodeEventName(eRet2) + "\"");
        System.out.println("Tau (" + Long.toHexString(EventCode.TAU_ACTION) + "): \"" + instance.decodeEventName(EventCode.TAU_ACTION) + "\"");
        assertEquals(instance.decodeMethodName(EventCode.decodeMethod(eCall1)), m1);
        assertEquals(instance.decodeMethodName(EventCode.decodeMethod(eRet1)), m1);
        assertEquals(3, EventCode.decodeReturnValue(eRet1));
        assertEquals(instance.decodeMethodName(EventCode.decodeMethod(eCall2)), m2);
        assertEquals(instance.decodeMethodName(EventCode.decodeMethod(eRet2)), m2);
        assertEquals(EventCode.VOID_VALUE, EventCode.decodeReturnValue(eRet2));
        assertEquals(2 + 3 * (3 + 4 * 1), EventCode.decodeParameters(eCall1));
        assertEquals(EventCode.EventType.METHOD_CALL, EventCode.decodeEventType(eCall1));
        assertEquals(EventCode.EventType.METHOD_RETURN, EventCode.decodeEventType(eRet1));
        assertEquals(EventCode.VOID_VALUE, EventCode.decodeParameters(eCall2));
        assertEquals(EventCode.EventType.METHOD_CALL, EventCode.decodeEventType(eCall2));
        assertEquals(EventCode.EventType.METHOD_RETURN, EventCode.decodeEventType(eRet2));
        assertEquals(EventCode.EventType.SPECIAL, EventCode.decodeEventType(EventCode.TAU_ACTION));
        assertEquals(EventCode.EventType.SPECIAL, EventCode.decodeEventType(EventCode.BAD_ACTIVITY));
        assertEquals(EventCode.EventType.SPECIAL, EventCode.decodeEventType(EventCode.NO_ACTIVITY));
        assertEquals(EventCode.EventType.SPECIAL, EventCode.decodeEventType(EventCode.IGNORE_BRANCH));
    }
}