/*
 *
 * Threaded Behavior Protocols  - Parsers, Transformations
 * Copyright (C) 2008   DSRG, Charles University in Prague
 *                      http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.fm.tbplib.parser;

import org.junit.Test;
import org.ow2.dsrg.fm.tbplib.architecture.Architecture;
import org.ow2.dsrg.fm.tbplib.architecture.Component;
import org.ow2.dsrg.fm.tbplib.node.ParserResult;
import org.ow2.dsrg.fm.tbplib.node.TBPComposition;
import org.ow2.dsrg.fm.tbplib.node.TBPSpecification;
import org.ow2.dsrg.fm.tbplib.reference.Reference;
import org.ow2.dsrg.fm.tbplib.util.CodeWriter;
import org.ow2.dsrg.fm.tbplib.util.TransformationChain;

import java.util.ArrayList;
import java.util.List;

/**
 * @author caitt3am
 *
 */
public class CompositionTest {

    @Test
    public void testComposition() throws Exception {
        Architecture arch = new Architecture("testing-arch");
        List<TBPSpecification<String>> tbps = new ArrayList<TBPSpecification<String>>();

        ParserResult pr1 = ParserFacade.parseFile("test/input/old/1.tbp");
        TBPSpecification<String> tbp1 =pr1.getSpecification();
        arch.addComponent(new Component(tbp1.getComponent(),pr1.getTypes()));

        ParserResult pr2 = ParserFacade.parseFile("test/input/old/2.tbp");
        TBPSpecification<String> tbp2 =pr2.getSpecification();
        arch.addComponent(new Component(tbp2.getComponent(),pr2.getTypes()));

        tbps.add(tbp1);
        tbps.add(tbp2);

        TransformationChain.collectSymbolsFromArchitecture(tbps, arch);

        arch.getComponents().get(0).getRequiredInterfaces().get("InformationSystem").bindTo(
                arch.getComponents().get(1).getProvidedInterfaces().get("InformationSystem"));
        arch.getComponents().get(0).getRequiredInterfaces().get("Payment").bindTo(
                arch.getComponents().get(1).getProvidedInterfaces().get("Payment"));
        arch.getComponents().get(0).getRequiredInterfaces().get("X").bindTo(
                arch.getComponents().get(1).getProvidedInterfaces().get("X"));
        arch.getComponents().get(0).getRequiredInterfaces().get("Display").bindTo(
                arch.getComponents().get(1).getProvidedInterfaces().get("Display"));

        TransformationChain.performImplicitBinding(arch);
        
        // Resolve symbols
        TBPSpecification<Reference> rtbp1 = TransformationChain.resolveTBP(tbp1, arch.getComponents().get(0));
        TBPSpecification<Reference> rtbp2 = TransformationChain.resolveTBP(tbp2, arch.getComponents().get(1));

        // Compose specifications
        TBPComposition<Reference> tbpComp = new TBPComposition<Reference>();
        tbpComp.addTBP(rtbp1);
        tbpComp.addTBP(rtbp2);
        TBPSpecification<Reference> rtbp3 = tbpComp.getComposition(arch);

        // Print protocols
        TransformationChain.printTBP(new CodeWriter(System.out, true), rtbp1,
                arch.getComponents().get(0));
        TransformationChain.printTBP(new CodeWriter(System.out, true), rtbp2,
                arch.getComponents().get(1));
    }
}
